package be.ac.ua.broker.allocation;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import be.ac.ua.broker.allocation.reserved.TippingPointCalculator;
import be.ac.ua.broker.allocation.spot.SpotAllocator;
import be.ac.ua.broker.input.pricing.EC2Constants;
import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.input.pricing.OnDemandPrice;
import be.ac.ua.broker.input.pricing.PricingManager;
import be.ac.ua.broker.input.pricing.ReservedPrice;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerResult;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;
import be.ac.ua.broker.utils.InputVerifier;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

/**
 * This class is used during the resource allocation phase of the broker. For
 * every instance on which certain task hours were scheduled in the previous
 * phase of the broker, is determined what pricing model will be used and the
 * actual prices are coupled to the schedule. There are different options
 * available, only on-demand instances, no spot instances, spot-enabled (which
 * uses the spot model to determine the bid) and spot-optimal (which used the
 * actual spot price history to determine the bid).
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class Allocator {

	/** Whether the first workload model is being used */
	private boolean useWorkloadModel1 = false;

	/**
	 * Constructor for Allocator
	 * 
	 * @param uSE_TASK_HOURS
	 *            whether the first workload model is being used
	 */
	public Allocator(boolean uSE_TASK_HOURS) {
		useWorkloadModel1 = uSE_TASK_HOURS;
	}

	/**
	 * Assign pricing models to the resources of the schedule in a way that the
	 * total cost price is minimized.
	 * 
	 * @param version
	 *            the allocation scheme option to use
	 * @param results
	 *            the original schedule
	 * @return the updated schedule marked with actual prices
	 * @throws InstanceNotAvailableException
	 * @throws InvalidInputException
	 */
	public SchedulerResultCollection allocate(String version,
			SchedulerResultCollection results)
			throws InstanceNotAvailableException, InvalidInputException {
		InputVerifier.allocationVersionExists(version);
		SchedulerResultCollection returnResults = new SchedulerResultCollection(
				results.getStartDate(), results.getEndDate());
		Set<String> regions = EC2Specifications.getRegionMapping().keySet();
		for (String reg : regions) {
			List<SchedulerResult> regResults = results.get(reg);
			if (regResults != null) {
				for (SchedulerResult schedulerResult : regResults) {
					returnResults.add(handleInstanceResourceAllocations(
							version, schedulerResult, results.getStartDate(),
							results.getEndDate()));
				}
			}
		}
		return returnResults;
	}

	/**
	 * For every single schedulerResult that is part of the given
	 * collection/total schedule, is determined what the appropriate pricing
	 * model is in order to minimize the total cost.
	 * 
	 * @param version
	 *            the allocation scheme option that is begin used
	 * @param schedulerResult
	 *            the original schedule on a single instance
	 * @param start
	 *            the start time of the schedule
	 * @param end
	 *            the end time of the schedule
	 * @return the updated schedule
	 * @throws InstanceNotAvailableException
	 * @throws InvalidInputException
	 */
	private SchedulerResult handleInstanceResourceAllocations(String version,
			SchedulerResult schedulerResult, Date start, Date end)
			throws InstanceNotAvailableException, InvalidInputException {
		SpotAllocator spotAllocator = new SpotAllocator(schedulerResult,
				useWorkloadModel1, start, end);
		ReservedPrice reserved = PricingManager
				.getReservedPrice(schedulerResult.getInstanceSpecification());
		OnDemandPrice onDemand = PricingManager
				.getOnDemandPrice(schedulerResult.getInstanceSpecification());
		SchedulerResult returnResult = new SchedulerResult(
				schedulerResult.getInstanceSpecification());
		long tippingPointResOnD = TippingPointCalculator
				.determineThresholdReservedOnDemand(schedulerResult
						.getInstanceSpecification());
		List<ScheduledTaskCollection> resources = schedulerResult
				.getScheduledTaskCollections();
		for (ScheduledTaskCollection scheduledTaskCollection : resources) {
			scheduledTaskCollection.fillInTimeStampsSubTasks(start);
			if (version.equals(AllocatorOptions.ALLONDEMAND)) {
				returnResult
						.addScheduledTaskCollection(handleOnDemandAllocation(
								onDemand, scheduledTaskCollection));
			} else if (version.equals(AllocatorOptions.NOSPOT)) {
				returnResult
						.addScheduledTaskCollection(handleReservedAllocation(
								reserved, onDemand, tippingPointResOnD,
								scheduledTaskCollection));
			} else {
				List<ScheduledTaskCollection> l = handleSpotAllocation(version,
						schedulerResult, start, end, spotAllocator, reserved,
						onDemand, tippingPointResOnD, scheduledTaskCollection);
				for (ScheduledTaskCollection scheduledTaskCollection2 : l) {
					returnResult
							.addScheduledTaskCollection(scheduledTaskCollection2);
				}
			}
		}
		return returnResult;
	}

	/**
	 * Handle the allocation phase when only on-demand instances are being used
	 * 
	 * @param onDemand
	 *            the on-demand price that corresponds to the instance
	 *            specification of the schedule
	 * @param scheduledTaskCollection
	 *            the original schedule
	 * @return the schedule updated with price tags
	 */
	private ScheduledTaskCollection handleOnDemandAllocation(
			OnDemandPrice onDemand,
			ScheduledTaskCollection scheduledTaskCollection) {
		scheduledTaskCollection.setPricingModel(EC2Constants.ONDEMAND);
		scheduledTaskCollection.setPricesSubTasks(onDemand);
		return scheduledTaskCollection;
	}

	/**
	 * Handle the allocation phase when only on-demand and reserved instances
	 * (no spot option) are being used
	 * 
	 * @param reserved
	 *            the reserved price that corresponds to the instance
	 *            specification of the schedule
	 * @param onDemand
	 *            the on-demand price that corresponds to the instance
	 *            specification of the schedule
	 * @param tippingPointResOnD
	 *            the tipping point utilization rate for reserved being the
	 *            preferred pricing model choice in order to minimize the total
	 *            cost (compared to the on-demand pricing model)
	 * @param scheduledTaskCollection
	 *            the original schedule
	 * @return the schedule updated with price tags
	 */
	private ScheduledTaskCollection handleReservedAllocation(
			ReservedPrice reserved, OnDemandPrice onDemand,
			long tippingPointResOnD,
			ScheduledTaskCollection scheduledTaskCollection) {
		int taskHours = scheduledTaskCollection.getNumberOfTaskHours();
		if (taskHours >= tippingPointResOnD) {
			scheduledTaskCollection.setPricingModel(EC2Constants.RESERVED);
			int nrTaskHours = scheduledTaskCollection.getNumberOfTaskHours();
			ReservedPrice reservedPrice = new ReservedPrice(
					reserved.getHourlyPrice(),
					(reserved.getFixedPrice() / (double) nrTaskHours));
			scheduledTaskCollection.setPricesSubTasks(reservedPrice);
		} else {
			scheduledTaskCollection.setPricingModel(EC2Constants.ONDEMAND);
			scheduledTaskCollection.setPricesSubTasks(onDemand);
		}
		return scheduledTaskCollection;
	}

	/**
	 * Handle the allocation phase when all three pricing models may be
	 * considered
	 * 
	 * @param version
	 *            the allocation scheme option: spot-enabled or spot-optimal in
	 *            this case, the spot-enabled option uses the spot model to
	 *            determine the appropriate bid
	 * @param schedulerResult
	 *            the original full schedule
	 * @param start
	 *            the start time of the schedule
	 * @param end
	 *            the end time of the schedule
	 * @param spotAllocator
	 *            an allocation scheme implementation for spot instances, it has
	 *            the ability to reschedule task parts in order to achieve a
	 *            lower total cost. It uses a spot bid in order to take
	 *            decisions.
	 * @param reserved
	 *            the reserved price that corresponds to the instance
	 *            specification of the schedule
	 * @param onDemand
	 *            the on-demand price that corresponds to the instance
	 *            specification of the schedule
	 * @param tippingPointResOnD
	 *            the tipping point utilization rate for reserved being the
	 *            preferred pricing model choice in order to minimize the total
	 *            cost (compared to the on-demand pricing model)
	 * @param scheduledTaskCollection
	 *            the original schedule
	 * @return the schedule updated with price tags
	 * @throws InstanceNotAvailableException
	 * @throws InvalidInputException
	 */
	private List<ScheduledTaskCollection> handleSpotAllocation(String version,
			SchedulerResult schedulerResult, Date start, Date end,
			SpotAllocator spotAllocator, ReservedPrice reserved,
			OnDemandPrice onDemand, long tippingPointResOnD,
			ScheduledTaskCollection scheduledTaskCollection)
			throws InstanceNotAvailableException, InvalidInputException {
		List<ScheduledTaskCollection> returnList = new ArrayList<ScheduledTaskCollection>();
		if (scheduledTaskCollection.onlySpotEnabledSubTasks()) {
			ScheduledTaskCollection spotEnabledColl = spotAllocator
					.determinePrices(version, scheduledTaskCollection);
			spotEnabledColl.setPricingModel(EC2Constants.SPOT);
			returnList.add(spotEnabledColl);
		} else if (scheduledTaskCollection.onlyNonSpotEnabledTasks()) {
			int taskHours = scheduledTaskCollection.getNumberOfTaskHours();
			if (taskHours >= tippingPointResOnD) {
				scheduledTaskCollection.setPricingModel(EC2Constants.RESERVED);
				int nrTaskHours = scheduledTaskCollection
						.getNumberOfTaskHours();
				ReservedPrice reservedPrice = new ReservedPrice(
						reserved.getHourlyPrice(),
						(reserved.getFixedPrice() / (double) nrTaskHours));
				scheduledTaskCollection.setPricesSubTasks(reservedPrice);
			} else {
				scheduledTaskCollection.setPricingModel(EC2Constants.ONDEMAND);
				scheduledTaskCollection.setPricesSubTasks(onDemand);
			}
			returnList.add(scheduledTaskCollection);
		} else {
			ScheduledTaskCollection spotEn = scheduledTaskCollection
					.getSpotEnabledSubTasks();
			ScheduledTaskCollection nonSpotEn = scheduledTaskCollection
					.getNonSpotEnabledSubTasks();
			int nrSpotEnabledTasks = spotEn.getNumberOfTaskHours();
			int nrNonSpotEnabledTasks = nonSpotEn.getNumberOfTaskHours();
			if (TippingPointCalculator.determineReservedvsSpot(
					schedulerResult.getInstanceSpecification(),
					nrSpotEnabledTasks, nrNonSpotEnabledTasks, start, end)) {
				scheduledTaskCollection.setPricingModel(EC2Constants.RESERVED);
				int nrTaskHours = scheduledTaskCollection
						.getNumberOfTaskHours();
				ReservedPrice reservedPrice = new ReservedPrice(
						reserved.getHourlyPrice(),
						(reserved.getFixedPrice() / (double) nrTaskHours));
				scheduledTaskCollection.setPricesSubTasks(reservedPrice);
				returnList.add(scheduledTaskCollection);
			} else {
				ScheduledTaskCollection spotEnabledColl = spotAllocator
						.determinePrices(version, spotEn);
				spotEnabledColl.setPricingModel(EC2Constants.SPOT);
				nonSpotEn.setPricingModel(EC2Constants.ONDEMAND);
				nonSpotEn.setPricesSubTasks(onDemand);
				returnList.add(spotEnabledColl);
				returnList.add(nonSpotEn);
			}
		}
		return returnList;
	}

}
