package be.ac.ua.broker.allocation.reserved;

import java.util.Date;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.pricing.PricingManager;
import be.ac.ua.broker.input.pricing.ReservedPrice;
import be.ac.ua.broker.utils.InstanceNotAvailableException;

/**
 * Calculator used during the resource allocation phase of the broker, to
 * determine the tipping point utilization ratio for when reserved is cheaper
 * than using an on-demand instance. It is also used to determine the tipping
 * point utilization ratio for when reserved is cheaper than using a spot
 * instance (uses the average spot price of the schedule period).
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class TippingPointCalculator {

	/**
	 * Determine the tipping point utilization ratio for when a reserved
	 * instance becomes cheaper than using an on-demand instance.
	 * 
	 * @param inst
	 *            the instance specification to determine the threshold value
	 *            for
	 * @return the tipping for reserved versus on-demand
	 */
	public static long determineThresholdReservedOnDemand(
			InstanceSpecification inst) {
		long threshold = 0;
		try {
			ReservedPrice reservedPriceObj = PricingManager
					.getReservedPrice(inst);
			double reservedPrice = reservedPriceObj.getHourlyPrice();
			double reservedFixed = reservedPriceObj.getFixedPrice();
			double ondemandPrice = PricingManager.getOnDemandPrice(inst)
					.getHourlyPrice();
			threshold = Math
					.round(Math
							.ceil((reservedFixed / (double) (ondemandPrice - reservedPrice))));
		} catch (InstanceNotAvailableException e) {
			e.printStackTrace();
		}
		return threshold;
	}

	/**
	 * Determine the tipping point utilization ratio for when a reserved
	 * instance becomes cheaper than using a spot instance. Note that in this
	 * calculation the average spot price during the whole scheduling period is
	 * being used. The number of task hours that are scheduled on the instance
	 * that originate from a spot-enabled task and the number of hours from a
	 * not spot-enabled task are taken into account as well.
	 * 
	 * @param inst
	 *            the instance specification to determine the threshold value
	 *            for
	 * @param nrSpotEnabledSubtasks
	 *            number of task hours scheduled on the instance originating
	 *            from spot-enabled tasks
	 * @param nrNonSpotEnabledSubtasks
	 *            number of task hours scheduled on the instance originating
	 *            from not spot-enabled tasks
	 * @param start
	 *            start time of the schedule
	 * @param end
	 *            end time of the schedule
	 * @return the tipping for reserved versus spot
	 */
	public static boolean determineReservedvsSpot(InstanceSpecification inst,
			int nrSpotEnabledSubtasks, int nrNonSpotEnabledSubtasks,
			Date start, Date end) {
		boolean useReserved = false;
		try {
			ReservedPrice reservedPriceObj = PricingManager
					.getReservedPrice(inst);
			double reservedPrice = reservedPriceObj.getHourlyPrice();
			double reservedFixed = reservedPriceObj.getFixedPrice();
			double ondemandPrice = PricingManager.getOnDemandPrice(inst)
					.getHourlyPrice();
			double leftEq = ((nrSpotEnabledSubtasks + nrNonSpotEnabledSubtasks) * reservedPrice)
					+ reservedFixed;
			DescriptiveStatistics statistics = PricingManager
					.getStatisticsSpotPrice(inst, start, end);
			double rightEq = (nrNonSpotEnabledSubtasks * ondemandPrice)
					+ (nrSpotEnabledSubtasks * statistics.getMean());
			useReserved = (leftEq <= rightEq);
		} catch (InstanceNotAvailableException e) {
			e.printStackTrace();
		}
		return useReserved;
	}

}
