package be.ac.ua.broker.allocation.spot;

import java.util.Date;

import be.ac.ua.broker.allocation.AllocatorOptions;
import be.ac.ua.broker.allocation.spot.model.SpotModel;
import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.pricing.PricingManager;
import be.ac.ua.broker.input.pricing.SpotPrice;
import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerResult;
import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.InputVerifier;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

/**
 * This class is used during the resource allocation phase of the broker, when
 * an instance is tagged as a spot instance to determine the appropriate
 * scheduling for the task parts that were assigned to the instance. The bid
 * price is determined using the spot model implementation, or using the actual
 * spot prices when this option is enabled. The spot prices are also set in this
 * class, they are requested from the spot history input files using the date
 * and time information of the schedule.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class SpotAllocator {

	/** Whether workload model 1 is being used */
	private boolean isWLM1 = false;

	/** Start date of the schedule */
	private Date startDate = null;

	/** Deadline of the schedule */
	private Date endDate = null;

	/**
	 * Constructor for SpotAllocator
	 * 
	 * @param schedulerResult
	 *            the original schedulerResult
	 * @param useWorkloadModel1
	 *            whether workload model 1 is being used
	 * @param start
	 *            the start date of the schedule
	 * @param end
	 *            the deadline of the scheduleF
	 */
	public SpotAllocator(SchedulerResult schedulerResult,
			boolean useWorkloadModel1, Date start, Date end) {
		isWLM1 = useWorkloadModel1;
		startDate = start;
		endDate = end;
	}

	/**
	 * Determine the schedule that costs the least and set the actual spot
	 * prices of each task hour.
	 * 
	 * @param version
	 *            the allocation scheme option to use
	 * @param scheduledTaskCollection
	 *            the original schedule
	 * @return the updated schedule
	 * @throws InstanceNotAvailableException
	 * @throws InvalidInputException
	 */
	public ScheduledTaskCollection determinePrices(String version,
			ScheduledTaskCollection scheduledTaskCollection)
			throws InstanceNotAvailableException, InvalidInputException {
		InputVerifier.allocationVersionExists(version);
		int hoursavailable = DateUtility.getNumberOfHoursBetween(startDate,
				endDate);
		int subtaskavailable = scheduledTaskCollection.getNumberOfTaskHours();
		int numberToSkip = hoursavailable - subtaskavailable;
		InstanceSpecification spec = null;
		for (SubTask s : scheduledTaskCollection.getSubTasks()) {
			if (!s.isDummy()) {
				spec = s.getSpecification().getInstanceSpecification();
			}
		}
		if (spec != null) {
			SpotModel decisionModel = new SpotModel();
			SpotPrice skipPrice;
			if (version.equals(AllocatorOptions.SPOT)) {
				try {
					// use spotmodel
					skipPrice = decisionModel.executeSimulation(spec,
							subtaskavailable, hoursavailable);
				} catch (Exception e1) {
					// optimal
					skipPrice = PricingManager.getTippingPointSpotPrice(spec,
							startDate, endDate, numberToSkip);
				}
			} else {
				// optimal
				skipPrice = PricingManager.getTippingPointSpotPrice(spec,
						startDate, endDate, numberToSkip);
			}
			int skipped = 0;
			if (isWLM1) {
				// Rescheduling
				int hoursToAdd = 0;
				for (SubTask subTask : scheduledTaskCollection.getSubTasks()) {
					if (!subTask.isDummy()) {
						Date start = new Date(startDate.getTime() + hoursToAdd
								* DateUtility.HOURINMILLISECONDS);
						SpotPrice spotPrice = PricingManager.getSpotPrice(
								subTask.getSpecification()
										.getInstanceSpecification(), start);
						while (spotPrice.getHourlyPrice() > skipPrice
								.getHourlyPrice() && skipped < numberToSkip) {
							// skip
							skipped++;
							hoursToAdd++;
							start = new Date(startDate.getTime() + hoursToAdd
									* DateUtility.HOURINMILLISECONDS);
							spotPrice = PricingManager.getSpotPrice(subTask
									.getSpecification()
									.getInstanceSpecification(), start);
						}
						Date end = new Date(startDate.getTime()
								+ (hoursToAdd + 1)
								* DateUtility.HOURINMILLISECONDS);
						subTask.setTimePeriod(start, end);
						subTask.setPrice(spotPrice);
					}
					hoursToAdd++;
				}
			} else {
				// Set Prices
				int hoursToAdd = 0;
				for (SubTask subTask : scheduledTaskCollection.getSubTasks()) {
					if (!subTask.isDummy()) {
						SpotPrice spotPrice = PricingManager.getSpotPrice(
								subTask.getSpecification()
										.getInstanceSpecification(), subTask
										.getStartDate());
						subTask.setPrice(spotPrice);
					}
					hoursToAdd++;
				}
			}
		}
		return scheduledTaskCollection;
	}

}
