package be.ac.ua.broker.allocation.spot.model;

/**
 * The Checkpoint class stores all the different calculated parameters for a
 * certain checkpointing scheme (optimal or hourly).
 * 
 * @author Vermeersch Kurt (Based on Sangho Yi c++ implementation of the spot
 *         model)
 * @date October 2010
 */
public class Checkpoint {

	/** The total required execution time */
	private double time[] = new double[SpotModelOptions.MAX_GRAN_BID];
	/** The remaining execution time */
	private double price[] = new double[SpotModelOptions.MAX_GRAN_BID];
	/** The number of tasks being executed */
	private int number_task[] = new int[SpotModelOptions.MAX_GRAN_BID];
	/** The filename for the output */
	private String filename;
	/** Description of checkpointing method used */
	private String description;

	/** Distribution Usage Ratio (UR) */
	private double dist_usage_ratio[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_DIST_UR];
	/** Mean Usage Ratio (UR) */
	private double mean_usage_ratio[] = new double[SpotModelOptions.MAX_GRAN_BID];
	/** Usage Ratio (UR) */
	private double usage_ratio[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_SIMU_RECORD];

	/** Distribution Availability Ratio (AR) */
	private double dist_avail_ratio[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_DIST_AR];
	/** Mean Availability Ratio (AR) */
	private double mean_avail_ratio[] = new double[SpotModelOptions.MAX_GRAN_BID];
	/** Availability Time */
	private double time_avail[] = new double[SpotModelOptions.MAX_GRAN_BID];
	/** Availability Ratio (AR) */
	private double avail_ratio[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_SIMU_RECORD];

	/** Distribution Execution Time (ET) */
	private double dist_exec_time[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_DIST_EX];
	/** Mean Execution Time (ET) */
	private double exec_time[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_SIMU_RECORD];

	/** Distribution Gross Workload (GW) */
	private double dist_gros_time[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_DIST_GW];
	/** Gross Workload (GW) */
	private double gros_time[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_SIMU_RECORD];

	/** Expected price per hour (P) */
	private double price_per_hour[] = new double[SpotModelOptions.MAX_GRAN_BID];

	/** Distribution Total Cost (TC aka M) (total cost = GW * P) */
	private double dist_total_cost[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_DIST_TC];
	/** Total Cost (TC aka M) (total cost = GW * P) */
	private double total_cost[][] = new double[SpotModelOptions.MAX_GRAN_BID][SpotModelOptions.MAX_SIMU_RECORD];

	/**
	 * Constructor for Checkpoint which initializes all the values
	 */
	public Checkpoint() {
		for (int j = 0; j < SpotModelOptions.MAX_GRAN_BID; j++) {
			price[j] = 0;
			time[j] = 0;
			number_task[j] = 0;
			// FOR P
			price_per_hour[j] = 0;
			// FOR UR
			for (int l = 0; l < SpotModelOptions.MAX_DIST_UR; l++) {
				dist_usage_ratio[j][l] = 0;
			}
			mean_usage_ratio[j] = 0;
			for (int l = 0; l < SpotModelOptions.MAX_SIMU_RECORD; l++) {
				usage_ratio[j][l] = 0;
			}
			// FOR AR
			for (int l = 0; l < SpotModelOptions.MAX_DIST_AR; l++) {
				dist_avail_ratio[j][l] = 0;
			}
			mean_avail_ratio[j] = 0;
			time_avail[j] = 0;
			for (int l = 0; l < SpotModelOptions.MAX_SIMU_RECORD; l++) {
				avail_ratio[j][l] = 0;
			}
			// FOR EX
			for (int l = 0; l < SpotModelOptions.MAX_DIST_EX; l++) {
				dist_exec_time[j][l] = 0;
			}
			for (int l = 0; l < SpotModelOptions.MAX_SIMU_RECORD; l++) {
				exec_time[j][l] = 0;
			}
			// FOR GW
			for (int l = 0; l < SpotModelOptions.MAX_DIST_GW; l++) {
				dist_gros_time[j][l] = 0;
			}
			for (int l = 0; l < SpotModelOptions.MAX_SIMU_RECORD; l++) {
				gros_time[j][l] = 0;
			}
			// FOR TC
			for (int l = 0; l < SpotModelOptions.MAX_DIST_TC; l++) {
				dist_total_cost[j][l] = 0;
			}
			for (int l = 0; l < SpotModelOptions.MAX_SIMU_RECORD; l++) {
				total_cost[j][l] = 0;
			}
		}
	}

	/**
	 * Setter for the name of the output file
	 * 
	 * @param fn
	 *            the name of the output file
	 */
	public void setFileName(final String fn) {
		filename = fn;
	}

	/**
	 * Setter for the description of the corresponding checkpointing scheme
	 * 
	 * @param desc
	 *            description of the checkpointing scheme
	 */
	public void setDescription(final String desc) {
		description = desc;
	}

	/**
	 * Getter for the total required execution time for a certain
	 * category/instance type
	 * 
	 * @return the requested total required execution times
	 */
	public double[] getTime() {
		return time;
	}

	/**
	 * Getter for the remaining execution time for a certain category/instance
	 * type
	 * 
	 * @return the requested remaining execution times
	 */
	public double[] getPrice() {
		return price;
	}

	/**
	 * Getter for the number of tasks being executed for a certain
	 * category/instance type
	 * 
	 * @return the number of tasks being executed
	 */
	public int[] getNumber_task() {
		return number_task;
	}

	/**
	 * Getter for the distribution usage ratio for a certain category/instance
	 * type
	 * 
	 * @return the requested distribution usage ratios
	 */
	public double[][] getDist_usage_ratio() {
		return dist_usage_ratio;
	}

	/**
	 * Getter for the mean usage ratio for a certain category/instance type
	 * 
	 * @return the requested mean usage ratios
	 */
	public double[] getMean_usage_ratio() {
		return mean_usage_ratio;
	}

	/**
	 * Getter for the usage ratio for a certain category/instance type
	 * 
	 * @return the requested usage ratios
	 */
	public double[][] getUsage_ratio() {
		return usage_ratio;
	}

	/**
	 * Getter for the distribution availability ratios for a certain
	 * category/instance type
	 * 
	 * @return the requested distribution availability ratios
	 */
	public double[][] getDist_avail_ratio() {
		return dist_avail_ratio;
	}

	/**
	 * Getter for the mean availability ratios for a certain category/instance
	 * type
	 * 
	 * @return the requested mean availability ratios
	 */
	public double[] getMean_avail_ratio() {
		return mean_avail_ratio;
	}

	/**
	 * Getter for the availability times for a certain category/instance type
	 * 
	 * @return the requested availability times
	 */
	public double[] getTime_avail() {
		return time_avail;
	}

	/**
	 * Getter for the availability ratios for a certain category/instance type
	 * 
	 * @return the requested availability ratios
	 */
	public double[][] getAvail_ratio() {
		return avail_ratio;
	}

	/**
	 * Getter for the distribution execution time for a certain
	 * category/instance type
	 * 
	 * @return the requested distribution execution times
	 */
	public double[][] getDist_exec_time() {
		return dist_exec_time;
	}

	/**
	 * Getter for the execution times for a certain category/instance type
	 * 
	 * @return the requested execution times
	 */
	public double[][] getExec_time() {
		return exec_time;
	}

	/**
	 * Getter for the distribution gross workload for a certain
	 * category/instance type
	 * 
	 * @return the requested distribution gross workloads
	 */
	public double[][] getDist_gros_time() {
		return dist_gros_time;
	}

	/**
	 * Getter for the gross workload for a certain category/instance type
	 * 
	 * @return the requested gross workloads
	 */
	public double[][] getGros_time() {
		return gros_time;
	}

	/**
	 * Getter for the expected price per hour for a certain category/instance
	 * type
	 * 
	 * @return the requested expected price per hours
	 */
	public double[] getPrice_per_hour() {
		return price_per_hour;
	}

	/**
	 * Getter for the distribution total cost for a certain category/instance
	 * type
	 * 
	 * @return the requested distribution total costs
	 */
	public double[][] getDist_total_cost() {
		return dist_total_cost;
	}

	/**
	 * Getter for the total cost for a certain category/instance type
	 * 
	 * @return the requested total costs
	 */
	public double[][] getTotal_cost() {
		return total_cost;
	}

	/**
	 * @return the filename of the corresponding output file
	 */
	public String getFileName() {
		return filename;
	}

	/**
	 * @return the description of the checkpointing scheme that is being used
	 */
	public String getDescription() {
		return description;
	}

}
