package be.ac.ua.broker.allocation.spot.model;

import java.io.File;
import java.io.IOException;

import com.csvreader.CsvWriter;

/**
 * The ResultWriter class is used to write all the results of the simulations to
 * CSV files.
 * 
 * @author Vermeersch Kurt (Based on Sangho Yi c++ implementation of the spot
 *         model)s
 * @date October 2010
 */
public class ResultWriter {

	/**
	 * The output path to use for the results written to file by this
	 * ResultWriter.
	 */
	private String outputPath = SpotModelOptions.OUTPUT_FOLDER;

	/** The state of the simulation */
	private State state;

	/**
	 * Constructor for ResultWriter
	 * 
	 * @param st
	 *            the state of the simulation
	 * @param instance
	 *            the instance type whose results will be written to file
	 */
	public ResultWriter(State st, final String instance) {
		state = st;
		outputPath += (instance + "/");
	}

	/**
	 * Store one result of a simulation to file
	 * 
	 * @param ckpt_method
	 *            the checkpointing scheme that was used
	 * @param time_appexec
	 *            the length of the task that was being used in the simulation
	 */
	public void storeEachResult(final int ckpt_method, final int time_appexec) {
		int i;
		double d_price, d_time;
		int number_task;
		// Create one directory
		boolean success = (new File(outputPath + "result_" + time_appexec
				+ "minutes")).mkdirs();
		if (success) {
			System.out.println("Directory: " + outputPath + "result_"
					+ time_appexec + "minutes" + " created");
		}
		Checkpoint ckpt = state.getCheckpoint(ckpt_method);
		SrcRecord r = state.getSourceRecord();
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/" + ckpt.getFileName());
			writer.write(ckpt.getDescription());
			writer.endRecord();
			writer.write("");
			writer.write("");
			writer.write("");
			writer.write("EC2_" + r.getInstanceType());
			writer.endRecord();
			writer.write("User's Bid");
			writer.write("price");
			writer.write("time");
			writer.write("n_task");
			// }
			writer.endRecord();
			for (i = 0; i < SpotModelOptions.MAX_GRAN_BID; i++) {
				writer.write(String.valueOf((double) (i) * 0.001));
				d_price = ckpt.getPrice()[i];
				d_time = ckpt.getTime()[i];
				number_task = ckpt.getNumber_task()[i];
				if (d_price == 0
						|| number_task != SpotModelOptions.MAX_SIMU_RECORD) {
					d_price = 1000000; // large enough number (infinite)
				}
				if (d_time == 0
						|| number_task != SpotModelOptions.MAX_SIMU_RECORD) {
					d_time = 1000000; // large enough number (infinite)
				}
				writer.write(String.valueOf(d_price));
				writer.write(String.valueOf(d_time));
				writer.write(String.valueOf(number_task));
				writer.endRecord();
				writer.flush();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Store the results of the simulation to file
	 * 
	 * @param time_appexec
	 *            the length of the simulation
	 */
	public void storeResults(final int time_appexec) {
		int i, k, l, m;
		double temp_d = 0.0;
		double prob[] = { 0.5, 0.6, 0.7, 0.8, 0.9, 0.99 };
		SrcRecord r = state.getSourceRecord();
		Task a = state.getTask();
		// store in one file (for easier comparison)
		try {
			CsvWriter writer = new CsvWriter(outputPath + "results.csv");
			// Itype // Ub // Ckpt // T(w_inst) // P // ET // GW // P // M // AR
			// // UR
			String titles[] = { "type", "bid", "ckpt", "T(w_inst)", "prob",
					"et", "gw", "ep", "m", "ar", "ur" };
			writer.writeRecord(titles);
			for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
					.getHighest() * 1000.0 + 0.5) - 1; i++) {
				if (i < 0) {
					continue;
				}
				for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
					Checkpoint ckpt = state.getCheckpoint(l);
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}

					for (m = 0; m < 6; ++m) {
						double et = 0, gw = 0, ep, tc = 0, ar = 0, ur = 0;
						// et
						for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_EX; k++) {
							temp_d += ckpt.getDist_exec_time()[i][k];
							if (temp_d >= prob[m]) {
								et = (double) (k)
										* SpotModelOptions.INTERVAL_EX;
								break;
							}
						}

						// gw (at)
						for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_GW; k++) {
							temp_d += ckpt.getDist_gros_time()[i][k];
							if (temp_d >= prob[m]) {
								gw = (double) (k)
										* SpotModelOptions.INTERVAL_GW;
								break;
							}
						}

						// ar
						for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_AR; k++) {
							temp_d += ckpt.getDist_avail_ratio()[i][k];
							if (temp_d >= prob[m]) {
								ar = (double) k / 100.0;
								break;
							}
						}

						// ur
						for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_UR; k++) {
							temp_d += ckpt.getDist_usage_ratio()[i][k];
							if (temp_d >= prob[m]) {
								ur = (double) k / 100.0;
								break;
							}
						}

						// tc (M)
						for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_TC; k++) {
							temp_d += ckpt.getDist_total_cost()[i][k];
							if (temp_d >= prob[m]) {
								tc = (double) (k)
										* (double) (SpotModelOptions.INTERVAL_TC);
								break;
							}
						}

						// ep
						ep = ckpt.getPrice_per_hour()[i];

						writer.write(r.getInstanceType());
						double bid = (double) (i) * 0.001;
						writer.write(String.valueOf(bid));
						if (l == SpotModelOptions.OPT_CKPT) {
							writer.write("O");
						} else {
							writer.write("H");
						}
						Double time = a.getTotalExecutionTime();
						writer.write(String.valueOf(time));
						writer.write(String.valueOf(prob[m]));
						writer.write(String.valueOf(et));
						writer.write(String.valueOf(gw));
						writer.write(String.valueOf(ep));
						writer.write(String.valueOf(tc));
						writer.write(String.valueOf(ar));
						writer.write(String.valueOf(ur));
						writer.endRecord();
						writer.flush();
					}
				}
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// store in one file (for easier comparison)
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/price.csv");
			// Labels
			writer.write("User's Bid");
			// max # of loops: MAX_CHECKPOINT
			for (k = 0; k < SpotModelOptions.MAX_CHECKPOINT; k++) {
				writer.write("");
			}
			writer.write("EC2_" + r.getInstanceType());
			writer.endRecord();
			// Contents
			for (i = 0; i < SpotModelOptions.MAX_GRAN_BID; i++) {
				double temporary = (double) (i) * 0.001;
				writer.write(String.valueOf(temporary));
				// max # of loops: MAX_CHECKPOINT
				for (k = 0; k < SpotModelOptions.MAX_CHECKPOINT; k++) {
					Checkpoint ckpt = state.getCheckpoint(k);
					double price = ckpt.getPrice()[i];
					if (price == 0
							|| ckpt.getNumber_task()[i] != SpotModelOptions.MAX_SIMU_RECORD) {
						// large enough price (infinite)
						writer.write("1000000");
					} else {
						writer.write(String.valueOf(price));
					}
				}
				writer.write("---");
				writer.endRecord();
				writer.flush();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// store in one file (for easier comparison)
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/time.csv");
			// Labels
			writer.write("User's Bid");
			// max # of loops: MAX_CHECKPOINT
			for (k = 0; k < SpotModelOptions.MAX_CHECKPOINT; k++) {
				writer.write("");
			}
			writer.write("EC2_" + r.getInstanceType());
			writer.endRecord();
			// Contents
			for (i = 0; i < SpotModelOptions.MAX_GRAN_BID; i++) {
				double temporary = (double) (i) * 0.001;
				writer.write(String.valueOf(temporary));
				// max # of loops: MAX_CHECKPOINT
				for (k = 0; k < SpotModelOptions.MAX_CHECKPOINT; k++) {
					Checkpoint ckpt = state.getCheckpoint(k);
					double time = ckpt.getTime()[i];
					if (time == 0
							|| ckpt.getNumber_task()[i] != SpotModelOptions.MAX_SIMU_RECORD) {
						// large enough time (infinite)
						writer.write("1000000");
					} else {
						writer.write(String.valueOf(time));
					}
				}
				writer.write("---");
				writer.endRecord();
				writer.flush();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// store in one file (for easier comparison)
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/product.csv");
			// Labels
			writer.write("User's Bid");
			// max # of loops: MAX_CHECKPOINT
			for (k = 0; k < SpotModelOptions.MAX_CHECKPOINT; k++) {
				writer.write("");
			}
			writer.write("EC2_" + r.getInstanceType());
			writer.endRecord();
			// Contents
			for (i = 0; i < SpotModelOptions.MAX_GRAN_BID; i++) {
				double temporary = (double) (i) * 0.001;
				writer.write(String.valueOf(temporary));
				// max # of loops: MAX_CHECKPOINT
				for (k = 0; k < SpotModelOptions.MAX_CHECKPOINT; k++) {
					Checkpoint ckpt = state.getCheckpoint(k);
					double time = ckpt.getTime()[i];
					double price = ckpt.getPrice()[i];
					if (time == 0
							|| price == 0
							|| ckpt.getNumber_task()[i] != SpotModelOptions.MAX_SIMU_RECORD) {
						// large enough product number (infinite)
						writer.write("1000000");
					} else {
						writer.write(String.valueOf(time * price));
					}
				}
				writer.write("---");
				writer.endRecord();
				writer.flush();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// FOR TC (a.k.a. M)
		temp_d = handleTC(time_appexec, temp_d, r);
		// FOR P
		handleEP(time_appexec, r);
		// FOR GW
		handleGW(time_appexec, temp_d, r);
		// FOR ET
		handleET(time_appexec, r);
		// FOR AR
		handleAR(time_appexec, r);
		// FOR UR
		handleUR(time_appexec, r);
	}

	/**
	 * Write total monetary cost results to file
	 */
	private double handleTC(final int time_appexec, double temp_d, SrcRecord r) {
		int i;
		int k;
		int l;
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/tc_" + r.getInstanceType()
					+ ".csv");
			writer.write("*********************************************************************");
			writer.endRecord();
			writer.write("CDF of Total Cost (Cumulative Probability Distribution of Total Cost)");
			writer.endRecord();
			writer.write("User's Bid");
			// all things for TC (dividing them by the total number of
			// intervals)
			for (i = 0; i < SpotModelOptions.MAX_DIST_TC; i++) {
				writer.write(String.valueOf(i
						* SpotModelOptions.INTERVAL_TC));
			}
			writer.endRecord();
			for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
					.getHighest() * 1000.0 + 0.5) - 1; i++) {
				Checkpoint ckpt0 = state.getCheckpoint(0);
				if (ckpt0.getNumber_task()[i] == 0) {
					continue;
				}
				writer.write(String.valueOf((double) (i) * 0.001) + "-OPT");
				for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_TC; k++) {
					temp_d += ckpt0.getDist_total_cost()[i][k];
					writer.write(String.valueOf(temp_d));
				}
				writer.endRecord();
				Checkpoint ckpt1 = state.getCheckpoint(1);
				if (ckpt1.getNumber_task()[i] == 0) {
					continue;
				}
				writer.write(String.valueOf((double) (i) * 0.001) + "-HOUR");
				for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_TC; k++) {
					temp_d += ckpt1.getDist_total_cost()[i][k];
					writer.write(String.valueOf(temp_d));
				}
				writer.endRecord();
				writer.flush();
			}

			for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
				writer.write("*********************************************************************");
				writer.endRecord();
				Checkpoint ckpt = state.getCheckpoint(l);
				writer.write(ckpt.getDescription());
				writer.endRecord();
				writer.endRecord();
				writer.write("RAW data of Total Cost (all raw results)");
				writer.endRecord();
				writer.write("User's Bid");
				for (i = 0; i < SpotModelOptions.MAX_SIMU_RECORD; i++) {
					writer.write(String.valueOf(i));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_SIMU_RECORD; k++) {
						writer.write(String.valueOf(ckpt.getTotal_cost()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.flush();
				writer.endRecord();
				writer.endRecord();
				writer.write("PDF of Total Cost (Probability Distribution of Total Cost)");
				writer.endRecord();
				writer.write("User's Bid");
				// all things for TC (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_TC; i++) {
					writer.write(String.valueOf(i
							* SpotModelOptions.INTERVAL_TC));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_DIST_TC; k++) {
						writer.write(String.valueOf(ckpt.getDist_total_cost()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("CDF of Total Cost (Cumulative Probability Distribution of Total Cost)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				// all things for TC (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_TC; i++) {
					writer.write(String.valueOf(i
							* SpotModelOptions.INTERVAL_TC));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_TC; k++) {
						temp_d += ckpt.getDist_total_cost()[i][k];
						writer.write(String.valueOf(temp_d));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("Expected Total Cost");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					writer.write(String.valueOf(ckpt.getPrice()[i]));
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return temp_d;
	}

	/**
	 * Write price results to file
	 */
	private void handleEP(final int time_appexec, SrcRecord r) {
		int i;
		int l;
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/p_" + r.getInstanceType()
					+ ".csv");
			for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
				writer.write("*********************************************************************");
				writer.endRecord();
				Checkpoint ckpt = state.getCheckpoint(l);
				writer.write(ckpt.getDescription());
				writer.endRecord();
				writer.endRecord();
				writer.write("Expected Price per hour");
				writer.endRecord();
				writer.write("User's Bid");
				writer.endRecord();
				writer.flush();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					writer.write(String.valueOf(ckpt.getPrice_per_hour()[i]));
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Write gross workload results to file
	 */
	private void handleGW(final int time_appexec, double temp_d, SrcRecord r) {
		int i;
		int k;
		int l;
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/gw_" + r.getInstanceType()
					+ ".csv");
			for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
				writer.write("*********************************************************************");
				writer.endRecord();
				Checkpoint ckpt = state.getCheckpoint(l);
				writer.write(ckpt.getDescription());
				writer.endRecord();
				writer.endRecord();
				writer.write("RAW data of Gross Workload - GW (all raw results)");
				writer.endRecord();
				writer.write("User's Bid");
				for (i = 0; i < SpotModelOptions.MAX_SIMU_RECORD; i++) {
					writer.write(String.valueOf(i));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_SIMU_RECORD; k++) {
						writer.write(String.valueOf(ckpt.getGros_time()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.flush();
				writer.endRecord();
				writer.endRecord();
				writer.write("PDF of Gross Workload - GW (Probability Distribution of Gross Workload)");
				writer.endRecord();
				writer.write("User's Bid");
				// all things for GW (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_GW; i++) {
					writer.write(String.valueOf(i
							* SpotModelOptions.INTERVAL_GW));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_DIST_GW; k++) {
						writer.write(String.valueOf(ckpt.getDist_gros_time()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("CDF of Gross Workload - GW (Cumulative Probability Distribution of Gross Workload)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				// all things for GW (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_GW; i++) {
					writer.write(String.valueOf(i
							* SpotModelOptions.INTERVAL_GW));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_GW; k++) {
						temp_d += ckpt.getDist_gros_time()[i][k];
						writer.write(String.valueOf(temp_d));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("Expected Gross Workload");
				writer.endRecord();
				writer.write("User's Bid");
				writer.endRecord();
				writer.flush();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					for (temp_d = k = 0; k < SpotModelOptions.MAX_SIMU_RECORD; k++) {
						temp_d += ckpt.getGros_time()[i][k];
					}
					temp_d /= SpotModelOptions.MAX_SIMU_RECORD;
					writer.write(String.valueOf((double) (i) * 0.001));
					writer.write(String.valueOf(temp_d));
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Write execution time results to file
	 */
	private void handleET(final int time_appexec, SrcRecord r) {
		int i;
		int k;
		int l;
		double temp_d;
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/et_" + r.getInstanceType()
					+ ".csv");
			writer.write("*********************************************************************");
			writer.endRecord();
			writer.write("CDF of Execution Time (Cumulative Probability Distribution of Execution Time)");
			writer.endRecord();
			writer.flush();
			writer.write("User's Bid");
			// all things for ET (dividing them by the total number of
			// intervals)
			for (i = 0; i < SpotModelOptions.MAX_DIST_EX; i++) {
				writer.write(String.valueOf(i
						* SpotModelOptions.INTERVAL_EX));
			}
			writer.endRecord();
			for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
					.getHighest() * 1000.0 + 0.5) - 1; i++) {
				Checkpoint ckpt0 = state.getCheckpoint(0);
				if (ckpt0.getNumber_task()[i] == 0) {
					continue;
				}
				writer.write(String.valueOf((double) (i) * 0.001) + "-OPT");
				for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_EX; k++) {
					temp_d += ckpt0.getDist_exec_time()[i][k];
					writer.write(String.valueOf(temp_d));
				}
				writer.endRecord();
				Checkpoint ckpt1 = state.getCheckpoint(1);
				if (ckpt1.getNumber_task()[i] == 0) {
					continue;
				}
				writer.write(String.valueOf((double) (i) * 0.001) + "-HOUR");
				for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_EX; k++) {
					temp_d += ckpt1.getDist_exec_time()[i][k];
					writer.write(String.valueOf(temp_d));
				}
			}
			for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
				writer.write("*********************************************************************");
				writer.endRecord();
				Checkpoint ckpt = state.getCheckpoint(l);
				writer.write(ckpt.getDescription());
				writer.endRecord();
				writer.endRecord();
				writer.write("RAW data of Execution Time (all raw results)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				for (i = 0; i < SpotModelOptions.MAX_SIMU_RECORD; i++) {
					writer.write(String.valueOf(i));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_SIMU_RECORD; k++) {
						writer.write(String.valueOf(ckpt.getExec_time()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("PDF of Execution Time (Probability Distribution of Execution Time)");
				writer.endRecord();
				writer.write("User's Bid");
				// all things for ET (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_EX; i++) {
					writer.write(String.valueOf(i
							* SpotModelOptions.INTERVAL_EX));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_DIST_EX; k++) {
						writer.write(String.valueOf(ckpt.getDist_exec_time()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("CDF of Execution Time (Cumulative Probability Distribution of Execution Time)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				// all things for ET (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_EX; i++) {
					writer.write(String.valueOf(i
							* SpotModelOptions.INTERVAL_EX));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (temp_d = k = 0; k < SpotModelOptions.MAX_DIST_EX; k++) {
						temp_d += ckpt.getDist_exec_time()[i][k];
						writer.write(String.valueOf(temp_d));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("Expected Execution Time");
				writer.endRecord();
				writer.write("User's Bid");
				writer.endRecord();
				writer.flush();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					writer.write(String.valueOf(ckpt.getTime()[i]));
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Write availability ratio results to file
	 */
	private void handleAR(final int time_appexec, SrcRecord r) {
		int i;
		int k;
		int l;
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/ar_" + r.getInstanceType()
					+ ".csv");
			for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
				writer.write("*********************************************************************");
				writer.endRecord();
				Checkpoint ckpt = state.getCheckpoint(l);
				writer.write(ckpt.getDescription());
				writer.endRecord();
				writer.endRecord();
				writer.write("RAW data of Availability Ratio (all raw results)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				for (i = 0; i < SpotModelOptions.MAX_SIMU_RECORD; i++) {
					writer.write(String.valueOf(i));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_SIMU_RECORD; k++) {
						writer.write(String.valueOf(ckpt.getAvail_ratio()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				// print the Distribution of Available Ratio
				writer.write("PDF of Availability Ratio (Probability Distribution of AR)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				for (i = 0; i < SpotModelOptions.MAX_DIST_AR; i++) {
					writer.write(String.valueOf((double) i
							/ (double) SpotModelOptions.MAX_DIST_AR));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_DIST_AR; k++) {
						writer.write(String.valueOf(ckpt.getDist_avail_ratio()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("Expected Availability Ratio");
				writer.endRecord();
				writer.write("User's Bid");
				writer.endRecord();
				writer.flush();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					writer.write(String.valueOf(ckpt.getMean_avail_ratio()[i]));
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Write utilization ratio results to file
	 */
	private void handleUR(final int time_appexec, SrcRecord r) {
		int i;
		int k;
		int l;
		// FOR UR
		try {
			CsvWriter writer = new CsvWriter(outputPath + "result_"
					+ time_appexec + "minutes/ur_" + r.getInstanceType()
					+ ".csv");
			for (l = 0; l < SpotModelOptions.MAX_CHECKPOINT; l++) {
				writer.write("*********************************************************************");
				writer.endRecord();
				Checkpoint ckpt = state.getCheckpoint(l);
				writer.write(ckpt.getDescription());
				writer.endRecord();
				writer.endRecord();
				writer.write("RAW data of Usage Ratio (all raw results)");
				writer.endRecord();
				writer.flush();
				writer.write("User's Bid");
				for (i = 0; i < SpotModelOptions.MAX_SIMU_RECORD; i++) {
					writer.write(String.valueOf(i));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_SIMU_RECORD; k++) {
						writer.write(String.valueOf(ckpt.getUsage_ratio()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("PDF of Usage Ratio (Probability Distribution of UR - Artur says 'NR')");
				writer.endRecord();
				writer.write("User's Bid");
				writer.flush();
				// all things for UR (dividing them by the total number of
				// intervals)
				for (i = 0; i < SpotModelOptions.MAX_DIST_UR; i++) {
					writer.write(String.valueOf((double) i
							/ (double) SpotModelOptions.MAX_DIST_UR));
				}
				writer.endRecord();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					for (k = 0; k < SpotModelOptions.MAX_DIST_UR; k++) {
						writer.write(String.valueOf(ckpt.getDist_usage_ratio()[i][k]));
					}
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
				writer.write("Expected Usage Ratio");
				writer.endRecord();
				writer.write("User's Bid");
				writer.endRecord();
				writer.flush();
				for (i = (int) (r.getLowest() * 1000.0 + 0.5) - 1; i <= (int) (r
						.getHighest() * 1000.0 + 0.5) - 1; i++) {
					if (ckpt.getNumber_task()[i] == 0) {
						continue;
					}
					writer.write(String.valueOf((double) (i) * 0.001));
					writer.write(String.valueOf(ckpt.getMean_usage_ratio()[i]));
					writer.endRecord();
				}
				writer.endRecord();
				writer.endRecord();
				writer.endRecord();
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
