package be.ac.ua.broker.allocation.spot.model;

/**
 * The Simulator class performs the actual simulations by first creating all the
 * needed data structures, then performing the simulation and finally writing the
 * results to file.
 * 
 * @author Vermeersch Kurt (Based on Sangho Yi c++ implementation of the spot
 *         model)
 * @date October 2010
 */
public class Simulator {

	/** State used for the simulation */
	private State state;

	/** ResultWriter used to write the results to file */
	private ResultWriter rw;

	/**
	 * Perform an actual simulation: by first of all creating the needed
	 * datastructures, then performing the simulation and finally writing the
	 * results to file.
	 * 
	 * @param fileName
	 *            the name of the output file to use
	 * @param time_appexec
	 *            the lenth of the simulation
	 */
	public void simulate(final String fileName, final String instance, final int time_appexec) {
		// initialize all data structures used in this simulator
		SrcRecord r = new SrcRecord(fileName,instance);
		Parameter p = new Parameter();
		Task a = new Task(time_appexec,instance);
		Checkpoint c[] = new Checkpoint[SpotModelOptions.MAX_CHECKPOINT];
		for (int i = 0; i < c.length; i++) {
			c[i] = new Checkpoint();
		}
		if (SpotModelOptions.MAX_CHECKPOINT > 0) {
			c[0].setFileName("result_01_opt_ckpt.csv");
			c[0].setDescription("(1) Optimal Checkpointing\n");
		}
		if (SpotModelOptions.MAX_CHECKPOINT > 1) {
			c[1].setFileName("result_02_hr_ckpt.csv");
			c[1].setDescription("(2) Hour Checkpointing\n");
		}
		state = new State(r, p, a, c);
		rw = new ResultWriter(state,instance);
		// now read the source data
		System.out
				.println("PHASE 1: reading source data from file:                   ");
		DataReader dr = new DataReader(state);
		dr.readSourceData();
		System.out.println("[ OK ]\n");

		// now simulate several checkpointing schemes
		System.out
				.println("PHASE 2.1: optimal checkpointing:                           ");
		simulateCkpt(SpotModelOptions.OPT_CKPT, time_appexec);
		System.out.println("[ OK ]\n");
		System.out
				.println("PHASE 2.2: hour checkpointing:                              ");
		simulateCkpt(SpotModelOptions.HR_CKPT, time_appexec);
		System.out.println("[ OK ]\n");

		// store results to files
		System.out
				.println("PHASE 3: storing results:                                 ");
		rw.storeResults(time_appexec);
		System.out.println("[ OK ]\n");
		for (int i = 0; i < c.length; i++) {
			c[i] = null;
		}
		r = null;
	}

	/**
	 * Simulate the actual checkpointing (0=optimal,1=hourly) for the specified
	 * amount of time
	 * 
	 * @param checkpointingTechnique
	 *            the checkpointing scheme to use
	 * @param time_appexec
	 *            the task length to use
	 */
	private void simulateCkpt(final int checkpointingTechnique, final int time_appexec) {
		if (checkpointingTechnique != SpotModelOptions.OPT_CKPT
				&& checkpointingTechnique != SpotModelOptions.HR_CKPT) {
			System.err.println("Unknown checkpointing technique");
		} else {
			int i, k, x;
			double max_bid;
			int avail_flag;
			int price_flag;
			double result_time, result_price;

			// FOR UR
			double usage_ratio;

			// FOR AR
			double avail_ratio;

			// (0) optimal ckpt
			max_bid = 0;
			SrcRecord r = state.getSourceRecord();
			Parameter p = state.getParameter();
			Task a = state.getTask();
			Checkpoint c = state.getCheckpoint(checkpointingTechnique);

			double[] origdata = r.getSrcdata();
			double[] timeavail = c.getTime_avail();
			double[] cprice = c.getPrice();
			double[] ctime = c.getTime();
			int[] cnumbertask = c.getNumber_task();
			double[][] cusageratio = c.getUsage_ratio();
			double[] cmeanusageratio = c.getMean_usage_ratio();
			double[][] cdistusageratio = c.getDist_usage_ratio();
			double[][] cavailratio = c.getAvail_ratio();
			double[] cmeanavailratio = c.getMean_avail_ratio();
			double[][] cdistavailratio = c.getDist_avail_ratio();
			double[][] cexectime = c.getExec_time();
			double[][] cdistexectime = c.getDist_exec_time();
			double[][] cgrostime = c.getGros_time();
			double[][] cdistgrostime = c.getDist_gros_time();
			double[][] ctotalcost = c.getTotal_cost();
			double[][] cdisttotalcost = c.getDist_total_cost();
			double[] cpriceperhour = c.getPrice_per_hour();

			for (k = (int) (r.getLowest() * 1000.0 + 0.5) - 1, max_bid = r
					.getLowest(); max_bid <= r.getHighest(); k++, max_bid += 0.001) {
				if (k >= SpotModelOptions.MAX_GRAN_BID) {
					System.out.println("ERROR on k - should be fixed! \n");
					break;
				}
				if (k < 0) {
					break;
				}
				for (x = 0; x < SpotModelOptions.MAX_SIMU_RECORD; x++) {
					result_time = result_price = 0;

					// FOR AR
					timeavail[k] = 0;

					for (price_flag = avail_flag = -1, i = x
							* SpotModelOptions.INTERVAL_SIMU; i < r
							.getRecord(); i++) {
						// starting at every hour (x*60)
						if (origdata[i] >= max_bid) {
							// for each available duration
							if (avail_flag != -1) {
								if (checkpointingTechnique == SpotModelOptions.OPT_CKPT) {
									result_time += (i - price_flag)
											- p.getCheckpointingTime()
											- p.getCheckpointingOverheadTime();
								} else if (checkpointingTechnique == SpotModelOptions.HR_CKPT) {
									// do nothing
								}
								avail_flag = -1;
								price_flag = -1;
							}
						} else {
							// starting of each available duration

							// FOR AR
							timeavail[k] += 1.0;

							if (avail_flag == -1) {
								avail_flag = i;
								// add the waiting time
								price_flag = i + (int) p.getWaitingTime();
							}
							if ((i - (avail_flag + (int) (p.getWaitingTime()))) % 60 == 59
									&& (avail_flag + (int) (p.getWaitingTime())) <= i) {

								if (checkpointingTechnique == SpotModelOptions.OPT_CKPT) {
									// accumulate time
									result_time += i - (price_flag - 1);
									// move price_flag to
									// the next starting hour
									price_flag = i + 1;
								} else if (checkpointingTechnique == SpotModelOptions.HR_CKPT) {
									if ((i - (price_flag - 1)) >= p
											.getCheckpointingTime()
											+ p.getCheckpointingOverheadTime()) {
										// accumulate time
										result_time += (i - (price_flag - 1))
												- p.getCheckpointingTime()
												- p.getCheckpointingOverheadTime();
										// move price_flag to the
										// next starting hour
										price_flag = i + 1;
									}
								}
								// accumulate price
								result_price += origdata[i];
							}
							if (price_flag != -1
									&& result_time + (i - (price_flag - 1)) >= a
											.getTotalExecutionTime()) {
								// starting at every hour (x*60)
								result_time = i
										- (x
												* SpotModelOptions.INTERVAL_SIMU - 1);
								if (price_flag <= i) {
									// accumulate price
									result_price += origdata[i];
								}
								// store the results
								cprice[k] += result_price;
								ctime[k] += result_time;
								cnumbertask[k]++;

								// FOR UR
								usage_ratio = a.getTotalExecutionTime()
										/ result_time;
								cusageratio[k][x] = usage_ratio;
								cmeanusageratio[k] += usage_ratio;
								i = (int) ((usage_ratio - (double) (1.0 / SpotModelOptions.MAX_DIST_UR)) * SpotModelOptions.MAX_DIST_UR);
								if (i < 0) {
									i = 0;
								}
								if (i >= SpotModelOptions.MAX_DIST_UR) {
									i = SpotModelOptions.MAX_DIST_UR - 1;
								}
								cdistusageratio[k][i] += 1.0;

								// FOR AR
								avail_ratio = timeavail[k] / result_time;
								cavailratio[k][x] = avail_ratio;
								cmeanavailratio[k] += avail_ratio;
								i = (int) ((avail_ratio - (double) (1.0 / SpotModelOptions.MAX_DIST_AR)) * SpotModelOptions.MAX_DIST_AR);
								if (i < 0) {
									i = 0;
								}
								if (i >= SpotModelOptions.MAX_DIST_AR) {
									i = SpotModelOptions.MAX_DIST_AR - 1;
								}
								cdistavailratio[k][i] += 1.0;

								// FOR ET
								cexectime[k][x] = result_time;
								i = (int) result_time
										/ SpotModelOptions.INTERVAL_EX;
								if (i < 0) {
									i = 0;
								}
								if (i >= SpotModelOptions.MAX_DIST_EX) {
									i = SpotModelOptions.MAX_DIST_EX - 1;
								}
								cdistexectime[k][i] += 1.0;

								// FOR GW
								cgrostime[k][x] = result_time * avail_ratio;
								i = (int) (result_time * avail_ratio)
										/ SpotModelOptions.INTERVAL_GW;
								if (i < 0) {
									i = 0;
								}
								if (i >= SpotModelOptions.MAX_DIST_GW) {
									i = SpotModelOptions.MAX_DIST_GW - 1;
								}
								cdistgrostime[k][i] += 1.0;

								// FOR TC
								ctotalcost[k][x] = result_price;
								i = (int) (result_price / SpotModelOptions.INTERVAL_TC);
								if (i < 0) {
									i = 0;
								}
								if (i >= SpotModelOptions.MAX_DIST_TC) {
									i = SpotModelOptions.MAX_DIST_TC - 1;
								}
								cdisttotalcost[k][i] += 1.0;

								// FOR P
								if (checkpointingTechnique == SpotModelOptions.OPT_CKPT) {
									cpriceperhour[k] += (result_price * 60.0)
											/ cgrostime[k][x];
								} else if (checkpointingTechnique == SpotModelOptions.HR_CKPT) {
									cpriceperhour[k] += result_price
											/ cgrostime[k][x];
								}
								break;
							}
						}
					}
				}
				i = setMeanValue(k, timeavail, cprice, ctime, cnumbertask,
						cmeanusageratio, cdistusageratio, cmeanavailratio,
						cdistavailratio, cdistexectime, cdistgrostime,
						cdisttotalcost, cpriceperhour);
			}

			rw.storeEachResult(checkpointingTechnique, time_appexec);
		}
	}

	/** 
	 * Set the mean value by dividing total values by the number of tasks used in the simulation
	 */
	private int setMeanValue(int k, double[] timeavail, double[] cprice,
			double[] ctime, int[] cnumbertask, double[] cmeanusageratio,
			double[][] cdistusageratio, double[] cmeanavailratio,
			double[][] cdistavailratio, double[][] cdistexectime,
			double[][] cdistgrostime, double[][] cdisttotalcost,
			double[] cpriceperhour) {
		int i;
		// to get the mean value,
		// we divide it by the number of tasks
		if (cnumbertask[k] != 0) {
			cprice[k] /= cnumbertask[k];
			ctime[k] /= cnumbertask[k];

			// FOR UR
			cmeanusageratio[k] /= (double) cnumbertask[k];
			for (i = 0; i < SpotModelOptions.MAX_DIST_UR; i++) {
				cdistusageratio[k][i] /= (double) cnumbertask[k];
			}

			// FOR AR
			cmeanavailratio[k] /= (double) cnumbertask[k];
			for (i = 0; i < SpotModelOptions.MAX_DIST_AR; i++) {
				cdistavailratio[k][i] /= (double) cnumbertask[k];
			}

			// FOR ET
			for (i = 0; i < SpotModelOptions.MAX_DIST_EX; i++) {
				cdistexectime[k][i] /= (double) cnumbertask[k];
			}

			// FOR GW
			for (i = 0; i < SpotModelOptions.MAX_DIST_GW; i++) {
				cdistgrostime[k][i] /= (double) cnumbertask[k];
			}

			// FOR TC
			for (i = 0; i < SpotModelOptions.MAX_DIST_TC; i++) {
				cdisttotalcost[k][i] /= (double) cnumbertask[k];
			}

			// FOR P
			cpriceperhour[k] /= (double) cnumbertask[k];
		} else {
			cprice[k] = 0;
			ctime[k] = 0;

			// FOR UR
			cmeanusageratio[k] = 0;
			for (i = 0; i < SpotModelOptions.MAX_DIST_UR; i++) {
				cdistusageratio[k][i] = 0;
			}

			// FOR AR
			timeavail[k] = 0;
			cmeanavailratio[k] = 0;
			for (i = 0; i < SpotModelOptions.MAX_DIST_AR; i++) {
				cdistavailratio[k][i] = 0;
			}

			// FOR ET
			for (i = 0; i < SpotModelOptions.MAX_DIST_EX; i++) {
				cdistexectime[k][i] = 0;
			}

			// FOR GW
			for (i = 0; i < SpotModelOptions.MAX_DIST_GW; i++) {
				cdistgrostime[k][i] = 0;
			}

			// FOR TC
			for (i = 0; i < SpotModelOptions.MAX_DIST_TC; i++) {
				cdisttotalcost[k][i] = 0;
			}

			// FOR P
			cpriceperhour[k] = 0;
		}
		return i;
	}

}
