package be.ac.ua.broker.allocation.spot.model;

/**
 * The SrcRecord class contains the price history data for the instance/category
 * that is being simulated. It also determines the lowest and highest value for
 * that category/instance type.
 * 
 * @author Vermeersch Kurt (Based on Sangho Yi c++ implementation of the spot
 *         model)
 * @date October 2010
 */
public class SrcRecord {

	/** Price history of the different instances/categories */
	private double srcdata[] = new double[SpotModelOptions.MAX_RECORD];

	/** Name of the price history input CSV file */
	private String filename;

	/** Number of records */
	private int n_record;

	/** Lowest value of every category/instance */
	private double lowest = 0.0;

	/** Highest value of every category/instance */
	private double highest = 0.0;

	/** The instance type/category the corresponding data is for */
	private String instanceType = "";

	/**
	 * Constructor for SrcRecord
	 * 
	 * @param fileName
	 *            name of the price history input CSV file
	 * @param instance
	 *            the instance type/category the corresponding file contains the
	 *            data of
	 * 
	 */
	public SrcRecord(final String fileName, final String instance) {
		filename = fileName;
		instanceType = instance;
	}

	/**
	 * Setter for the historic spot price of the given record.
	 * 
	 * @param i
	 *            the record number
	 * @param val
	 *            the price value
	 */
	public void setData(final int i, final double val) {
		srcdata[i] = val;
	}

	/**
	 * Getter for the array containing all the spot prices over time
	 * 
	 * @return the requested spot price history data
	 */
	public double[] getSrcdata() {
		return srcdata;
	}

	/**
	 * Setter for the number of records
	 * 
	 * @param cat
	 *            the number of records
	 */
	public void setRecord(final int rec) {
		n_record = rec;
	}

	/**
	 * @return number of records
	 */
	public int getRecord() {
		return n_record;
	}

	/**
	 * Getter for the lowest value/spot price.
	 * 
	 * @return the lowest historic spot price
	 */
	public Double getLowest() {
		return lowest;
	}

	/**
	 * Getter for the highest value/spot price .
	 * 
	 * @return the highest historic spot price
	 */
	public Double getHighest() {
		return highest;
	}

	/**
	 * @return the name of the price history input CSV file
	 */
	public String getFileName() {
		return filename;
	}

	/**
	 * @return the instance type/category the spot price history is for
	 */
	public String getInstanceType() {
		return instanceType;
	}

	/**
	 * Determine the lowest and highest historic price.
	 */
	public void determineLowestAndHighest() {
		lowest = Double.MAX_VALUE; 
		highest = Double.MIN_VALUE; 
		for (int j = 0; j < n_record; j++) {
			if (lowest > srcdata[j]) {
				lowest = srcdata[j];
			}
			if (highest < srcdata[j]) {
				highest = srcdata[j];
			}
		}
	}

}
