package be.ac.ua.broker.evaluation;

import java.io.IOException;
import java.rmi.activation.UnknownObjectException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Scanner;
import java.util.Set;

import be.ac.ua.broker.allocation.Allocator;
import be.ac.ua.broker.allocation.AllocatorOptions;
import be.ac.ua.broker.input.pricing.EC2Constants;
import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.input.pricing.PricingManager;
import be.ac.ua.broker.input.tasks.TaskCollection;
import be.ac.ua.broker.input.tasks.specification.TaskReader;
import be.ac.ua.broker.output.gui.ScheduleGUI;
import be.ac.ua.broker.output.textual.PriceCalculator;
import be.ac.ua.broker.output.textual.PriceDescriptorNode;
import be.ac.ua.broker.scheduling.region.RegionAllocator;
import be.ac.ua.broker.scheduling.region.RegionWorkloadCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerOptions;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;
import be.ac.ua.broker.utils.ExecutionTimer;
import be.ac.ua.broker.utils.FolderUtils;
import be.ac.ua.broker.utils.InputVerifier;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

import com.csvreader.CsvWriter;

/**
 * Class represents the run of a benchmark which consists of different runs of
 * the broker application in which the four phases of the brokering process are
 * timed and the resulting costs overview is saved to file. This benchmark is
 * build on top of the broker to evaluate its performance in terms of cost and
 * time.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class Benchmark {

	/**
	 * CsvWriter to write the cost and timing results of a benchmark run to.
	 */
	private CsvWriter csvWriterResults;

	/**
	 * CsvWriter to write the workload tasks used during the benchmark run to.
	 */
	private CsvWriter csvWriterTasks;

	/**
	 * List of task arrays to store the tasks' locations in during the benchmark
	 * runs.
	 */
	private List<String[]> taskArr = new ArrayList<String[]>();

	/**
	 * Timer used to time the different phases of a benchmark run.
	 */
	private ExecutionTimer timer;

	/**
	 * Price descriptions tree to store the pricing information of the last
	 * broker run in.
	 */
	private PriceDescriptorNode tree;

	/**
	 * Run the benchmark using the parameters specified in the BenchmarkOptions.
	 * 
	 * @param args
	 *            the program arguments
	 */
	public static void main(String[] args) {
		System.out.println("================================");
		System.out.println("= BenchmarkRunner              =");
		System.out.println("================================");
		Benchmark runner = new Benchmark(BenchmarkOptions.USE_WLM1);
		if (BenchmarkOptions.USE_WLM1) {
			runner.run(BenchmarkOptions.BENCHMARKING_REGION,
					BenchmarkOptions.OUTPUT_FOLDER_WM1 + "results",
					BenchmarkOptions.OUTPUT_FOLDER_WM1 + "tasks");
		} else {
			runner.run(BenchmarkOptions.BENCHMARKING_REGION,
					BenchmarkOptions.OUTPUT_FOLDER_WM2 + "results",
					BenchmarkOptions.OUTPUT_FOLDER_WM2 + "tasks");
		}
	}

	/**
	 * Constructor for Benchmark
	 * 
	 * @param wlm1
	 *            whether the first workload model is used for the tasks making
	 *            up the workload for which the broker will be used during the
	 *            benchmarking
	 */
	public Benchmark(final boolean wlm1) {
		if (wlm1) {
			FolderUtils.removeFilesInFolder(BenchmarkOptions.FOLDER_WM1);
			FolderUtils.removeFilesInFolder(BenchmarkOptions.OUTPUT_FOLDER_WM1);
		} else {
			FolderUtils.removeFilesInFolder(BenchmarkOptions.FOLDER_WM2);
			FolderUtils.removeFilesInFolder(BenchmarkOptions.OUTPUT_FOLDER_WM2);
		}
		FolderUtils.removeFilesAndFoldersInFolder("output/spotmodel/");
	}

	/**
	 * Run the benchmarking of the broker for the given geographical region. The
	 * benchmark loops over all possible instance types and operating systems,
	 * and all the possible scheduling and allocation options. For every
	 * combination the broker is run for a number of workload, according to the
	 * benchmarking version used.
	 * 
	 * @param reg
	 *            the geographical region to use while benchmarking the broker
	 * @param outputResults
	 *            file to store the benchmarking results (cost and timing) in.
	 * @param outputTasks
	 *            file to store the specification of the workload used during
	 *            the benchmarking in
	 */
	public void run(final String reg, final String outputResults,
			final String outputTasks) {
		RandomWorkloadGenerator workloadGenerator = new RandomWorkloadGenerator(
				BenchmarkOptions.FOLDER_WM1, BenchmarkOptions.FOLDER_WM2,
				BenchmarkOptions.startDate,
				BenchmarkOptions.PROBABILITY_SPOT_ALLOWED_TASK,
				BenchmarkOptions.PROBABILITY_LATE_DEADLINE_TASK);
		try {
			DecimalFormat f = new DecimalFormat("###.##");
			Set<String> instances = EC2Specifications.getInstanceTypeMapping()
					.keySet();
			Set<Entry<String, String>> osEntries = EC2Specifications
					.getOSMapping().entrySet();
			int totalCounter = instances.size()
					* osEntries.size()
					* (BenchmarkOptions.MAX_NUMBER_OF_TASKS / BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN)
					* InputVerifier.getSchedulingVersions().size()
					* InputVerifier.getAllocationsVersions().size();
			int counter = 1;
			for (String inst : instances) {
				for (Entry<String, String> os : osEntries) {
					csvWriterResults = new CsvWriter(outputResults + "(" + reg
							+ "-" + inst + "-" + os.getValue() + ").csv");
					String headerResults[] = { "NumerOfTasks",
							"TypeOfScheduling", "TypeOfAllocation",
							"TotalTiming", "InputPhaseTiming",
							"SchedulingPhaseTiming", "AllocationTiming",
							"OutputTime", "TotalCost", "OnDemandCost",
							"ReservedCost", "SpotCost" };
					csvWriterResults.writeRecord(headerResults);
					csvWriterResults.flush();
					csvWriterTasks = new CsvWriter(outputTasks + "(" + reg
							+ "-" + inst + "-" + os.getValue() + ").csv");
					String headerTasks[] = { "TaskName", "Region", "Instance",
							"OS", "Deadline", "Length", "SpotAllowed" };
					csvWriterTasks.writeRecord(headerTasks);
					csvWriterTasks.flush();
					for (int i = 0; i < BenchmarkOptions.MAX_NUMBER_OF_TASKS; i++) {
						if (BenchmarkOptions.USE_WLM1) {
							boolean success = false;
							if (!BenchmarkOptions.BENCH_ALTER) {
								for (int j = 0; j < BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN; j++) {
									taskArr.add(workloadGenerator
											.addRandomWM1Task(reg, inst,
													os.getKey()));
								}
							}
							while (success == false) {
								try {
									if (BenchmarkOptions.BENCH_ALTER) {
										for (int j = 0; j < BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN; j++) {
											taskArr.add(workloadGenerator
													.addRandomWM1Task(reg,
															inst, os.getKey()));
										}
									}
									this.runBroker(BenchmarkOptions.FOLDER_WM1,
											SchedulerOptions.SPOTOPTIMIZED,
											AllocatorOptions.SPOT, i);
									success = true;
									this.writeTasksToFile();
									for (String schedulingVersion : InputVerifier
											.getSchedulingVersions()) {
										for (String allocationVersion : InputVerifier
												.getAllocationsVersions()) {
											this.runBroker(
													BenchmarkOptions.FOLDER_WM1,
													schedulingVersion,
													allocationVersion, i);
											this.writeResultsToFile(
													schedulingVersion,
													allocationVersion, i);
											System.err
													.println(counter
															+ "/"
															+ totalCounter
															+ " done! ("
															+ f.format((((double) counter / (double) totalCounter) * 100))
															+ "%)");
											counter++;
										}
									}
								} catch (InstanceNotAvailableException e) {
									if (!BenchmarkOptions.BENCH_ALTER) {
										for (int j = 0; j < BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN; j++) {
											taskArr.add(workloadGenerator
													.modifyCurrentWM1Task(reg,
															inst, os.getKey()));
										}
									}
								}
							}
						} else {
							boolean success = false;
							if (!BenchmarkOptions.BENCH_ALTER) {
								for (int j = 0; j < BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN; j++) {
									taskArr.add(workloadGenerator
											.addRandomWM2Task(reg, inst,
													os.getKey()));
								}
							}
							while (success == false) {
								try {
									if (BenchmarkOptions.BENCH_ALTER) {
										for (int j = 0; j < BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN; j++) {
											taskArr.add(workloadGenerator
													.addRandomWM2Task(reg,
															inst, os.getKey()));
										}
									}
									this.runBroker(BenchmarkOptions.FOLDER_WM2,
											SchedulerOptions.SPOTOPTIMIZED,
											AllocatorOptions.SPOT, i);
									success = true;
									this.writeTasksToFile();
									for (String schedulingVersion : InputVerifier
											.getSchedulingVersions()) {
										for (String allocationVersion : InputVerifier
												.getAllocationsVersions()) {
											this.runBroker(
													BenchmarkOptions.FOLDER_WM2,
													schedulingVersion,
													allocationVersion, i);
											this.writeResultsToFile(
													schedulingVersion,
													allocationVersion, i);
											System.err
													.println(counter
															+ "/"
															+ totalCounter
															+ " done! ("
															+ f.format((((double) counter / (double) totalCounter) * 100))
															+ "%)");
											counter++;
										}
									}
								} catch (InstanceNotAvailableException e) {
									if (!BenchmarkOptions.BENCH_ALTER) {
										for (int j = 0; j < BenchmarkOptions.NUMBER_OF_TASKS_PER_RUN; j++) {
											taskArr.add(workloadGenerator
													.modifyCurrentWM2Task(reg,
															inst, os.getKey()));
										}
									}
								}
							}
						}
						if (BenchmarkOptions.BENCH_ALTER) {
							if (BenchmarkOptions.USE_WLM1) {
								FolderUtils
										.removeFilesInFolder(BenchmarkOptions.FOLDER_WM1);
								workloadGenerator.reset();
							} else {
								FolderUtils
										.removeFilesInFolder(BenchmarkOptions.FOLDER_WM2);
								workloadGenerator.reset();
							}
						}
					}
					csvWriterResults.close();
					csvWriterTasks.close();
					counter++;
					if (!BenchmarkOptions.BENCH_ALTER) {
						if (BenchmarkOptions.USE_WLM1) {
							FolderUtils
									.removeFilesInFolder(BenchmarkOptions.FOLDER_WM1);
							workloadGenerator.reset();
						} else {
							FolderUtils
									.removeFilesInFolder(BenchmarkOptions.FOLDER_WM2);
							workloadGenerator.reset();
						}
					}
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}

	}

	/**
	 * A single run of the broker, during this run the cost and timing metrics
	 * are monitored. Every brokering phase (input, scheduling, allocation,
	 * output) is timed and the cost results are written to file as well.
	 * 
	 * @param inputDir
	 *            location where the task specification files of the workload
	 *            that has to be presented to the broker can be found.
	 * @param schedulingType
	 *            the scheduling option the broker has to use
	 * @param allocationType
	 *            the allocation scheme option the broker has to use
	 * @param numberOfTasks
	 *            the number of tasks in the workload presented to the broker
	 * @throws InstanceNotAvailableException
	 *             exception thrown when one of the input options is not
	 *             supported by the broker according to the input verifier
	 */
	public synchronized void runBroker(final String inputDir,
			final String schedulingType, final String allocationType,
			final int numberOfTasks) throws InstanceNotAvailableException {
		timer = new ExecutionTimer();
		// INPUT
		if (BenchmarkOptions.USE_SYSOUT) {
			System.out.println("INPUT [START]");
		}
		timer.startTiming(BenchmarkOptions.PHASE_INPUT);
		TaskReader reader = new TaskReader();
		TaskCollection coll;
		try {
			coll = reader.parseInputDir(inputDir, BenchmarkOptions.startDate);
			PricingManager.setup();
			timer.stopTiming(BenchmarkOptions.PHASE_INPUT);
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("INPUT [END]");
			}
			// SCHEDULING
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("SCHEDULING [START]");
			}
			timer.startTiming(BenchmarkOptions.PHASE_SCHEDULING);
			RegionAllocator regionAllocator = new RegionAllocator(coll);
			List<RegionWorkloadCollection> workloadCollections = regionAllocator
					.allocateRegions();
			SchedulerResultCollection results = new SchedulerResultCollection(
					BenchmarkOptions.startDate, coll.getLatestDeadline());
			for (RegionWorkloadCollection regionWorkloadCollection : workloadCollections) {
				results.addAll(regionWorkloadCollection.schedule(
						BenchmarkOptions.USE_WLM1, schedulingType));
			}
			timer.stopTiming(BenchmarkOptions.PHASE_SCHEDULING);
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("SCHEDULING [END]");
			}
			// ALLOCATION
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("ALLOCATION [START]");
			}
			timer.startTiming(BenchmarkOptions.PHASE_ALLOCATION);
			Allocator pricingModelDeterminator = new Allocator(
					BenchmarkOptions.USE_WLM1);
			SchedulerResultCollection resultsWithPrices = pricingModelDeterminator
					.allocate(allocationType, results);
			timer.stopTiming(BenchmarkOptions.PHASE_ALLOCATION);
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("ALLOCATION [END]");
			}
			// OUTPUT
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("OUTPUT [START]");
			}
			timer.startTiming(BenchmarkOptions.PHASE_OUTPUT);
			PriceCalculator calc = new PriceCalculator(results);
			tree = calc.createTree(PriceCalculator.PRICINGMODEL,
					PriceCalculator.INSTANCETYPE, PriceCalculator.REGION,
					PriceCalculator.OS);
			timer.stopTiming(BenchmarkOptions.PHASE_OUTPUT);
			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("OUTPUT [END]");
			}

			if (BenchmarkOptions.USE_GUI) {
				ScheduleGUI gui = new ScheduleGUI();
				gui.render(resultsWithPrices);

				Scanner sc = new Scanner(System.in);
				String answer = "";
				while (!(answer.equalsIgnoreCase("y") || answer
						.equalsIgnoreCase("yes"))) {
					System.out.print("type y/yes to continue ...");
					answer = sc.next();
				}
			}

		} catch (InvalidInputException e) {
			e.printStackTrace();
		} catch (UnknownObjectException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Write the cost and timing results of a single run of the broker to file.
	 * 
	 * @param schedulingType
	 *            the scheduling option the broker has used
	 * @param allocationType
	 *            the allocation scheme option the broker has used
	 * @param numberOfTasks
	 *            the number of tasks of the workload presented to the broker
	 */
	public void writeResultsToFile(final String schedulingType,
			final String allocationType, final int numberOfTasks) {
		try {
			// BENCHMARK OUTPUT
			String totalTime = String.valueOf(timer
					.getTotalElapsedTimeInSeconds());
			String inputTime = String.valueOf(timer
					.getElapsedTimeInSeconds(BenchmarkOptions.PHASE_INPUT));
			String schedulingTime = String
					.valueOf(timer
							.getElapsedTimeInSeconds(BenchmarkOptions.PHASE_SCHEDULING));
			String allocationTime = String
					.valueOf(timer
							.getElapsedTimeInSeconds(BenchmarkOptions.PHASE_ALLOCATION));
			String outputTime = String.valueOf(timer
					.getElapsedTimeInSeconds(BenchmarkOptions.PHASE_OUTPUT));

			String totalPrice = String.valueOf(tree.getPrice());
			String ondemandPrice = "0.0";
			String reservedPrice = "0.0";
			String spotPrice = "0.0";
			List<PriceDescriptorNode> children = tree.getChildren();
			for (PriceDescriptorNode priceDescriptorNode : children) {
				if (priceDescriptorNode.getDescription().equalsIgnoreCase(
						EC2Constants.ONDEMAND)) {
					ondemandPrice = String.valueOf(priceDescriptorNode
							.getPrice());
				} else if (priceDescriptorNode.getDescription()
						.equalsIgnoreCase(EC2Constants.RESERVED)) {
					reservedPrice = String.valueOf(priceDescriptorNode
							.getPrice());
				} else if (priceDescriptorNode.getDescription()
						.equalsIgnoreCase(EC2Constants.SPOT)) {
					spotPrice = String.valueOf(priceDescriptorNode.getPrice());
				}
			}

			if (BenchmarkOptions.USE_SYSOUT) {
				System.out.println("Total Time In Seconds = " + totalTime);
				System.out.println("Input Phase In Seconds = " + inputTime);
				System.out.println("Scheduling Phase In Seconds = "
						+ schedulingTime);
				System.out.println("Allocation Phase In Seconds = "
						+ allocationTime);
				System.out.println("Output Phase In Seconds = " + outputTime);

				System.out.println("Total Price In Dollars = " + totalPrice);
				System.out.println("OnDemand Price In Dollars = "
						+ ondemandPrice);
				System.out.println("Reserved Price In Dollars = "
						+ reservedPrice);
				System.out.println("Spot Price In Dollars = " + spotPrice);
			}

			String resultsRec[] = { String.valueOf(numberOfTasks),
					schedulingType, allocationType, totalTime, inputTime,
					schedulingTime, allocationTime, outputTime, totalPrice,
					ondemandPrice, reservedPrice, spotPrice };
			csvWriterResults.writeRecord(resultsRec);
			csvWriterResults.flush();
		} catch (UnknownObjectException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Write the task information of the workload used during a single run of
	 * the broker to file.
	 */
	public void writeTasksToFile() {
		try {
			for (String[] rec : taskArr) {
				csvWriterTasks.writeRecord(rec);
			}
			csvWriterTasks.flush();
			taskArr.clear();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
