package be.ac.ua.broker.evaluation;

import java.util.Date;

import be.ac.ua.broker.input.tasks.generation.wm1.TaskHoursGenerator;
import be.ac.ua.broker.input.tasks.generation.wm2.TaskGenerator;
import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.RandomGenerator;

/**
 * Class used to generate semi-random tasks that make up the workload for a
 * benchmark run, semi in the sense that certain variables can be set and
 * certain properties are not random but take a certain value with a given
 * probability.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class RandomWorkloadGenerator {

	/**
	 * Location where the task specification for workload model one tasks are
	 * stored.
	 */
	private String FOLDER_WM1 = "";

	/**
	 * Location where the task specification for workload model two tasks are
	 * stored.
	 */
	private String FOLDER_WM2 = "";

	/**
	 * The start date of the one-year period in which the task will get
	 * scheduled.
	 */
	private Date startDate;

	/**
	 * Counter for the number of workload model one tasks that have been
	 * generated during the current benchmark run.
	 */
	private Integer wm1Counter = 0;

	/**
	 * Counter for the number of workload model two tasks that have been
	 * generated during the current benchmark run.
	 */
	private Integer wm2Counter = 0;

	/**
	 * Probability the generated task has a deadline falling within the last
	 * week of the scheduling period (of one year), the randomness is biased
	 * because it has to be possible to get utilization ratios that allow
	 * reserved instances to be usefull.
	 */
	private Double probLateDeadline;

	/**
	 * Probability the generated task is allowed to be run on spot instances.
	 */
	private Double probSpotAllowed;

	/**
	 * Constructor for RandomWorkloadGenerator
	 * 
	 * @param fOLDERWM1
	 *            the location for the workload model one tasks
	 * @param fOLDERWM2
	 *            the location for the workload model two tasks
	 * @param start
	 *            the start date of the one-year period in which the generated
	 *            tasks will be scheduled
	 * @param pROBABILITYSPOTALLOWEDTASK
	 *            probability the generated task is allowed to be run on spot
	 *            instances
	 * @param pROBABILITYLATEDEADLINETASK
	 *            probability the generated task has a deadline falling in the
	 *            last week of the one-year scheduling period.
	 */
	public RandomWorkloadGenerator(String fOLDERWM1, String fOLDERWM2,
			Date start, Double pROBABILITYSPOTALLOWEDTASK,
			Double pROBABILITYLATEDEADLINETASK) {
		FOLDER_WM1 = fOLDERWM1;
		FOLDER_WM2 = fOLDERWM2;
		startDate = start;
		probLateDeadline = pROBABILITYLATEDEADLINETASK;
		probSpotAllowed = pROBABILITYSPOTALLOWEDTASK;
	}

	/**
	 * Reset the generator, start renumbering the generated tasks.
	 */
	public void reset() {
		wm1Counter = 0;
		wm2Counter = 0;
	}

	/**
	 * Add a randomly generated workload model one task to the existing
	 * workload.
	 * 
	 * @param reg
	 *            the geographical region to generate a task for
	 * @param inst
	 *            the instance type to generate a task for
	 * @param os
	 *            the operating system to generate a task for
	 * @return the string array description of the generated task
	 */
	public String[] addRandomWM1Task(final String reg, final String inst,
			final String os) {
		Date endDate = new Date(this.getRandomDeadline());
		int taskLength = RandomGenerator.generateRandomInt(1, 2400);
		TaskHoursGenerator hoursGenerator = new TaskHoursGenerator(FOLDER_WM1,
				"benchTask" + wm1Counter, "benchmarking task", inst, reg, os,
				DateUtility.dateToString(endDate), String.valueOf(taskLength),
				this.getRandomSpotAllowed());
		System.out.println("> task " + wm1Counter + " generated.");
		wm1Counter++;
		return hoursGenerator.getArrayDescription();
	}

	/**
	 * Add a randomly generated workload model two task to the existing
	 * workload.
	 * 
	 * @param reg
	 *            the geographical region to generate a task for
	 * @param inst
	 *            the instance type to generate a task for
	 * @param os
	 *            the operating system to generate a task for
	 * @return the string array description of the generated task
	 */
	public String[] addRandomWM2Task(final String reg, final String inst,
			final String os) {
		Date endDate = new Date(this.getRandomDeadline());
		int taskLength = RandomGenerator.generateRandomInt(1,
				(DateUtility.getNumberOfHoursBetween(startDate, endDate) - 1));
		TaskGenerator generator = new TaskGenerator(FOLDER_WM2, "benchTask"
				+ wm2Counter, "benchmarking task", inst, reg, os,
				DateUtility.dateToString(endDate), String.valueOf(taskLength),
				FOLDER_WM2 + "benchWorkload" + wm2Counter + ".csv",
				this.getRandomSpotAllowed());
		System.out.println("> task " + wm2Counter + " generated.");
		wm2Counter++;
		return generator.getArrayDescription();
	}

	/**
	 * Modify the randomly generated workload model one task corresponding to
	 * the current task number, it will get a new deadline and task length
	 * assigned.
	 * 
	 * @param reg
	 *            the geographical region of the generated task
	 * @param inst
	 *            the instance type of the generated task
	 * @param os
	 *            the operating system of the generated task
	 * @return the string array description of the modified task
	 */
	public String[] modifyCurrentWM1Task(String reg, String inst, String os) {
		Date endDate = new Date(this.getRandomDeadline());
		int taskLength = RandomGenerator.generateRandomInt(1, 2400);
		TaskHoursGenerator hoursGenerator = new TaskHoursGenerator(FOLDER_WM1,
				"benchTask" + (wm1Counter - 1), "benchmarking task", inst, reg,
				os, DateUtility.dateToString(endDate),
				String.valueOf(taskLength), String.valueOf(RandomGenerator
						.generateRandomBoolean()));
		System.out.println("> task " + (wm1Counter - 1) + " generated.");
		return hoursGenerator.getArrayDescription();
	}

	/**
	 * Modify the randomly generated workload model two task corresponding to
	 * the current task number, it will get a new deadline, task length, and
	 * load distribution assigned.
	 * 
	 * @param reg
	 *            the geographical region of the generated task
	 * @param inst
	 *            the instance type of the generated task
	 * @param os
	 *            the operating system of the generated task
	 * @return the string array description of the modified task
	 */
	public String[] modifyCurrentWM2Task(String reg, String inst, String os) {
		Date endDate = new Date(this.getRandomDeadline());
		int taskLength = RandomGenerator.generateRandomInt(1, 2400);
		TaskGenerator generator = new TaskGenerator(FOLDER_WM2, "benchTask"
				+ (wm2Counter - 1), "benchmarking task", inst, reg, os,
				DateUtility.dateToString(endDate), String.valueOf(taskLength),
				FOLDER_WM2 + "benchWorkload" + (wm2Counter - 1) + ".csv",
				String.valueOf(RandomGenerator.generateRandomBoolean()));
		System.out.println("> task " + (wm2Counter - 1) + " generated.");
		return generator.getArrayDescription();
	}

	/**
	 * Create a semi-random deadline for a generated task, it falls within the
	 * last week of the one-year scheduling period with a given probability.
	 * 
	 * @return the semi-randomly generated deadline
	 */
	private long getRandomDeadline() {
		Long randomDL = startDate.getTime()
				+ RandomGenerator.generateRandomInt(0, 8760)
				* DateUtility.HOURINMILLISECONDS;
		double nr = 1.0 / probLateDeadline;
		int intNr = (int) Math.round(Math.ceil(nr));
		int random = RandomGenerator.generateRandomInt(1, intNr);
		if (random == 1) {
			// Not one week consists of 168 hours
			randomDL = startDate.getTime()
					+ RandomGenerator.generateRandomInt(8592, 8760)
					* DateUtility.HOURINMILLISECONDS;
		}
		return randomDL;
	}

	/**
	 * Create a semi-random decision whether the generated task is allowed to be
	 * run on spot instances.
	 * 
	 * @return the semi-randomly decision whether the generated task is allowed
	 *         to be run on spot instances
	 */
	private String getRandomSpotAllowed() {
		String returnString = "false";
		double nr = 1.0 / probSpotAllowed;
		int intNr = (int) Math.round(Math.ceil(nr));
		int random = RandomGenerator.generateRandomInt(1, intNr);
		if (random == 1) {
			returnString = "true";
		}
		return returnString;
	}

}
