package be.ac.ua.broker.input.pricing;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.InstanceNotAvailableException;

import com.csvreader.CsvReader;

/**
 * Class that uses java CSV reader to read in the data from the CSV
 * file downloaded from the cloudexchange.org website
 * 
 * @see http://sourceforge.net/projects/javacsv/
 * @author Vermeersch Kurt
 * @date October 2010
 */
public class CSVHandler {

	/** CSVReader instance */
	private CsvReader fReader;

	/** List of list in which the contents of the CSV file are stored */
	protected List<List<String>> fContents = new ArrayList<List<String>>();

	/**
	 * Parse the given CSV file and store its contents
	 * 
	 * @param inputFile
	 *            the input CSV file to parse and to analyze the content of
	 */
	public void parse(final String inputFile) {
		try {
			fReader = new CsvReader(inputFile);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		try {
			if (fReader != null) {
				fContents = new ArrayList<List<String>>();
				while (fReader.readRecord()) {
					int cols = fReader.getColumnCount();
					List<String> content = new ArrayList<String>();
					for (int i = 0; i < cols; i++) {
						content.add(fReader.get(i));
					}
					fContents.add(content);
				}
				fReader.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Get the price from the CSV file that is stated for the given instance
	 * type, geographical region and operating system.
	 * 
	 * @param inst
	 *            the instance type to get the price for
	 * @param reg
	 *            the geographical region to get the price for
	 * @param os
	 *            the operating system to get the price for
	 * @return the price for an instance with the provided parameter values.
	 * @throws InstanceNotAvailableException
	 */
	public String getValue(String inst, String reg, String os)
			throws InstanceNotAvailableException {
		String returnValue = EC2Constants.NOTAVAILABLE;
		List<String> firstRow = fContents.get(0);
		List<String> secondRow = fContents.get(1);
		int columnIndex = -1;
		for (int i = 0; i < firstRow.size(); i++) {
			if (firstRow.get(i).equalsIgnoreCase(reg)
					&& secondRow.get(i).equalsIgnoreCase(os)) {
				columnIndex = i;
				break;
			}
		}
		if (columnIndex != -1) {
			for (List<String> l : fContents) {
				if (l.get(0).equalsIgnoreCase(inst)) {
					returnValue = l.get(columnIndex);
					break;
				}
			}
		}
		if (returnValue.equalsIgnoreCase(EC2Constants.NOTAVAILABLE)
				|| columnIndex == -1) {
			throw new InstanceNotAvailableException(inst, reg, os);
		}
		return returnValue;
	}

	/**
	 * Function that generates a list of statistics object using the contents
	 * that were found in the input CSV file
	 * 
	 * @param filter
	 *            filter that decides whether a certain data item is still part
	 *            of the filtered contents
	 * @return a list of statistics object
	 */
	public DescriptiveStatistics getStatistics(CSVFilterInterface filter) {
		DescriptiveStatistics statsAll = new DescriptiveStatistics();
		for (int i = 0; i < fContents.size(); i++) {
			List<String> record = fContents.get(i);
			String str = record.get(filter.getColumn());
			if (filter.keepValue(str)) {
				statsAll.addValue(Double.valueOf(record.get(1)));
			}
		}
		return statsAll;
	}

	/**
	 * Get the spot price value that is the tipping point bid that enables the
	 * optimal scheduling of the subtasks
	 * 
	 * @param filter
	 *            the date filter of the time frame in which the spot prices
	 *            need to be analyzed
	 * @param index
	 *            number of hours the spot price does not need to be available
	 * @return the spot price value
	 * @throws Exception
	 */
	public Double getHighValue(CSVFilterInterface filter, int index)
			throws Exception {
		Map<Double, Double> statsAll = new HashMap<Double, Double>();
		Double prevVal = null;
		Date prevDate = null;
		Double latestPrice = null;
		for (int i = 0; i < fContents.size(); i++) {
			List<String> record = fContents.get(i);
			String str = record.get(filter.getColumn());
			if (filter.keepValue(str)) {
				double val = Double.valueOf(record.get(1));
				Date st = DateUtility.toDate(record.get(0));
				if (prevDate != null && prevVal != null) {
					Double hours = 0.0;
					if (statsAll.get(prevVal) != null) {
						hours = statsAll.get(prevVal);
					}
					statsAll.put(
							prevVal,
							DateUtility.getDoubleNumberOfHoursBetween(prevDate,
									st) + hours);
				} else if (latestPrice != null) {
					statsAll.put(latestPrice, DateUtility
							.getDoubleNumberOfHoursBetween(
									((DateFilter) filter).getStartDate(), st));
				}
				prevVal = val;
				prevDate = st;
			} else {
				latestPrice = Double.valueOf(record.get(1));
			}
		}
		if (prevDate != null && prevVal != null) {
			Double hours = 0.0;
			if (statsAll.get(prevVal) != null) {
				hours = statsAll.get(prevVal);
			}
			statsAll.put(
					prevVal,
					hours
							+ DateUtility.getDoubleNumberOfHoursBetween(
									prevDate,
									((DateFilter) filter).getEndDate()));
		}
		ArrayList<Double> keys = new ArrayList<Double>(statsAll.keySet());
		Collections.sort(keys);
		Collections.reverse(keys);
		double val = 0.0;
		for (Double double1 : keys) {
			Double nrHours = statsAll.get(double1);
			val += nrHours;
			if (val > index) {
				return double1;
			}
		}
		throw new Exception("highest value not found");
	}

}
