package be.ac.ua.broker.input.pricing;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import be.ac.ua.broker.utils.InstanceNotAvailableException;

/**
 * This class provides access to the EC2 on-demand prices that are inputted
 * through CSV files to the broker
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class EC2OnDemandPriceWatch implements PriceWatch {

	/**
	 * Location of the input files containing the on-demand prices.
	 */
	public static String ONDEMAND_INPUT = "input/pricing/on-demand/ondemand.csv";

	/**
	 * Caching of instance identifier to the corresponding on-demand price, such
	 * that the price does not need to be read from file every time it is
	 * requested
	 */
	private Map<String, OnDemandPrice> cache = new HashMap<String, OnDemandPrice>();

	/**
	 * The CSV handler to use to read on-demand prices from file.
	 */
	private CSVHandler handler = new CSVHandler();

	/**
	 * Constructor for EC2OnDemandPriceWatch
	 */
	public EC2OnDemandPriceWatch() {
		this.update();
	}

	/**
	 * Constructor for EC2OnDemandPriceWatch
	 * 
	 * @param inputLocation
	 *            the location of the input files containing the on-demand
	 *            prices.
	 */
	public EC2OnDemandPriceWatch(String inputLocation) {
		ONDEMAND_INPUT = inputLocation;
		this.update();
	}

	@Override
	public OnDemandPrice getPrice(InstanceSpecification inst)
			throws InstanceNotAvailableException {
		String id = inst.getIdentifier();
		if (cache.get(id) == null) {
			String val = handler.getValue(inst.getInstance(), inst.getRegion(),
					inst.getOS());
			OnDemandPrice priceObj = new OnDemandPrice(Double.valueOf(val));
			cache.put(id, priceObj);
		}
		return cache.get(id);
	}

	@Override
	public void update() {
		cache.clear();
		handler.parse(ONDEMAND_INPUT);
	}

	@Override
	public String getPricingModel() {
		return EC2Constants.ONDEMAND;
	}

	@Override
	public Price getPrice(InstanceSpecification inst, Date timestamp)
			throws InstanceNotAvailableException {
		return this.getPrice(inst);
	}

}
