package be.ac.ua.broker.input.pricing;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import be.ac.ua.broker.utils.InstanceNotAvailableException;

/**
 * This class provides access to the EC2 reserved prices that are inputted
 * through CSV files to the broker
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class EC2ReservedPriceWatch implements PriceWatch {

	/**
	 * Location of the input files containing the reserved prices.
	 */
	public static String RESERVED_INPUT = "input/pricing/reserved/reserved.csv";

	/**
	 * Caching of instance identifier to the corresponding reserved price, such
	 * that the price does not need to be read from file every time it is
	 * requested
	 */
	private Map<String, ReservedPrice> cache = new HashMap<String, ReservedPrice>();

	/**
	 * The CSV handler to use to read reserved prices from file.
	 */
	private CSVHandler handler = new CSVHandler();

	/**
	 * Constructor for EC2ReservedPriceWatch
	 */
	public EC2ReservedPriceWatch() {
		this.update();
	}

	/**
	 * Constructor for EC2ReservedPriceWatch
	 * 
	 * @param inputLocation
	 *            the location of the input files containing the reserved
	 *            prices.
	 */
	public EC2ReservedPriceWatch(String inputLocation) {
		RESERVED_INPUT = inputLocation;
		this.update();
	}

	@Override
	public ReservedPrice getPrice(InstanceSpecification inst)
			throws InstanceNotAvailableException {
		String id = inst.getIdentifier();
		if (cache.get(id) == null) {
			String val = handler.getValue(inst.getInstance(), inst.getRegion(),
					inst.getOS());
			String val2 = handler.getValue(inst.getInstance(),
					inst.getRegion(), "Fixed");
			ReservedPrice priceObj = new ReservedPrice(Double.valueOf(val),
					Double.valueOf(val2));
			cache.put(id, priceObj);
		}
		return cache.get(id);
	}

	@Override
	public void update() {
		cache.clear();
		handler.parse(RESERVED_INPUT);
	}

	@Override
	public String getPricingModel() {
		return EC2Constants.RESERVED;
	}

	@Override
	public Price getPrice(InstanceSpecification inst, Date timestamp)
			throws InstanceNotAvailableException {
		return this.getPrice(inst);
	}

}
