package be.ac.ua.broker.input.pricing;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import be.ac.ua.broker.utils.InvalidInputException;

/**
 * This class provides the EC2 instance types, geographical regions and
 * operating systems that are supported by our broker.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class EC2Specifications {

	/**
	 * Mapping of the operating system identifier to the official EC2 operating
	 * system string representation.
	 */
	private static Map<String, String> mapOS = new HashMap<String, String>();

	/**
	 * Mapping of the geographical region identifier to the official EC2
	 * geographical region string representation.
	 */
	private static Map<String, String> mapRegions = new HashMap<String, String>();

	/**
	 * Mapping of the instance type identifier to the official EC2 instance type
	 * string representation.
	 */
	private static Map<String, String> mapInstanceTypes = new HashMap<String, String>();

	/**
	 * A list of all the possible instance specifications using the above
	 * mentioned instance types, geographical regions and operating systems. For
	 * the cluster-compute and cluster-gpu instance types only the us-east linux
	 * instances are added to this list.
	 */
	private static List<InstanceSpecification> instances = new ArrayList<InstanceSpecification>();

	/**
	 * Initialize all the instance types, geographical regions and operating
	 * systems mappings.
	 */
	public static void init() {
		// Operating Systems
		mapOS.put(EC2Constants.WINDOWS, "windows");
		mapOS.put(EC2Constants.LINUX, "linux");
		// Regions
		mapRegions.put(EC2Constants.ASIANORTH, "ap-northeast-1");
		mapRegions.put(EC2Constants.ASIASOUTH, "ap-southeast-1");
		mapRegions.put(EC2Constants.EU, "eu-west-1");
		mapRegions.put(EC2Constants.USEAST, "us-east-1");
		mapRegions.put(EC2Constants.USWEST, "us-west-1");
		// Instance Types
		mapInstanceTypes.put(EC2Constants.STANDARD_SMALL, "m1.small");
		mapInstanceTypes.put(EC2Constants.STANDARD_LARGE, "m1.large");
		mapInstanceTypes.put(EC2Constants.STANDARD_EXTRALARGE, "m1.xlarge");
		mapInstanceTypes.put(EC2Constants.MICRO_MICRO, "t1.micro");
		mapInstanceTypes.put(EC2Constants.HIGHMEMORY_EXTRALARGE, "m2.xlarge");
		mapInstanceTypes.put(EC2Constants.HIGHMEMORY_DOUBLEEXTRALARGE,
				"m2.2xlarge");
		mapInstanceTypes.put(EC2Constants.HIGHMEMORY_QUADRUPLEEXTRALARGE,
				"m2.4xlarge");
		mapInstanceTypes.put(EC2Constants.HIGHCPU_MEDIUM, "c1.medium");
		mapInstanceTypes.put(EC2Constants.HIGHCPU_EXTRALARGE, "c1.xlarge");
		mapInstanceTypes.put(EC2Constants.CLUSTERCOMPUTE_QUADRUPLEEXTRALARGE,
				"cc1.4xlarge");
		mapInstanceTypes.put(EC2Constants.CLUSTERGPU_QUADRUPLEEXTRALARGE,
				"cg1.4xlarge");
	}

	/**
	 * Initialize the list of possible instance specifications. For the
	 * cluster-compute and cluster-gpu instance types only the us-east linux
	 * instances are added to this list.
	 */
	public static void initInstances() {
		for (Entry<String, String> itemInst : mapInstanceTypes.entrySet()) {
			for (Entry<String, String> itemReg : mapRegions.entrySet()) {
				for (Entry<String, String> itemOS : mapOS.entrySet()) {
					boolean isInstance = true;
					if (itemInst.getKey().equals(
							EC2Constants.CLUSTERCOMPUTE_QUADRUPLEEXTRALARGE)
							|| itemInst
									.getKey()
									.equals(EC2Constants.CLUSTERGPU_QUADRUPLEEXTRALARGE)) {
						isInstance = false;
						if (itemOS.getKey().equals(EC2Constants.LINUX)
								&& itemReg.getKey().equals(EC2Constants.USEAST)) {
							isInstance = true;
						}
					}
					if (isInstance) {
						try {
							instances.add(new InstanceSpecification(itemInst
									.getKey(), itemOS.getKey(), itemReg
									.getKey()));
						} catch (InvalidInputException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
	}

	/**
	 * @return the list of possible instance specifications (note that not all
	 *         possibilities for the cluster-compute and cluster-gpu instance
	 *         types are included)
	 */
	public static List<InstanceSpecification> getInstanceSpecifications() {
		if (instances.isEmpty()) {
			EC2Specifications.initInstances();
		}
		return instances;
	}

	/**
	 * @return the operating system mapping
	 */
	public static Map<String, String> getOSMapping() {
		EC2Specifications.checkInitialization();
		return mapOS;
	}

	/**
	 * @return the geographical region mapping
	 */
	public static Map<String, String> getRegionMapping() {
		EC2Specifications.checkInitialization();
		return mapRegions;
	}

	/**
	 * @return the instance type mapping
	 */
	public static Map<String, String> getInstanceTypeMapping() {
		EC2Specifications.checkInitialization();
		return mapInstanceTypes;
	}

	/**
	 * Check whether the mappings are initialized.
	 */
	private static void checkInitialization() {
		if (mapInstanceTypes.isEmpty() || mapRegions.isEmpty()
				|| mapOS.isEmpty()) {
			EC2Specifications.init();
		}
	}

}
