package be.ac.ua.broker.input.pricing;

import java.util.Date;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import be.ac.ua.broker.utils.InstanceNotAvailableException;

/**
 * This class provides access to the EC2 spot prices that are inputted through
 * CSV files to the broker
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class EC2SpotPriceWatch implements PriceWatch {

	/**
	 * Location of the input files containing the spot price history.
	 */
	public static String SPOT_INPUT = "input/pricing/spot/";

	/**
	 * The CSV handler to use to read prices from file.
	 */
	private SpotCSVHandler handler = null;

	/**
	 * Constructor for EC2SpotPriceWatch
	 */
	public EC2SpotPriceWatch() {
		handler = new SpotCSVHandler(SPOT_INPUT);
	}

	/**
	 * Constructor for EC2SpotPriceWatch
	 * 
	 * @param inputLocation
	 *            the location of the input files containing the spot price
	 *            history.
	 */
	public EC2SpotPriceWatch(String inputLocation) {
		SPOT_INPUT = inputLocation;
		handler = new SpotCSVHandler(SPOT_INPUT);
	}

	@Override
	public Price getPrice(InstanceSpecification inst)
			throws InstanceNotAvailableException {
		return this.getPrice(inst, new Date());
	}

	@Override
	public void update() {
		handler.update(SPOT_INPUT);
	}

	@Override
	public String getPricingModel() {
		return EC2Constants.SPOT;
	}

	@Override
	public SpotPrice getPrice(InstanceSpecification inst, Date timestamp)
			throws InstanceNotAvailableException {
		String val = handler.getSpotValue(inst, timestamp);
		SpotPrice priceObj = new SpotPrice(Double.valueOf(val));
		return priceObj;
	}

	/**
	 * Getter for the spot price history statistics within the given time frame
	 * for the given instance specification
	 * 
	 * @param inst
	 *            the instance specification to get the spot price statistics
	 *            for
	 * @param start
	 *            the start date of the time frame to get the spot price
	 *            statistics for
	 * @param end
	 *            the end date of the time frame to get the spot price
	 *            statistics for
	 * @return the spot price statistics that were requested for a certain
	 *         instance specification and time frame
	 * @throws InstanceNotAvailableException
	 */
	public DescriptiveStatistics getStatisticsSpotPrice(
			InstanceSpecification inst, Date start, Date end)
			throws InstanceNotAvailableException {
		return handler.getSpotStatistics(inst, start, end);
	}

	/**
	 * Getter for the tipping spot bid price such that all task hours can be run
	 * on the cheapest spot price hours, the number of hours that can be skipped
	 * is given as well as the instance specification and time frame to
	 * determine the spot price tipping point for.
	 * 
	 * @param inst
	 *            the instance specification to get the bid spot price for
	 * @param start
	 *            the start date of the time interval to get the bid spot price
	 *            for
	 * @param end
	 *            the end date of the time interval to get the bid spot price
	 *            for
	 * @param index
	 *            the number of hours that can be skipped, the spot instance
	 *            does not need to be in use
	 * @return the appropriate spot bid price
	 * @throws InstanceNotAvailableException
	 */
	public SpotPrice getTippingPointSpotPrice(InstanceSpecification inst,
			Date start, Date end, int index)
			throws InstanceNotAvailableException {
		SpotPrice priceObj = new SpotPrice(handler.getHighValue(inst, start,
				end, index));
		return priceObj;
	}

}
