package be.ac.ua.broker.input.pricing;

import java.util.Date;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import be.ac.ua.broker.utils.InstanceNotAvailableException;

/**
 * This class provides static access to the on-demand, reserved and spot prices
 * that are provided to the broker using input files. This class redirects calls
 * to the appropriate PriceWatch implementations.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class PricingManager {

	/**
	 * The on-demand PriceWatch implementation used to access on-demand prices.
	 */
	private static EC2OnDemandPriceWatch ONDEMAND = new EC2OnDemandPriceWatch();

	/**
	 * The reserved PriceWatch implementation used to access reserved prices.
	 */
	private static EC2ReservedPriceWatch RESERVED = new EC2ReservedPriceWatch();

	/**
	 * The spot PriceWatch implementation used to access spot prices.
	 */
	private static EC2SpotPriceWatch SPOT = new EC2SpotPriceWatch();

	/**
	 * Initialize/setup the price watches
	 */
	public static void setup() {
		ONDEMAND.update();
		RESERVED.update();
		SPOT.update();
	}

	/**
	 * Get the on-demand price for a given instance specification
	 * 
	 * @param inst
	 *            the instance specification to get the on-demand price for
	 * @return the on-demand price for the provided instance
	 * @throws InstanceNotAvailableException
	 */
	public static OnDemandPrice getOnDemandPrice(InstanceSpecification inst)
			throws InstanceNotAvailableException {
		return ONDEMAND.getPrice(inst);
	}

	/**
	 * Get the reserved price for a given instance specification
	 * 
	 * @param inst
	 *            the instance specification to get the reserved price for
	 * @return the reserved price for the provided instance
	 * @throws InstanceNotAvailableException
	 */
	public static ReservedPrice getReservedPrice(InstanceSpecification inst)
			throws InstanceNotAvailableException {
		return RESERVED.getPrice(inst);
	}

	/**
	 * Get the spot price for a given instance specification at a given moment
	 * in time.
	 * 
	 * @param inst
	 *            the instance specification to get the spot price for
	 * @return the spot price for the provided instance and moment in time
	 * @throws InstanceNotAvailableException
	 */
	public static SpotPrice getSpotPrice(InstanceSpecification inst,
			Date timestamp) throws InstanceNotAvailableException {
		return SPOT.getPrice(inst, timestamp);
	}

	/**
	 * Getter for the spot price history statistics within the given time frame
	 * for the given instance specification
	 * 
	 * @param inst
	 *            the instance specification to get the spot price statistics
	 *            for
	 * @param start
	 *            the start date of the time frame to get the spot price
	 *            statistics for
	 * @param end
	 *            the end date of the time frame to get the spot price
	 *            statistics for
	 * @return the spot price statistics that were requested for a certain
	 *         instance specification and time frame
	 * @throws InstanceNotAvailableException
	 */
	public static DescriptiveStatistics getStatisticsSpotPrice(
			InstanceSpecification inst, Date start, Date end)
			throws InstanceNotAvailableException {
		return SPOT.getStatisticsSpotPrice(inst, start, end);
	}

	/**
	 * Getter for the tipping spot bid price such that all task hours can be run
	 * on the cheapest spot price hours, the number of hours that can be skipped
	 * is given as well as the instance specification and time frame to
	 * determine the spot price tipping point for.
	 * 
	 * @param inst
	 *            the instance specification to get the bid spot price for
	 * @param start
	 *            the start date of the time interval to get the bid spot price
	 *            for
	 * @param end
	 *            the end date of the time interval to get the bid spot price
	 *            for
	 * @param index
	 *            the number of hours that can be skipped, the spot instance
	 *            does not need to be in use
	 * @return the appropriate spot bid price
	 * @throws InstanceNotAvailableException
	 */
	public static SpotPrice getTippingPointSpotPrice(
			InstanceSpecification inst, Date start, Date end, int numberToSkip)
			throws InstanceNotAvailableException {
		return SPOT.getTippingPointSpotPrice(inst, start, end, numberToSkip);
	}

}
