package be.ac.ua.broker.input.pricing;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

/**
 * Class that extends the CSVHandler in order to handle spot price access
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class SpotCSVHandler extends CSVHandler {

	/**
	 * Mapping from a single instance specified by its identifier to the
	 * CSVHandler that handles the spot price access for the instance
	 */
	private Map<String, CSVHandler> map = new HashMap<String, CSVHandler>();

	/**
	 * Constructor for SpotCSVHandler
	 * 
	 * @param inputLocation
	 *            the location of the spot price history input files
	 */
	public SpotCSVHandler(String inputLocation) {
		init(inputLocation);
	}

	/**
	 * Initialize the mappings
	 * 
	 * @param inputLocation
	 *            the location of the spot price history input files
	 */
	private void init(String inputLocation) {
		// Operating Systems
		Map<String, String> mapOS = EC2Specifications.getOSMapping();
		// Regions
		Map<String, String> mapRegions = EC2Specifications.getRegionMapping();
		// Instance Types
		Map<String, String> mapInstanceTypes = EC2Specifications
				.getInstanceTypeMapping();
		// Init
		for (Entry<String, String> itemInst : mapInstanceTypes.entrySet()) {
			for (Entry<String, String> itemReg : mapRegions.entrySet()) {
				for (Entry<String, String> itemOS : mapOS.entrySet()) {
					boolean doParsing = true;
					if (itemInst.getKey().equals(
							EC2Constants.CLUSTERCOMPUTE_QUADRUPLEEXTRALARGE)
							|| itemInst
									.getKey()
									.equals(EC2Constants.CLUSTERGPU_QUADRUPLEEXTRALARGE)) {
						doParsing = false;
						if (itemOS.getKey().equals(EC2Constants.LINUX)
								&& itemReg.getKey().equals(EC2Constants.USEAST)) {
							doParsing = true;
						}
					}
					if (doParsing) {
						try {
							InstanceSpecification instSpec = new InstanceSpecification(
									itemInst.getKey(), itemOS.getKey(),
									itemReg.getKey());
							String inputFile = inputLocation
									+ itemOS.getValue() + "/"
									+ itemReg.getValue() + "/"
									+ itemReg.getValue() + "."
									+ itemOS.getValue() + "."
									+ itemInst.getValue() + ".csv";
							CSVHandler handler = new CSVHandler();
							handler.parse(inputFile);
							map.put(instSpec.getIdentifier(), handler);
						} catch (InvalidInputException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
	}

	/**
	 * Getter for the spot price of the given instance at a certain moment in
	 * time using the pricing model associated with the PriceWatch
	 * implementation
	 * 
	 * @param inst
	 *            the instance specification to get the spot price for
	 * @param timestamp
	 *            time to get the spot price for
	 * @return the price for the given instance at the given moment in time
	 * @throws InstanceNotAvailableException
	 */
	public String getSpotValue(InstanceSpecification inst, Date timestamp)
			throws InstanceNotAvailableException {
		CSVHandler handler = map.get(inst.getIdentifier());
		if (handler != null) {
			String prevValue = null;
			String val = null;
			for (List<String> l : handler.fContents) {
				prevValue = val;
				val = l.get(1);
				Date date = DateUtility.toDate(l.get(0));
				if (date.after(timestamp)) {
					if (prevValue == null) {
						throw new InstanceNotAvailableException(inst);
					}
					return prevValue;
				}
			}
			;
			if (val == null) {
				throw new InstanceNotAvailableException(inst);
			}
			return val;
		}
		throw new InstanceNotAvailableException(inst);
	}

	/**
	 * Update the spot price access, reload the prices form file.
	 * 
	 * @param inputLocation
	 *            the location of the spot price history input files
	 */
	public void update(String inputLocation) {
		this.init(inputLocation);
	}

	/**
	 * Getter for the spot price history statistics within the given time frame
	 * for the given instance specification
	 * 
	 * @param inst
	 *            the instance specification to get the spot price statistics
	 *            for
	 * @param start
	 *            the start date of the time frame to get the spot price
	 *            statistics for
	 * @param end
	 *            the end date of the time frame to get the spot price
	 *            statistics for
	 * @return the spot price statistics that were requested for a certain
	 *         instance specification and time frame
	 * @throws InstanceNotAvailableException
	 */
	public DescriptiveStatistics getSpotStatistics(InstanceSpecification inst,
			Date start, Date end) throws InstanceNotAvailableException {
		CSVHandler handler = map.get(inst.getIdentifier());
		if (handler != null) {
			DateFilter filter = new DateFilter(start, end);
			return handler.getStatistics(filter);
		}
		throw new InstanceNotAvailableException(inst);
	}

	/**
	 * Get the spot price value that is the tipping point bid that enables the
	 * optimal scheduling of the subtasks
	 * 
	 * @param inst
	 *            the instance specification to get the spot bid for
	 * @param start
	 *            the start of the time frame to get the spot bid for
	 * @param end
	 *            the end of the time frame to get the spot bid for
	 * @param index
	 *            number of hours the spot price does not need to be available
	 * @return the requested spot price value
	 * @throws Exception
	 */
	public Double getHighValue(InstanceSpecification inst, Date start,
			Date end, int index) throws InstanceNotAvailableException {
		CSVHandler handler = map.get(inst.getIdentifier());
		try {
			if (handler != null) {
				DateFilter filter = new DateFilter(start, end);
				return handler.getHighValue(filter, index);
			}
		} catch (Exception e) {
			throw new InstanceNotAvailableException(inst);
		}
		throw new InstanceNotAvailableException(inst);
	}

}
