package be.ac.ua.broker.input.tasks;

import java.util.Date;

import org.jfree.data.gantt.Task;
import org.jfree.data.time.SimpleTimePeriod;

import be.ac.ua.broker.input.pricing.Price;

/**
 * This class represents a subtask which is used to represent a single task hour
 * of a task, it is the smallest unit used during the scheduling phase of our
 * broker process. During the allocation phase an actual time frame and
 * corresponding price is associated with the sub task.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class SubTask extends Task {

	/** Serial Version ID */
	private static final long serialVersionUID = 1L;

	/**
	 * The hour of the original task this subtask corresponds to.
	 */
	private int taskHour;

	/**
	 * The specification of the original task this sub task is a part of
	 */
	private TaskSpecification specification;

	/**
	 * The price associated with the subTask during the allocation phase of the
	 * broker process.
	 */
	private Price partPrice;

	/**
	 * The start time of the subTask.
	 */
	private Date start;

	/**
	 * The end time of the subTask
	 */
	private Date end;

	/**
	 * Constructor for SubTask
	 * 
	 * @param hour
	 *            the hour of the original task this subtask corresponds to.
	 * @param spec
	 *            the specification of the original task this sub task is a part
	 *            of
	 */
	public SubTask(int hour, TaskSpecification spec) {
		super(spec.getDescription(), new Date(), new Date());
		taskHour = hour;
		specification = spec;
	}

	/**
	 * Constructor for SubTask
	 * 
	 * @param desc
	 *            only a name is given to the subTask, this possibility is
	 *            provided to bes able to generate dummy subTasks
	 */
	public SubTask(String desc) {
		super(desc, new Date(), new Date());
	}

	/**
	 * Constructor for SubTask
	 * 
	 * @param subTask
	 *            the subTask that is being tagged with a start and end time
	 * @param time
	 *            the start time of the subTask
	 * @param time2
	 *            the end time of the subTask
	 */
	public SubTask(SubTask subTask, Date time, Date time2) {
		super(subTask.getSpecification().getDescription(), time, time2);
		taskHour = subTask.getTaskHour();
		specification = subTask.getSpecification();
		partPrice = subTask.getPrice();
		start = time;
		end = time2;
	}

	/**
	 * @return the task hour of the task this sub task represents
	 */
	public int getTaskHour() {
		return taskHour;
	}

	/**
	 * @return the specification of the original task this sub task is a part of
	 */
	public TaskSpecification getSpecification() {
		return specification;
	}

	/**
	 * Setter for the price associated with the subTask
	 * 
	 * @param price
	 *            the price associated with the subTask
	 */
	public void setPrice(Price price) {
		partPrice = price;
	}

	/**
	 * @return the price associated with the subTask
	 */
	public Price getPrice() {
		return partPrice;
	}

	/**
	 * @return whether the subTask is a dummy or it corresponds to an actual
	 *         task hour of an existing task
	 */
	public boolean isDummy() {
		return false;
	}

	/**
	 * @return whether the task this subTask is a part of is allowed to be run
	 *         on spot instances
	 */
	public boolean isSpotEnabled() {
		if (this.isDummy()) {
			return false;
		}
		return specification.isSpotAllowed();
	}

	/**
	 * @return the start time of the subTask
	 */
	public Date getStartDate() {
		return start;
	}

	/**
	 * @return the end time of the subTask
	 */
	public Date getEndDate() {
		return end;
	}

	/**
	 * Tag the subTask with the desired time period
	 * 
	 * @param startDate
	 *            the start time of the subTask
	 * @param endDate
	 *            the end timse of the subTask
	 */
	public void setTimePeriod(Date startDate, Date endDate) {
		start = startDate;
		end = endDate;
		super.setDuration(new SimpleTimePeriod(startDate, endDate));
	}

}
