package be.ac.ua.broker.input.tasks;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import be.ac.ua.broker.utils.DateUtility;

/**
 * This class represents a workload of workload model two.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class WorkloadWM2 implements Workload {

	/**
	 * The list of collections of subTasks that belong to the workload, so a
	 * subTaskCollection is provided for every task hour of the associated
	 * workload model two task
	 */
	private List<SubTaskCollection> subTasks = new ArrayList<SubTaskCollection>();

	/**
	 * The specification of the task the workload belongs to
	 */
	private TaskSpecification specification;

	/**
	 * The number of hours between the start of the scheduling period and the
	 * deadline of the associated task
	 */
	private Integer hoursTimeframe = null;

	/**
	 * Constructor for WorkloadWM2
	 * 
	 * @param spec
	 *            specification of the task the workload belongs to
	 */
	public WorkloadWM2(TaskSpecification spec) {
		specification = spec;
	}

	/**
	 * Constructor for WorkloadWM2
	 * 
	 * @param spec
	 *            specification of the task the workload belongs to
	 * @param part
	 *            list of a collection of subTasks that belong to the workload
	 * @param hoursTimeFrame
	 *            the number of hours between the start of the scheduling period
	 *            and the deadline of the associated task
	 */
	public WorkloadWM2(TaskSpecification spec, List<List<SubTask>> part,
			int hoursTimeFrame) {
		specification = spec;
		hoursTimeframe = hoursTimeFrame;
		for (List<SubTask> list : part) {
			subTasks.add(new SubTaskCollection(list));
		}
	}

	/**
	 * Add a subTaskCollection to the workload, it is automatically associated
	 * with the next task hour of the associated task
	 * 
	 * @param coll
	 *            the subTaskCollection to add
	 */
	public void addTaskHour(SubTaskCollection coll) {
		subTasks.add(coll);
	}

	/**
	 * Getter for the subTaskCollection associated with a given task hour of the
	 * associated task
	 * 
	 * @param i
	 *            the task hour to get the associated collection of subtasks for
	 * @return the subTaskCollection associated with given task hour
	 */
	public SubTaskCollection getTasksOnHour(int i) {
		if (i <= this.getLength()) {
			return subTasks.get(i);
		}
		throw new IndexOutOfBoundsException();
	}

	/**
	 * @return the list of collections of subtasks that are associated with
	 *         every task hour of the tasks
	 */
	public List<List<SubTask>> getWorkload() {
		List<List<SubTask>> wl = new ArrayList<List<SubTask>>();
		for (SubTaskCollection taskColl : subTasks) {
			wl.add(taskColl.getWorkload());
		}
		return wl;
	}

	/**
	 * @return the number of hours between the start of the scheduling period
	 *         and the deadline of the associated task
	 */
	public int getHoursTimeFrame() {
		if (hoursTimeframe != null) {
			return hoursTimeframe;
		}
		return DateUtility.getNumberOfHoursBetween(specification.getStart(),
				specification.getDeadline());
	}

	@Override
	public int getLength() {
		return subTasks.size();
	}

	@Override
	public Date getStartDate() {
		return specification.getStart();
	}

	@Override
	public Date getDeadline() {
		return specification.getDeadline();
	}

	@Override
	public TaskSpecification getTaskSpecification() {
		return specification;
	}

	@Override
	public int getWorkload(int j) {
		SubTaskCollection subs = subTasks.get(j);
		return subs.getLength();
	}

	@Override
	public int compareTo(Workload wl) {
		if (wl.getDeadline().before(this.getDeadline())) {
			return 1;
		} else if (this.getDeadline().before(wl.getDeadline())) {
			return -1;
		}
		return 0;
	}

}
