package be.ac.ua.broker.input.tasks.generation.wm2;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import be.ac.ua.broker.utils.InputVerifier;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

import com.csvreader.CsvReader;

/**
 * This class is used to read the mapping from Amazon EC2 supported instance
 * type to the associated number of compute units. This number is used to create
 * a workload load for every task hour of a workload model two task according to
 * the ratios provided by these ECUs.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class ComputeInstanceMapper {

	/** CSVReader instance */
	private CsvReader fReader;

	/**
	 * Mapping from instance type to a number of compute units associated with
	 * the instance by EC2
	 */
	protected Map<String, Double> fContents = new HashMap<String, Double>();

	/**
	 * The location of the CSV file describing the number of EC2 compute units
	 * associated with every instance type supporte
	 */
	private String inputFile;

	/**
	 * Constructor for ComputeInstanceMapper
	 * 
	 * @param inputComputeInstanceCSV
	 *            the location of the CSV file describing the number of EC2
	 *            compute units associated with every instance type supported
	 */
	public ComputeInstanceMapper(String inputComputeInstanceCSV) {
		inputFile = inputComputeInstanceCSV;
	}

	/**
	 * Parse the given CSV file and store its contents
	 */
	public void parse() {
		try {
			fReader = new CsvReader(inputFile);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		try {
			if (fReader != null) {
				while (fReader.readRecord()) {
					int cols = fReader.getColumnCount();
					List<String> content = new ArrayList<String>();
					for (int i = 0; i < cols; i++) {
						content.add(fReader.get(i));
					}
					try {
						if (content.size() == 2) {
							double val = Double.parseDouble(content.get(1));
							fContents.put(content.get(0), val);
						}
					} catch (NumberFormatException e) {
						e.printStackTrace();
					}
				}
				fReader.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Getter for the number of compute units associated by EC2 with a given
	 * instance type
	 * 
	 * @param instance
	 *            the instance type to get the associated number of compute
	 *            units for
	 * @return the number of compute units associated by EC2 with a given
	 *         instance type
	 * @throws InstanceNotAvailableException
	 * @throws InvalidInputException
	 */
	public Double getNumberOfComputeUnits(final String instance)
			throws InstanceNotAvailableException, InvalidInputException {
		InputVerifier.instanceExists(instance);
		Double returnValue = fContents.get(instance);
		if (returnValue == null) {
			throw new InstanceNotAvailableException(instance);
		}
		return returnValue;
	}

}
