package be.ac.ua.broker.input.tasks.generation.wm2;

import java.io.IOException;

import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.InputVerifier;
import be.ac.ua.broker.utils.InvalidInputException;

import com.csvreader.CsvWriter;

/**
 * This class is used to generate a workload model two task that uses the
 * specification provided using the program arguments. A possible program
 * argument array is: 'task1 descr "Micro - Micro" "EU - Ireland" "Linux/Unix"
 * "2011-11-11 00:00" 3600 workload1 false'. We first provide a desired name and
 * description for the task, followed by information about the instance the task
 * has to run on: the instance type, the geographical region and the operating
 * system. We also provide the deadline of the task, the length of the task in
 * hours, a name associated with the randomly generated workload created for the
 * task and whether or not the task is allowed to be run on spot instances.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class TaskGenerator {

	/**
	 * The location to store the description file of the generated workload
	 * model two tasks
	 */
	public static String DEFAULT_OUTPUT_FOLDER = "input/tasks/wm2/task/";

	/**
	 * The location of the file that contains the mapping information from the
	 * supported EC2 instance types to the associated number of compute units.
	 */
	public static String COMPUTE_INSTANCE_INPUT_FILE = "input/general/EC2ComputeInstances.csv";

	/**
	 * Tag name used for the name of the task.
	 */
	public static String TASK_NAME = "name";

	/**
	 * Tag name used for the description of the task.
	 */
	public static String TASK_DESCRIPTION = "decription";

	/**
	 * Tag name used for the instance type of the instance the task has to run
	 * on.
	 */
	public static String TASK_INSTANCE = "instance";

	/**
	 * Tag name used for the geographical region of the instance the task has to
	 * run on.
	 */
	public static String TASK_REGION = "region";

	/**
	 * Tag name used for the operating system of the instance the task has to
	 * run on.
	 */
	public static String TASK_OS = "os";

	/**
	 * Tag name used for the deadline of the task.
	 */
	public static String TASK_DEADLINE = "deadline";

	/**
	 * Tag name used for the length (in hours) of the task.
	 */
	public static String TASK_LENGTH = "length";

	/**
	 * Tag name used for the file in which the generated workload distribution
	 * of the task is stored.
	 */
	public static String TASK_WORKLOAD = "workload_file";

	/**
	 * Tag name used for the fact whether the task is allowed to be run on spot
	 * instances.
	 */
	public static String TASK_SPOT = "spot_allowed";

	/**
	 * An array description containing all the values for the variables of the
	 * generated task in the above described order: name, desc, inst, reg, os,
	 * dead, length, workload, spot.
	 */
	private String[] arrDescription;

	/**
	 * The TaskGenerator is used to generate task description files of tasks
	 * that use workload model two. A possible program argument array is: 'task1
	 * descr "Micro - Micro" "EU - Ireland" "Linux/Unix" "2011-11-11 00:00" 3600
	 * workload1 false'. We first provide a desired name and description for the
	 * task, followed by information about the instance the task has to run on:
	 * the instance type, the geographical region and the operating system. We
	 * also provide the deadline of the task, the length of the task in hours, a
	 * name associated with the randomly generated workload created for the task
	 * and whether or not the task is allowed to be run on spot instances.
	 * 
	 * @param args
	 *            program arguments
	 */
	public static void main(String[] args) {
		if (args.length != 9) {
			System.err.println("Usage: TaskGenerator <" + TASK_NAME + "> <"
					+ TASK_DESCRIPTION + "> <" + TASK_INSTANCE + "> <"
					+ TASK_REGION + "> <" + TASK_OS + "> <" + TASK_DEADLINE
					+ "> <" + TASK_LENGTH + "> <" + TASK_WORKLOAD + "> <"
					+ TASK_SPOT + ">");
			System.exit(1);
		}
		System.out.println("================================");
		System.out.println("= Task Generator               =");
		System.out.println("================================");
		new TaskGenerator(DEFAULT_OUTPUT_FOLDER, args[0], args[1], args[2],
				args[3], args[4], args[5], args[6], args[7], args[8]);
		System.out.println("Finished Generating Task.");
	}

	/**
	 * Constructor for TaskGenerator.
	 * 
	 * @param outputFolder
	 *            the location for the task description file created for the
	 *            generated task
	 * @param taskName
	 *            the name of the task
	 * @param taskDescription
	 *            the description of the task
	 * @param taskInstance
	 *            the instance type of the instance the task has to run on.
	 * @param taskRegion
	 *            the geographical region of the instance the task has to run
	 *            on.
	 * @param taskOS
	 *            the operating system of the instance the task has to run on.
	 * @param taskDeadline
	 *            the deadline of the task
	 * @param taskLength
	 *            the length in hours of the task
	 * @param taskWorkload
	 *            the file in which the generated workload distribution of the
	 *            task has to be stored.
	 * @param spotAllowed
	 *            whether the task is allowed to be run on spot instances.
	 */
	public TaskGenerator(String outputFolder, String taskName,
			String taskDescription, String taskInstance, String taskRegion,
			String taskOS, String taskDeadline, String taskLength,
			String taskWorkload, String spotAllowed) {
		InputVerifier.setup();
		try {
			String length = taskLength;
			// Create the output CSV file
			final CsvWriter csvWriter = new CsvWriter(outputFolder + taskName
					+ ".csv");
			// Write the column names to the output file
			String arr1[] = { TASK_NAME, taskName };
			String arr2[] = { TASK_DESCRIPTION, taskDescription };
			InputVerifier.instanceExists(taskInstance);
			String arr3[] = { TASK_INSTANCE, taskInstance };
			InputVerifier.osExists(taskOS);
			String arr5[] = { TASK_OS, taskOS };
			String arr6[] = { TASK_DEADLINE,
					DateUtility.dateToString(DateUtility.toDate(taskDeadline)) };
			String arr7[] = { TASK_LENGTH, length };
			String arr8[] = { TASK_WORKLOAD, taskWorkload };
			String arr9[] = { TASK_SPOT, spotAllowed };
			csvWriter.writeRecord(arr1);
			csvWriter.writeRecord(arr2);
			csvWriter.writeRecord(arr3);
			if (taskRegion != "") {
				InputVerifier.regionExists(taskRegion);
				String arr4[] = { TASK_REGION, taskRegion };
				csvWriter.writeRecord(arr4);
			}
			csvWriter.writeRecord(arr5);
			csvWriter.writeRecord(arr6);
			csvWriter.writeRecord(arr7);
			csvWriter.writeRecord(arr8);
			csvWriter.writeRecord(arr9);
			csvWriter.flush();
			csvWriter.close();
			WorkloadGenerator workloadGen = new WorkloadGenerator(
					COMPUTE_INSTANCE_INPUT_FILE);
			workloadGen.createWorkload(taskWorkload, Integer.parseInt(length));
			String arrDesc[] = { taskName, taskRegion, taskInstance, taskOS,
					taskDeadline, taskLength, spotAllowed };
			arrDescription = arrDesc;
		} catch (InvalidInputException e1) {
			e1.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return an array description of the generated task
	 */
	public String[] getArrayDescription() {
		return arrDescription;
	}

}
