package be.ac.ua.broker.input.tasks.generation.wm2;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

import com.csvreader.CsvWriter;

/**
 * This class is used to generate a workload distribution for a workload model
 * two task, it provides a load for every supported instance type according to
 * the ratios provided by the number of EC2 compute units associated with the
 * instance type.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class WorkloadGenerator {

	/**
	 * Random number generator used to generate the amount the load
	 * increases/decreases between conjunctive task hours.
	 */
	private static Random rand = new Random();

	/**
	 * Mapping from instance type to the fact whether the wave like generated
	 * workload distribution for that particular instance type has increased
	 * during the generation of the last processed task hour
	 */
	private Map<String, Boolean> signMapper = new HashMap<String, Boolean>();

	/**
	 * The instanceMapper provides access to the number of EC2 compute units
	 * associated with all the instance type supported.
	 */
	private ComputeInstanceMapper instanceMapper;

	/**
	 * Constructor for WorkloadGenerator
	 * 
	 * @param inputComputeInstanceCSV
	 *            the file providing the ECU information for the supported
	 *            instance types
	 */
	public WorkloadGenerator(String inputComputeInstanceCSV) {
		instanceMapper = new ComputeInstanceMapper(inputComputeInstanceCSV);
		instanceMapper.parse();
	}

	/**
	 * Create a random workload with a wave distribution for all the possible
	 * instance types for a task with a given length.
	 * 
	 * @param filePath
	 *            location of the file to store the generated workload in
	 * @param lengthInHours
	 *            the length of the task to generated the workload for
	 */
	public void createWorkload(String filePath, int lengthInHours) {
		try {
			// Create the output CSV file
			final CsvWriter csvWriter = new CsvWriter(filePath);
			// Write the column names to the output file
			List<String> instances = new ArrayList<String>(EC2Specifications
					.getInstanceTypeMapping().keySet());
			instances.add(0, "Hour");
			String[] strArray = new String[instances.size()];
			instances.toArray(strArray);
			csvWriter.writeRecord(strArray);
			csvWriter.flush();
			instances.remove(0);
			List<String> values = this.createRandomLoads(null, instances);
			for (int i = 0; i < lengthInHours; i++) {
				values.add(0, String.valueOf(i));
				String[] strArray2 = new String[values.size()];
				values.toArray(strArray2);
				csvWriter.writeRecord(strArray2);
				csvWriter.flush();
				values.remove(0);
				values = this.createRandomLoads(values, instances);
			}
			csvWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InstanceNotAvailableException e) {
			e.printStackTrace();
		} catch (InvalidInputException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create a random load associated with a single task hour of the provided
	 * task description. Note that the load does not cross certain upper and
	 * lower boundaries.
	 * 
	 * @param prevVals
	 *            the load during the previous task hour of the corresponding
	 *            task
	 * @param instances
	 *            the instance types we're generating workload distributions for
	 * @return the generated load for the task hour being processed
	 * @throws InstanceNotAvailableException
	 * @throws InvalidInputException
	 */
	private List<String> createRandomLoads(List<String> prevVals,
			List<String> instances) throws InstanceNotAvailableException,
			InvalidInputException {
		List<String> vals = new ArrayList<String>();
		int count = 0;
		for (String inst : instances) {
			double d = rand.nextDouble();
			double ratio = 1 / (double) instanceMapper
					.getNumberOfComputeUnits(inst);
			int maxInt = (int) Math.ceil(ratio * 20);
			int upper = 2 * maxInt;
			int lower = ((1 / 2) * maxInt);
			if (prevVals == null) {
				double val = d * maxInt;
				if (val > upper) {
					val = upper;
				} else if (val < lower) {
					val = lower;
				}
				vals.add(String.valueOf(val));
				signMapper.put(inst, rand.nextBoolean());
			} else {
				double prev = Double.parseDouble(prevVals.get(count));
				if (signMapper.get(inst)) {
					if (prev + d < upper) {
						vals.add(String.valueOf(prev + d));
					} else {
						vals.add(String.valueOf(prev - d));
						signMapper.put(inst, false);
					}
				} else {
					if (prev - d > lower) {
						vals.add(String.valueOf(prev - d));
					} else {
						vals.add(String.valueOf(prev + d));
						signMapper.put(inst, true);
					}
				}

			}
			count++;
		}
		return vals;
	}

}
