package be.ac.ua.broker.input.tasks.specification;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.tasks.SubTaskCollection;
import be.ac.ua.broker.input.tasks.Task;
import be.ac.ua.broker.input.tasks.TaskCollection;
import be.ac.ua.broker.input.tasks.TaskSpecification;
import be.ac.ua.broker.input.tasks.Workload;
import be.ac.ua.broker.input.tasks.WorkloadWM1;
import be.ac.ua.broker.input.tasks.generation.wm2.TaskGenerator;
import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.InvalidInputException;

import com.csvreader.CsvReader;

/**
 * This class is used to read a workload of model one or two from file.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class TaskReader {

	/**
	 * Location of the input workload model two tasks
	 */
	public static String INPUT_DIR_WM2 = "input/tasks/wm2/task/";

	/**
	 * Location of the input workload model one tasks
	 */
	public static String INPUT_DIR_WM1 = "input/tasks/wm1/task/";

	/**
	 * Parse the tasks found at the given path location
	 * 
	 * @param inputDir
	 *            the location of the task descriptions to parse
	 * @param start
	 *            the start time of the scheduling period for the tasks
	 * @return a task collection containing all the workloads found at the given
	 *         location
	 * @throws InvalidInputException
	 */
	public TaskCollection parseInputDir(String inputDir, Date start)
			throws InvalidInputException {
		TaskCollection taskCollection = new TaskCollection(start);
		File dir = new File(inputDir);
		String[] children = dir.list();
		if (children != null) {
			for (int i = 0; i < children.length; i++) {
				// Get filename of file or directory
				String filename = children[i];
				if (!filename.startsWith(".")) {
					Task task = this.createTask(inputDir + filename);
					taskCollection.addTask(task);
				}
			}
		}
		return taskCollection;
	}

	/**
	 * Parse a single task found at the given path location
	 * 
	 * @param inputDir
	 *            the location of the task description to parse
	 * @param start
	 *            the start time of the scheduling period for the tasks
	 * @return a task collection containing the workload found at the given
	 *         location
	 * @throws InvalidInputException
	 */
	public TaskCollection parseInputFile(String filePath, Date start)
			throws InvalidInputException {
		TaskCollection taskCollection = new TaskCollection(start);
		Task task = this.createTask(filePath);
		taskCollection.addTask(task);
		return taskCollection;
	}

	/**
	 * Read a single task from file
	 * 
	 * @param filePath
	 *            the location of the task description to parse
	 * @return the task representation of the task described by the given file
	 * @throws InvalidInputException
	 */
	private Task createTask(String filePath) throws InvalidInputException {
		Workload wl = null;
		String wlLocation = null;
		String nm = null;
		String desc = null;
		String instance = null;
		String region = null;
		String os = null;
		Date dead = null;
		boolean spotAllowed = false;
		int length = 0;
		try {
			CsvReader fReader = new CsvReader(filePath);
			if (fReader != null) {
				while (fReader.readRecord()) {
					int cols = fReader.getColumnCount();
					List<String> content = new ArrayList<String>();
					for (int i = 0; i < cols; i++) {
						content.add(fReader.get(i));
					}
					String label = content.get(0);
					String value = content.get(1);
					if (label.equalsIgnoreCase(TaskGenerator.TASK_NAME)) {
						nm = value;
					} else if (label
							.equalsIgnoreCase(TaskGenerator.TASK_DESCRIPTION)) {
						desc = value;
					} else if (label
							.equalsIgnoreCase(TaskGenerator.TASK_INSTANCE)) {
						instance = value;
					} else if (label
							.equalsIgnoreCase(TaskGenerator.TASK_REGION)) {
						region = value;
					} else if (label.equalsIgnoreCase(TaskGenerator.TASK_OS)) {
						os = value;
					} else if (label
							.equalsIgnoreCase(TaskGenerator.TASK_DEADLINE)) {
						dead = DateUtility.toDate(value);
					} else if (label
							.equalsIgnoreCase(TaskGenerator.TASK_WORKLOAD)) {
						wlLocation = value;
					} else if (label.equalsIgnoreCase(TaskGenerator.TASK_SPOT)) {
						spotAllowed = Boolean.parseBoolean(value);
					} else if (label
							.equalsIgnoreCase(TaskGenerator.TASK_LENGTH)) {
						length = Integer.parseInt(value);
					}
				}
				fReader.close();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (nm == null || desc == null || instance == null || os == null
				|| dead == null) {
			if (nm == null && desc == null && instance == null && os == null
					&& dead == null) {
				return null;
			}
			throw new IllegalArgumentException(
					"A required task parameter was not given");
		}
		InstanceSpecification instSpec;
		if (region == null) {
			instSpec = new InstanceSpecification(instance, os);
		} else {
			instSpec = new InstanceSpecification(instance, os, region);
		}
		TaskSpecification spec = new TaskSpecification(nm, desc, instSpec,
				dead, spotAllowed);
		if (wlLocation != null) {
			WorkloadReader wlReader = new WorkloadReader();
			wl = wlReader.getWorkload(wlLocation, spec
					.getInstanceSpecification().getInstance(), spec);
		} else {
			SubTaskCollection coll = new SubTaskCollection(length, spec);
			wl = new WorkloadWM1(spec, coll);
		}
		return new Task(spec, wl);
	}

}
