package be.ac.ua.broker.main;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Date;
import java.util.List;

import be.ac.ua.broker.allocation.Allocator;
import be.ac.ua.broker.allocation.AllocatorOptions;
import be.ac.ua.broker.input.pricing.PricingManager;
import be.ac.ua.broker.input.tasks.TaskCollection;
import be.ac.ua.broker.input.tasks.specification.TaskReader;
import be.ac.ua.broker.output.gui.ScheduleGUI;
import be.ac.ua.broker.output.textual.PriceCalculator;
import be.ac.ua.broker.output.textual.PriceDescriptorNode;
import be.ac.ua.broker.output.textual.SchedulePrinter;
import be.ac.ua.broker.scheduling.region.RegionAllocator;
import be.ac.ua.broker.scheduling.region.RegionWorkloadCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerOptions;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;
import be.ac.ua.broker.utils.DateUtility;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

import com.csvreader.CsvWriter;

/**
 * Main class of the EC2Broker this is a broker used to schedule tasks on EC2
 * instances such that the total cost is minimal while still satisfying the
 * given deadline constraints. The difficulty exists in scheduling these tasks
 * in time and determining when to use reserved, on-demand or spot instances. It
 * takes the output of the depending projects as input. For example the prices
 * of the different on-demand and reserved instances will be provided through
 * CSV files. The output of the spot price analysis will be used by the broker
 * as well. The output of this broker consists of CSV files containing all the
 * information about the scheduled tasks, but optionally everything can be
 * rendered in a GUI as well.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class EC2Broker {

	/** Specify whether workload model one is used */
	public static boolean USE_WORKLOAD_MODEL_ONE = true;

	/** The start date of the one-year scheduling period */
	public static Date startDate = DateUtility.toDate("2010-04-04 00:00");

	/** Output location for the schedule overview */
	private static String OUTPUT_SCHEDULE_FILE = "output/schedule/schedule.txt";

	/** Whether or not to print the output to file */
	private static boolean PRINT_TO_FILE = false;

	/** Whether or not to show textual debug/information output */
	private static boolean SHOW_TEXTUAL_OUTPUT = false;

	/** Whether or not to show the GUI representation of the generated schedule */
	private static boolean SHOW_GRAPHICAL_OUTPUT = true;

	/**
	 * The Main function, used to start the EC2 Broker application.
	 * 
	 * @param args
	 *            program arguments
	 */
	public static void main(String[] args) {
		System.out.println("================================");
		System.out.println("= EC2 Broker                   =");
		System.out.println("================================");
		try {
			System.out.println("Parsing input files");
			TaskReader reader = new TaskReader();
			String inputDir = TaskReader.INPUT_DIR_WM2;
			if (USE_WORKLOAD_MODEL_ONE) {
				inputDir = TaskReader.INPUT_DIR_WM1;
			}
			TaskCollection coll = reader.parseInputDir(inputDir, startDate);
			System.out.println("Scheduling tasks");
			PricingManager.setup();
			RegionAllocator regionAllocator = new RegionAllocator(coll);
			List<RegionWorkloadCollection> workloadCollections = regionAllocator
					.allocateRegions();
			SchedulerResultCollection results = new SchedulerResultCollection(
					startDate, coll.getLatestDeadline());
			for (RegionWorkloadCollection regionWorkloadCollection : workloadCollections) {
				// regionWorkloadCollection.print();
				results.addAll(regionWorkloadCollection.schedule(
						USE_WORKLOAD_MODEL_ONE, SchedulerOptions.SPOTOPTIMIZED));
			}
			System.out.println("Determining Prices");
			Allocator pricingModelDeterminator = new Allocator(
					USE_WORKLOAD_MODEL_ONE);
			SchedulerResultCollection resultsWithPrices = pricingModelDeterminator
					.allocate(AllocatorOptions.SPOT, results);
			if (SHOW_TEXTUAL_OUTPUT) {
				try {
					SchedulePrinter printer;
					if (PRINT_TO_FILE) {
						File file = new File(OUTPUT_SCHEDULE_FILE);
						printer = new SchedulePrinter(new PrintStream(file));
					} else {
						printer = new SchedulePrinter(System.out);
					}
					printer.print(resultsWithPrices);
					//Print cost overview to file
					PriceCalculator calc = new PriceCalculator(results);
					PriceDescriptorNode tree = calc.createTree(PriceCalculator.OS, PriceCalculator.INSTANCETYPE, PriceCalculator.REGION, PriceCalculator.PRICINGMODEL);
					final CsvWriter csvWriter = new CsvWriter("output/schedule/"
							+ "priceoverview" + ".csv");
					tree.toFile(csvWriter);
					csvWriter.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (SHOW_GRAPHICAL_OUTPUT) {
				System.out.println("Opening gui");
				ScheduleGUI gui = new ScheduleGUI();
				gui.render(resultsWithPrices);
			}
			System.out.println("Finished");
		} catch (InvalidInputException e) {
			e.printStackTrace();
		} catch (InstanceNotAvailableException e) {
			e.printStackTrace();
		}
	}

}
