package be.ac.ua.broker.output.gui;

import java.text.DecimalFormat;
import java.util.List;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import be.ac.ua.broker.output.textual.PriceCalculator;
import be.ac.ua.broker.output.textual.PriceDescriptorNode;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;

/**
 * This class is used to create a JScrollPane containing an overview of the
 * costs associated with the represented schedules generated by the broker, the
 * price overview is created using a PriceCalculator and can be given an order
 * of item to sort the price on in the tree. In this case it is first divided
 * between the different geographical regions, followed by a division between
 * the pricing models that were used, then the operating systems that were used
 * and finally the instance types used. This order can easily be changed.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class PriceTree {

	/** Serial Version ID */
	private static final long serialVersionUID = 1L;
	
	/** The schedule collection to create a tree view for */
	private SchedulerResultCollection collection;
	
	/** Decimal formatter */
	private DecimalFormat formatter;

	/**
	 * Constructor for PriceTree
	 * 
	 * @param result
	 *            the schedulerResultCollection to create a graphical tree
	 *            overview of the associated costs for
	 */
	public PriceTree(SchedulerResultCollection result) {
		collection=result;
		formatter = new DecimalFormat("#####.00");
	}

	/**
	 * Create a graphical tree overview of the associated costs for the given
	 * schedulerResultCollection
	 * 
	 * @param result
	 *            the schedulerResultCollection to create a graphical tree
	 *            overview of the associated costs for
	 * @return the graphical tree representation of the costs associated with
	 *         the given broker schedules
	 */
	public JTree getTree() {
		PriceCalculator calc = new PriceCalculator(collection);
		PriceDescriptorNode tree = calc.createTree(PriceCalculator.REGION,
				PriceCalculator.PRICINGMODEL, PriceCalculator.OS,
				PriceCalculator.INSTANCETYPE);
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
				tree.getDescription() + " [" + formatter.format(tree.getPrice()) + "]");
		if (tree.hasChildren()) {
			this.addChildren(root, tree.getChildren());
		}
		JTree guiTree = new JTree(root);
		int currentRow = 0;
		while (currentRow <= 2) {
			int initialrows = guiTree.getRowCount();
			for (int i = currentRow; i < initialrows; i++) {
				guiTree.expandRow(i + guiTree.getRowCount() - initialrows);
			}
			currentRow++;
		}
		return guiTree;

	}

	/**
	 * Add the given list of children to the given tree root.
	 * 
	 * @param root
	 *            the node of the tree to add children to
	 * @param children
	 *            the children to add to the given node
	 */
	private void addChildren(DefaultMutableTreeNode root,
			List<PriceDescriptorNode> children) {
		for (PriceDescriptorNode priceDescriptorNode : children) {
			DefaultMutableTreeNode child = new DefaultMutableTreeNode(
					priceDescriptorNode.getDescription() + " ["
							+ formatter.format(priceDescriptorNode.getPrice()) + "]");
			root.add(child);
			if (priceDescriptorNode.hasChildren()) {
				this.addChildren(child, priceDescriptorNode.getChildren());
			}
		}
	}

}
