package be.ac.ua.broker.output.gui;

import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.data.gantt.Task;
import org.jfree.data.gantt.TaskSeries;
import org.jfree.data.gantt.TaskSeriesCollection;

import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerResult;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;

/**
 * This class is used to generate the graphical representation of the provided
 * broker result. The schedulerResultCollection parts are first divided across
 * the geographical regions that occur, all are represented by a separate tab.
 * For every tab a Gantt-like chart for the corresponding broker schedule is
 * created. Another tab is added for the general cost overview.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class ScheduleRegionTabs extends JTabbedPane {

	/** Serial Version ID */
	private static final long serialVersionUID = 1L;

	/**
	 * Constructor for ScheduleRegionTabs
	 * 
	 * @param result
	 *            the schedulerResultCollection brokering result to create a
	 *            graphical representation of
	 */
	public ScheduleRegionTabs(final SchedulerResultCollection result) {
		super();
		Set<String> regions = EC2Specifications.getRegionMapping().keySet();
		TaskSeriesCollection taskSeriesColl = new TaskSeriesCollection();
		for (String reg : regions) {
			if (result.get(reg) != null && !result.get(reg).isEmpty()) {
				TaskSeries series = this.toTaskSeries(result.get(reg), reg,
						result.getStartDate(), result.getEndDate());
				TaskSeriesCollection coll = new TaskSeriesCollection();
				coll.add(series);
				JFreeChart chart = ChartFactory.createGanttChart("",
						"Instance", "Time", coll, false, true, false);
				ChartPanel panel1 = new ChartPanel(chart);
				chart.getCategoryPlot().setRangePannable(true);
				chart.getCategoryPlot().setRenderer(new ScheduleRenderer());
				panel1.setDisplayToolTips(true);
				this.addTab(reg, null, panel1);
				taskSeriesColl.add(series);
			}
		}
		PriceTree priceTree=new PriceTree(result);
		this.addTab("Price Overview", new JScrollPane(priceTree.getTree()));
	}

	/**
	 * Convert a list of schedulerResults all for the same geographical region
	 * to a taskSeries that can be used by JFreeChart to create the Gantt-like
	 * chart.
	 * 
	 * @param res
	 *            the list of schedulerResults to convert to a taskSeries
	 * @param reg
	 *            the regions to which the schedule is part of
	 * @param start
	 *            the start of the scheduling period
	 * @param end
	 *            the end of the scheduling period
	 * @return the taskSeries generated for the provided list of
	 *         schedulerResults
	 */
	public TaskSeries toTaskSeries(List<SchedulerResult> res, String reg,
			Date start, Date end) {
		TaskSeries all = new TaskSeries(reg);
		all.setDescription(reg);
		if (res != null) {
			for (SchedulerResult schedulerResult : res) {
				String label = schedulerResult.getInstanceSpecification()
						.getIdentifier();
				List<ScheduledTaskCollection> taskColls = schedulerResult
						.getScheduledTaskCollections();
				int count = 0;
				for (ScheduledTaskCollection scheduledTaskCollection : taskColls) {
					String key = label + "+"
							+ scheduledTaskCollection.getPricingModel() + " ["
							+ count + "]";
					Task taskInst = new Task(key, start, end);
					List<SubTask> subTasks = scheduledTaskCollection
							.getSubTasks();
					for (SubTask subTask : subTasks) {
						if (!subTask.isDummy()) {
							taskInst.addSubtask(subTask);
						}
					}
					all.add(taskInst);
					count++;
				}
			}
		}
		return all;
	}

}
