package be.ac.ua.broker.output.gui;

import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Map;

import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.CategoryItemEntity;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRendererState;
import org.jfree.chart.renderer.category.GanttRenderer;
import org.jfree.data.gantt.GanttCategoryDataset;
import org.jfree.data.gantt.Task;
import org.jfree.data.gantt.TaskSeries;
import org.jfree.data.gantt.TaskSeriesCollection;
import org.jfree.ui.RectangleEdge;

import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.utils.RandomGenerator;

/**
 * This class is used as renderer for the Gantt-chart representing the
 * broker schedule.
 * 
 * @author Vermeersch Kurt (based on GanttRenderer.java by JFreeChart)
 * @date May 2011
 */
public class ScheduleRenderer extends GanttRenderer {

	/** Serial Version ID */
	private static final long serialVersionUID = 1L;

	/**
	 * The ToolTipGenerator to use when the user hovers over a part of the
	 * Gantt-chart for the broker schedule.
	 */
	private ToolTipGenerator tooltipgenerator;

	/**
	 * Mapping of a task name to a color associated with the task, the subtasks
	 * rectangles in the Gantt-like chart for the broker schedule are all
	 * painted with the same color.
	 */
	private static Map<String, Paint> colorMap = new HashMap<String, Paint>();

	/**
	 * Constructor for ScheduleRenderer.
	 */
	public ScheduleRenderer() {
		super();
		tooltipgenerator = new ToolTipGenerator();
		this.setBaseToolTipGenerator(tooltipgenerator);
	}

	/**
	 * Draws the tasks/subtasks for one item.
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param state
	 *            the renderer state.
	 * @param dataArea
	 *            the data plot area.
	 * @param plot
	 *            the plot.
	 * @param domainAxis
	 *            the domain axis.
	 * @param rangeAxis
	 *            the range axis.
	 * @param dataset
	 *            the data.
	 * @param row
	 *            the row index (zero-based).
	 * @param column
	 *            the column index (zero-based).
	 */
	protected void drawTasks(Graphics2D g2, CategoryItemRendererState state,
			Rectangle2D dataArea, CategoryPlot plot, CategoryAxis domainAxis,
			ValueAxis rangeAxis, GanttCategoryDataset dataset, int row,
			int column) {

		int count = dataset.getSubIntervalCount(row, column);
		if (count == 0) {
			drawTask(g2, state, dataArea, plot, domainAxis, rangeAxis, dataset,
					row, column);
		}

		for (int subinterval = 0; subinterval < count; subinterval++) {

			RectangleEdge rangeAxisLocation = plot.getRangeAxisEdge();

			// value 0
			Number value0 = dataset.getStartValue(row, column, subinterval);
			if (value0 == null) {
				return;
			}
			double translatedValue0 = rangeAxis.valueToJava2D(
					value0.doubleValue(), dataArea, rangeAxisLocation);

			// value 1
			Number value1 = dataset.getEndValue(row, column, subinterval);
			if (value1 == null) {
				return;
			}
			double translatedValue1 = rangeAxis.valueToJava2D(
					value1.doubleValue(), dataArea, rangeAxisLocation);

			if (translatedValue1 < translatedValue0) {
				double temp = translatedValue1;
				translatedValue1 = translatedValue0;
				translatedValue0 = temp;
			}

			double rectStart = calculateBarW0(plot, plot.getOrientation(),
					dataArea, domainAxis, state, row, column);
			double rectLength = Math.abs(translatedValue1 - translatedValue0);
			double rectBreadth = state.getBarWidth();

			// DRAW THE BARS...
			Rectangle2D bar = null;

			if (plot.getOrientation() == PlotOrientation.HORIZONTAL) {
				bar = new Rectangle2D.Double(translatedValue0, rectStart,
						rectLength, rectBreadth);
			} else if (plot.getOrientation() == PlotOrientation.VERTICAL) {
				bar = new Rectangle2D.Double(rectStart, translatedValue0,
						rectBreadth, rectLength);
			}

			Paint seriesPaint = getItemPaint(row, column);
			TaskSeriesCollection series = (TaskSeriesCollection) dataset;
			TaskSeries serie = series.getSeries(row);
			Task task = serie.get(column);
			SubTask subtask = (SubTask) task.getSubtask(subinterval);
			String key = subtask.getSpecification().getName();
			Paint p = colorMap.get(key);
			if (p == null) {
				p = RandomGenerator.generateRandomColor();
				colorMap.put(key, p);
			}
			seriesPaint = p;

			g2.setPaint(seriesPaint);
			g2.fill(bar);

			if (isDrawBarOutline()
					&& state.getBarWidth() > BAR_OUTLINE_WIDTH_THRESHOLD) {
				g2.setStroke(getItemStroke(row, column));
				g2.setPaint(getItemOutlinePaint(row, column));
				g2.draw(bar);
			}

			// collect entity and tool tip information...
			if (state.getInfo() != null) {
				EntityCollection entities = state.getEntityCollection();
				if (entities != null) {
					String tip = null;
					String url = null;
					if (getItemURLGenerator(row, column) != null) {
						url = getItemURLGenerator(row, column).generateURL(
								dataset, row, column);
					}
					CategoryItemEntity entity = new SchedulerCategoryItemEntity(
							bar, tip, url, dataset, row,
							dataset.getColumnKey(column), column, subtask,
							tooltipgenerator);
					entities.add(entity);
				}
			}
		}
	}

}
