package be.ac.ua.broker.output.textual;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import be.ac.ua.broker.input.pricing.EC2Constants;
import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.input.pricing.Price;
import be.ac.ua.broker.input.pricing.ReservedPrice;
import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerResult;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;

/**
 * This class is used to generate a tree cost overview from a broker result. It
 * creates a cost tree using a PriceDescriptorNode as root node containing
 * PriceDescriptorNodes as children. The overview levels can be of a given
 * order, the possible divisions are: geographical region, operating system,
 * instance type of the instance or the pricing model being used by the
 * instance.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class PriceCalculator {

	/** String representing the geographical region level option */
	public static String REGION = "region";

	/** String representing the operating system level option */
	public static String OS = "os";

	/** String representing the instance type level option */
	public static String INSTANCETYPE = "instanceType";

	/** String representing the pricing model level option */
	public static String PRICINGMODEL = "pricingModel";

	/**
	 * The root node of the generated price tree, which contains the total price
	 * after the cost overview tree is generated.
	 */
	private PriceDescriptorNode priceTree = new PriceDescriptorNode(
			"Total Price");

	/**
	 * The schedulerResultCollection obtained by the broker after the allocation
	 * phase, for which a cost overview tree is generated.
	 */
	private SchedulerResultCollection schedule;

	/**
	 * Mapping from a string representing a value of the first level option
	 * provided to a list of subTasks associated with this value of the first
	 * level option.
	 */
	private Map<String, List<SubTask>> mapLevel1 = new HashMap<String, List<SubTask>>();

	/**
	 * Mapping from a string representing a value of the second level option
	 * provided to a list of subTasks associated with this value of the second
	 * level option.
	 */
	private Map<String, List<SubTask>> mapLevel2 = new HashMap<String, List<SubTask>>();

	/**
	 * Mapping from a string representing a value of the third level option
	 * provided to a list of subTasks associated with this value of the third
	 * level option.
	 */
	private Map<String, List<SubTask>> mapLevel3 = new HashMap<String, List<SubTask>>();

	/**
	 * Mapping from a string representing a value of the fourth level option
	 * provided to a list of subTasks associated with this value of the fourth
	 * level option.
	 */
	private Map<String, List<SubTask>> mapLevel4 = new HashMap<String, List<SubTask>>();

	/**
	 * Constructor for PriceCalculator
	 * 
	 * @param result
	 *            the schedulerResultCollection obtained by the broker after the
	 *            allocation phase, for which a cost overview tree has to be
	 *            generated.
	 */
	public PriceCalculator(final SchedulerResultCollection result) {
		schedule = result;
	}

	/**
	 * Create a cost overview tree according to the given order of level options
	 * 
	 * @param level1
	 *            level one option
	 * @param level2
	 *            level two option
	 * @param level3
	 *            level three option
	 * @param level4
	 *            level four option
	 * @return the generated cost overview tree
	 */
	public PriceDescriptorNode createTree(final String level1,
			final String level2, final String level3, final String level4) {
		List<PriceDescriptorNode> childs = new ArrayList<PriceDescriptorNode>();
		Set<String> regions = EC2Specifications.getRegionMapping().keySet();
		for (String reg : regions) {
			List<SchedulerResult> regionSchedules = schedule.get(reg);
			if (regionSchedules != null) {
				for (SchedulerResult schedulerResult : regionSchedules) {
					List<ScheduledTaskCollection> taskColl = schedulerResult
							.getScheduledTaskCollections();
					if (taskColl != null) {
						for (ScheduledTaskCollection scheduledTaskCollection : taskColl) {
							List<SubTask> subs = scheduledTaskCollection
									.getSubTasks();
							for (SubTask subTask : subs) {
								if (!subTask.isDummy()) {
									mapLevel(level1, 1, subTask);
								}
							}
						}
					}
				}
			}
		}
		double totalPriceAll = 0.0;
		for (Entry<String, List<SubTask>> entry : mapLevel1.entrySet()) {
			double totalPrice = this.calculateTotalPrice(entry.getValue());
			PriceDescriptorNode child1 = new PriceDescriptorNode(
					entry.getKey(), totalPrice);
			List<PriceDescriptorNode> childs1 = this.handleNextLevel(2,
					entry.getValue(), level1, level2, level3, level4);
			child1.addChildren(childs1);
			childs.add(child1);
			totalPriceAll += totalPrice;
		}
		priceTree.addChildren(childs);
		priceTree.setPrice(totalPriceAll);
		mapLevel1.clear();
		return priceTree;
	}

	/**
	 * Add an item to the appropriate map
	 * 
	 * @param levelString
	 *            the value of the item
	 * @param levelnumber
	 *            the number of the level that is being processed
	 * @param subTask
	 *            the subTask corresponding to the given levelString
	 */
	private void mapLevel(String levelString, int levelnumber, SubTask subTask) {
		if (REGION.equals(levelString)) {
			this.addToMap(levelnumber, subTask.getSpecification()
					.getInstanceSpecification().getRegion(), subTask);
		} else if (OS.equals(levelString)) {
			this.addToMap(levelnumber, subTask.getSpecification()
					.getInstanceSpecification().getOS(), subTask);
		} else if (INSTANCETYPE.equals(levelString)) {
			this.addToMap(levelnumber, subTask.getSpecification()
					.getInstanceSpecification().getInstance(), subTask);
		} else if (PRICINGMODEL.equals(levelString)) {
			if (!subTask.isDummy() && subTask.getPrice() != null) {
				this.addToMap(levelnumber,
						subTask.getPrice().getPricingModel(), subTask);
			}
		}
	}

	/**
	 * Handle the levels beneath the root level of the cost overview tree
	 * 
	 * @param levelNr
	 *            the level to handle
	 * @param values
	 *            the subtasks to process
	 * @param level1
	 *            level one option
	 * @param level2
	 *            level two option
	 * @param level3
	 *            level three option
	 * @param level4
	 *            level four option
	 * @return the children generated for this level using the provided list of
	 *         subTasks, these can be added to the cost overview tree by the
	 *         caller fo this method
	 */
	private List<PriceDescriptorNode> handleNextLevel(int levelNr,
			List<SubTask> values, String level1, String level2, String level3,
			String level4) {
		List<PriceDescriptorNode> childs = new ArrayList<PriceDescriptorNode>();
		if (levelNr == 2) {
			for (SubTask subTask : values) {
				mapLevel(level2, 2, subTask);
			}
			for (Entry<String, List<SubTask>> newEntry : mapLevel2.entrySet()) {
				double totalPrice = this.calculateTotalPrice(newEntry
						.getValue());
				PriceDescriptorNode child1 = new PriceDescriptorNode(
						newEntry.getKey(), totalPrice);
				List<PriceDescriptorNode> childs1 = this.handleNextLevel(3,
						newEntry.getValue(), level1, level2, level3, level4);
				child1.addChildren(childs1);
				childs.add(child1);
			}
			mapLevel2.clear();
		} else if (levelNr == 3) {
			for (SubTask subTask : values) {
				mapLevel(level3, 3, subTask);
			}
			for (Entry<String, List<SubTask>> newEntry : mapLevel3.entrySet()) {
				double totalPrice = this.calculateTotalPrice(newEntry
						.getValue());
				PriceDescriptorNode child1 = new PriceDescriptorNode(
						newEntry.getKey(), totalPrice);
				List<PriceDescriptorNode> childs1 = this.handleNextLevel(4,
						newEntry.getValue(), level1, level2, level3, level4);
				child1.addChildren(childs1);
				childs.add(child1);
			}
			mapLevel3.clear();
		} else if (levelNr == 4) {
			for (SubTask subTask : values) {
				mapLevel(level4, 4, subTask);
			}
			for (Entry<String, List<SubTask>> newEntry : mapLevel4.entrySet()) {
				double totalPrice = this.calculateTotalPrice(newEntry
						.getValue());
				PriceDescriptorNode child1 = new PriceDescriptorNode(
						newEntry.getKey(), totalPrice);
				childs.add(child1);
			}
			mapLevel4.clear();
		}
		return childs;
	}

	/**
	 * Calculate the total price of a list of subTasks
	 * 
	 * @param subs
	 *            the list of subTasks to calculate the total price for
	 * @return the total price of a list of subTasks
	 */
	private double calculateTotalPrice(List<SubTask> subs) {
		double returnValue = 0.0;
		for (SubTask subTask : subs) {
			Price p = subTask.getPrice();
			if (EC2Constants.RESERVED.equalsIgnoreCase(p.getPricingModel())) {
				ReservedPrice resP = (ReservedPrice) p;
				returnValue += (resP.getFixedPrice() + resP.getHourlyPrice());
			} else {
				returnValue += p.getHourlyPrice();
			}
		}
		return returnValue;
	}

	/**
	 * Add an item to the appropriate map
	 * 
	 * @param levelnumber
	 *            the number of the level that is being processed
	 * @param key
	 *            the value of the itemss
	 * @param subTask
	 *            the subTask corresponding to the given levelString
	 */
	private void addToMap(int levelNr, String key, SubTask sub) {
		if (levelNr == 1) {
			List<SubTask> list1 = mapLevel1.get(key);
			if (list1 == null) {
				list1 = new ArrayList<SubTask>();
			}
			list1.add(sub);
			mapLevel1.put(key, list1);
		}
		if (levelNr == 2) {
			List<SubTask> list2 = mapLevel2.get(key);
			if (list2 == null) {
				list2 = new ArrayList<SubTask>();
			}
			list2.add(sub);
			mapLevel2.put(key, list2);
		}
		if (levelNr == 3) {
			List<SubTask> list3 = mapLevel3.get(key);
			if (list3 == null) {
				list3 = new ArrayList<SubTask>();
			}
			list3.add(sub);
			mapLevel3.put(key, list3);
		}
		if (levelNr == 4) {
			List<SubTask> list4 = mapLevel4.get(key);
			if (list4 == null) {
				list4 = new ArrayList<SubTask>();
			}
			list4.add(sub);
			mapLevel4.put(key, list4);
		}
	}

}
