package be.ac.ua.broker.output.textual;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.csvreader.CsvWriter;

/**
 * This class is used as a node in the price tree that is generated using the
 * brokering results to represent a textual cost overview.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class PriceDescriptorNode {

	/**
	 * Children of the PriceDescriptorNode
	 */
	private List<PriceDescriptorNode> children = new ArrayList<PriceDescriptorNode>();

	/**
	 * String description of the node
	 */
	private String desc;

	/**
	 * Price associated with the node
	 */
	private Double price;

	/**
	 * Constructor for PriceDescriptorNode
	 * 
	 * @param description
	 *            string description of the price tree node
	 */
	public PriceDescriptorNode(final String description) {
		desc = description;
	}

	/**
	 * Constructor for PriceDescriptorNode
	 * 
	 * @param description
	 *            string description of the price tree node
	 * @param p
	 *            price associated with the price tree node
	 */
	public PriceDescriptorNode(final String description, final double p) {
		desc = description;
		price = p;
	}

	/**
	 * @return the children of the price tree node
	 */
	public List<PriceDescriptorNode> getChildren() {
		return children;
	}

	/**
	 * Add the provided list of child nodes to the price tree node
	 * 
	 * @param childs
	 *            list of children to add to the price tree node
	 */
	public void addChildren(List<PriceDescriptorNode> childs) {
		children.addAll(childs);
	}

	/**
	 * @return string description of the price tree node
	 */
	public String getDescription() {
		return desc;
	}

	/**
	 * @return price associated with the price tree node
	 */
	public Double getPrice() {
		return price;
	}

	/**
	 * Setter for the price associated with the price tree node
	 * 
	 * @param p
	 *            the price associated with the price tree node
	 */
	public void setPrice(final Double p) {
		price = p;
	}

	/**
	 * @return whether the price tree node has children
	 */
	public boolean hasChildren() {
		return !children.isEmpty();
	}

	/**
	 * Write the information of the price tree node to file using the provided
	 * CsvWriter. The information of the children is recursively written to file
	 * using the same writer. So, if this method is called for the root node of
	 * the price tree, the whole price overview will be written to file.
	 * 
	 * @param csvWriter
	 *            the writer to use to write the price tree node information to
	 *            file
	 */
	public void toFile(CsvWriter csvWriter) {
		String arr[] = { desc, String.valueOf(price) };
		try {
			csvWriter.writeRecord(arr);
		} catch (IOException e) {
			e.printStackTrace();
		}
		csvWriter.flush();
		for (PriceDescriptorNode child : children) {
			child.toFile(csvWriter);
		}
	}

}
