package be.ac.ua.broker.output.textual;

import java.io.PrintStream;
import java.util.List;
import java.util.Set;

import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.scheduling.workload.SchedulerResult;
import be.ac.ua.broker.scheduling.workload.SchedulerResultCollection;
import be.ac.ua.broker.utils.DateUtility;

/**
 * This class is used when the broker result is outputted in a textual manner,
 * it prints the generated schedules to a given outputStream in a structured
 * way.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class SchedulePrinter {

	/**
	 * The output stream to write to
	 */
	private PrintStream out;

	/**
	 * Constructor for SchedulePrinter
	 * 
	 * @param output
	 *            the outputStream to write the broker schedules to
	 */
	public SchedulePrinter(PrintStream output) {
		out = output;
	}

	/**
	 * Write the schedules contained in the provided schedulerResultCollection
	 * to the provided printStream.
	 * 
	 * @param results
	 *            the schedulerResultCollection broker results of which to write
	 *            the information to the outputStream
	 */
	public void print(final SchedulerResultCollection results) {
		out.println("@@@@@@@@@@@@@@@@@@@@@@");
		out.println("@ EC2BROKER SCHEDULE @");
		out.println("@@@@@@@@@@@@@@@@@@@@@@");
		out.println("\t Start=" + DateUtility.toTime(results.getStartDate()));
		out.println("\t End=" + DateUtility.toTime(results.getEndDate()));
		Set<String> regions = EC2Specifications.getRegionMapping().keySet();
		for (String reg : regions) {
			List<SchedulerResult> schedules = results.get(reg);
			if (schedules != null) {
				out.println("============================");
				out.println("Region: " + reg);
				out.println("============================");
				for (SchedulerResult schedulerResult : schedules) {
					InstanceSpecification instSpec = schedulerResult
							.getInstanceSpecification();
					out.println("#############################################################");
					out.println("Instance: " + instSpec.getIdentifier());
					out.println("#############################################################");
					this.printSchedulerResult(schedulerResult);
				}
			}
		}
		out.flush();
		out.close();
	}

	/**
	 * Write the information of a single geographical region, represented by a
	 * schedulerResult to the outputStream
	 * 
	 * @param schedulerResult
	 *            the broker schedules of a single geographical region
	 */
	private void printSchedulerResult(final SchedulerResult schedulerResult) {
		List<ScheduledTaskCollection> scheduledTaskColl = schedulerResult
				.getScheduledTaskCollections();
		int counter = 0;
		for (ScheduledTaskCollection scheduledTaskCollection : scheduledTaskColl) {
			String pricingModel = scheduledTaskCollection.getPricingModel();
			out.print(pricingModel + " resource [" + counter + "] ");
			List<SubTask> subTask = scheduledTaskCollection.getSubTasks();
			for (SubTask sub : subTask) {
				this.printSubTask(sub);
			}
			out.print("\n");
			out.flush();
			counter++;
		}

	}

	/**
	 * Write the information of a single subTask to the printStream
	 * 
	 * @param sub
	 *            the subTask to write the information for to the printStream
	 */
	private void printSubTask(SubTask sub) {
		out.print("<");
		if (sub.isDummy()) {
			out.print("0");
			out.flush();
		} else {
			out.print("X:" + sub.getSpecification().getName() + ":"
					+ sub.getTaskHour());
			out.flush();
		}
		out.print(">");

	}

}
