package be.ac.ua.broker.scheduling.region;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.pricing.Price;
import be.ac.ua.broker.input.pricing.PricingManager;
import be.ac.ua.broker.input.tasks.Task;
import be.ac.ua.broker.input.tasks.TaskCollection;
import be.ac.ua.broker.input.tasks.TaskSpecification;
import be.ac.ua.broker.utils.InstanceNotAvailableException;
import be.ac.ua.broker.utils.InvalidInputException;

/**
 * This class is used during the scheduling phase of the broker. Every task is
 * first appointed to a region where running the load is expected to be the
 * least costly. The region allocation step will appoint a task that is not
 * assigned to a geographical region by the user to a region.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class RegionAllocator {

	/**
	 * The collection of tasks to process
	 */
	private TaskCollection tasks;

	/**
	 * Constructor for RegionAllocator
	 * 
	 * @param coll
	 *            the collection of tasks to process, all the tasks of the
	 *            collection are assigned to an appropriate geographical region
	 */
	public RegionAllocator(TaskCollection coll) {
		tasks = coll;
	}

	/**
	 * Appoint every task of the taskCollection of the RegionAllocator to a
	 * region and return a list of regionWorkloadCollections, which is a
	 * collection of tasks associated with a certain geographical region.
	 * 
	 * @return a list of regionWorkloadCollections
	 */
	public List<RegionWorkloadCollection> allocateRegions() {
		Map<String, RegionWorkloadCollection> mapping = new HashMap<String, RegionWorkloadCollection>();
		for (Task t : tasks.getTasks()) {
			String region;
			if (t.hasRegionAssigned()) {
				region = t.getTaskSpecification().getInstanceSpecification()
						.getRegion();
			} else {
				TaskSpecification spec = t.getTaskSpecification();
				if (spec.isSpotAllowed()) {
					region = this.determineSpotCheapestRegion(spec
							.getInstanceSpecification(), tasks.getStartDate(),
							t.getTaskSpecification().getDeadline());
				} else {
					region = this.determineCheapestRegion(spec
							.getInstanceSpecification());
				}
			}
			try {
				t.setRegion(region);
			} catch (InvalidInputException e) {
				e.printStackTrace();
			}
			RegionWorkloadCollection regWorkloadColl = mapping.get(region);
			if (regWorkloadColl == null) {
				regWorkloadColl = new RegionWorkloadCollection(region);
			}
			regWorkloadColl.add(t.getTaskSpecification()
					.getInstanceSpecification(), t.getWorkload());
			mapping.put(region, regWorkloadColl);
		}
		return new ArrayList<RegionWorkloadCollection>(mapping.values());
	}

	/**
	 * Determine the region to be the cheapest for running a task with a given
	 * instance specification for the instance to run the task on. This method
	 * is used when the task is marked as not spot-enabled which means the user
	 * determined that the task is not suited to be run on spot instances.
	 * 
	 * @param instanceSpecification
	 *            the specification of the instance the task needs to be run on
	 * @return the string representation of the geographical region expected to
	 *         be the cheapest for a task that has to run on an instance with
	 *         the given specifications
	 */
	private String determineCheapestRegion(
			InstanceSpecification instanceSpecification) {
		Set<String> regions = EC2Specifications.getRegionMapping().keySet();
		String region = null;
		try {
			double smallestAmount = Double.MAX_VALUE;
			for (String reg : regions) {
				Price ondemand = PricingManager
						.getOnDemandPrice(instanceSpecification);
				if (ondemand.getHourlyPrice() < smallestAmount) {
					smallestAmount = ondemand.getHourlyPrice();
					region = reg;
				}
			}
		} catch (InstanceNotAvailableException e) {
			e.printStackTrace();
		}
		return region;
	}

	/**
	 * Determine the region to be the cheapest for running a task with a given
	 * instance specification for the instance to run the task on. This method
	 * is used when the task is marked as spot-enabled which means the user
	 * determined that the task is suited to be run on spot instances.
	 * 
	 * @param instanceSpecification
	 *            the specification of the instance the task needs to be run on
	 * @param start
	 *            the start time of the scheduling period
	 * @param end
	 *            the end time of the scheduling period
	 * @return the string representation of the geographical region expected to
	 *         be the cheapest for a task that has to run on an instance with
	 *         the given specifications
	 */
	private String determineSpotCheapestRegion(
			InstanceSpecification instanceSpecification, Date start, Date end) {
		Set<String> regions = EC2Specifications.getRegionMapping().keySet();
		String region = null;
		try {
			double smallestAmount = Double.MAX_VALUE;
			for (String reg : regions) {
				DescriptiveStatistics statistics = PricingManager
						.getStatisticsSpotPrice(instanceSpecification, start,
								end);
				if (statistics.getPercentile(75.0) < smallestAmount) {
					smallestAmount = statistics.getPercentile(75.0);
					region = reg;
				}
			}
		} catch (InstanceNotAvailableException e) {
			e.printStackTrace();
		}
		return region;
	}

}
