package be.ac.ua.broker.scheduling.region;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.tasks.Workload;
import be.ac.ua.broker.scheduling.workload.SchedulerResult;
import be.ac.ua.broker.scheduling.workload.TaskScheduler;
import be.ac.ua.broker.utils.InvalidInputException;

/**
 * This class is used to represent the workload that is appointed to a certain
 * geographical region by the RegionAllocator. It's possible the task was task
 * was already appointed to the region by the user of the broker, but it's also
 * possible that the RegionAllocator determined the region to be expected to be
 * the cheapest one to run the workload in.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class RegionWorkloadCollection {

	/**
	 * The geographical region the workloads are appointed to be run in
	 */
	private String region;

	/**
	 * Mapping of an instance type string representation to the workload
	 * associated with this instance type
	 */
	private Map<String, InstanceWorkloadCollection> mapping = new HashMap<String, InstanceWorkloadCollection>();

	/**
	 * Constructor for RegionWorkloadCollection
	 * 
	 * @param reg
	 *            the geographical region the workload is associated with
	 */
	public RegionWorkloadCollection(final String reg) {
		region = reg;
	}

	/**
	 * Add a workload that is associated with the region of this
	 * RegionWorkloadCollection, that is associated with a provided instance
	 * specification
	 * 
	 * @param spec
	 *            the instance specification of the instance the provided
	 *            workload has to run on
	 * @param workload
	 *            the workload
	 */
	public void add(InstanceSpecification spec, Workload workload) {
		InstanceWorkloadCollection instColl = mapping.get(spec.getIdentifier());
		if (instColl == null) {
			instColl = new InstanceWorkloadCollection(spec);
		}
		instColl.add(workload);
		mapping.put(spec.getIdentifier(), instColl);
	}

	/**
	 * @return the geographical region this workload collection is associated
	 *         with
	 */
	public String getRegion() {
		return region;
	}

	/**
	 * Schedule the workloads of this collection
	 * 
	 * @param wm1
	 *            whether workload model one is being used
	 * @param version
	 *            the scheduling option that is begin used
	 * @return the generated list of schedulerResults
	 * @throws InvalidInputException
	 *             exception thrown when an invalid scheduling option is
	 *             provided
	 */
	public List<SchedulerResult> schedule(boolean wm1, String version)
			throws InvalidInputException {
		List<SchedulerResult> results = new ArrayList<SchedulerResult>();
		for (InstanceWorkloadCollection wlColl : mapping.values()) {
			TaskScheduler scheduler = new TaskScheduler(wlColl, wm1);
			results.add(scheduler.schedule(version));
		}
		return results;
	}

	/**
	 * Print functionality for the regionWorkloadCollection
	 */
	public void print() {
		System.out.println("------------------------");
		System.out.println("REGIONWORKLOADCOLLECTION");
		System.out.println("------------------------");
		System.out.println("Region=" + region);
		for (Entry<String, InstanceWorkloadCollection> entry : mapping
				.entrySet()) {
			System.out.println("Instance=" + entry.getKey());
			entry.getValue().print();
		}
	}

}
