package be.ac.ua.broker.scheduling.workload;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import be.ac.ua.broker.input.pricing.Price;
import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.utils.DateUtility;

/**
 * This class represents a schedule generated for a single instance, it contains
 * a list of subTasks. During the allocation phase of the broker a pricing model
 * will be associated with the scheduledTaskCollection.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class ScheduledTaskCollection {

	/**
	 * List of subTasks that are scheduled on the single instance
	 */
	private List<SubTask> subTasks = new ArrayList<SubTask>();

	/**
	 * The pricing model that is associated with the scheduledTaskCollection
	 * during the allocation phase of the broker.
	 */
	private String pricingModel;

	/**
	 * @return the list of subtasks associated with the scheduledTaskCollection
	 */
	public List<SubTask> getSubTasks() {
		return subTasks;
	}

	/**
	 * Add a subtask to the scheduledTaskCollection
	 * 
	 * @param subT
	 *            the subTask to add to the collection
	 */
	public void add(SubTask subT) {
		subTasks.add(subT);
	}

	/**
	 * Merge the provided scheduledTaskCollection with this
	 * scheduledTaskCollection,
	 * 
	 * @param newPart
	 *            the scheduledTaskCollection to merge with the current one
	 */
	public void merge(ScheduledTaskCollection newPart) {
		this.subTasks.addAll(newPart.getSubTasks());
	}

	/**
	 * @return the number of subTasks in the scheduledTaskCollection
	 */
	public int getNumberOfSubTasks() {
		return this.subTasks.size();
	}

	/**
	 * Add a given number of dummySubTasks after the subtasks already in the
	 * collection
	 * 
	 * @param postFillAmount
	 *            the number of dummySubTasks to post append
	 */
	public void postFill(int postFillAmount) {
		for (int i = 0; i < postFillAmount; i++) {
			this.subTasks.add(new DummySubTask());
		}
	}

	/**
	 * Add a given number of dummySubTasks in front of the subtasks already in
	 * the collection
	 * 
	 * @param preFillAmount
	 *            the number of dummySubTasks to pre append to the collection
	 */
	public void preFill(int preFillAmount) {
		for (int i = 0; i < preFillAmount; i++) {
			this.subTasks.add(0, new DummySubTask());
		}
	}

	/**
	 * @return the spot-enabled subtasks that are part of the
	 *         scheduledTaskCollection
	 */
	public ScheduledTaskCollection getSpotEnabledSubTasks() {
		ScheduledTaskCollection coll = new ScheduledTaskCollection();
		for (SubTask subTask : subTasks) {
			if (subTask.isSpotEnabled()) {
				coll.add(subTask);
			} else {
				coll.add(new DummySubTask());
			}
		}
		return coll;
	}

	/**
	 * @return the not spot-enabled subtasks that are part of the
	 *         scheduledTaskCollection
	 */
	public ScheduledTaskCollection getNonSpotEnabledSubTasks() {
		ScheduledTaskCollection coll = new ScheduledTaskCollection();
		for (SubTask subTask : subTasks) {
			if (!subTask.isSpotEnabled()) {
				coll.add(subTask);
			} else {
				coll.add(new DummySubTask());
			}
		}
		return coll;
	}

	/**
	 * @return whether all the subTasks of the scheduledTaskCollection are spot
	 *         enabled
	 */
	public boolean onlySpotEnabledSubTasks() {
		boolean onlySpotEnabled = true;
		for (SubTask subTask : subTasks) {
			if (!subTask.isDummy() && !subTask.isSpotEnabled()) {
				onlySpotEnabled = false;
			}
		}
		return onlySpotEnabled;
	}

	/**
	 * @return whether all the subTasks of the scheduledTaskCollection are not
	 *         spot enabled
	 */
	public boolean onlyNonSpotEnabledTasks() {
		boolean onlyNonSpotEnabled = true;
		for (SubTask subTask : subTasks) {
			if (!subTask.isDummy() && subTask.isSpotEnabled()) {
				onlyNonSpotEnabled = false;
			}
		}
		return onlyNonSpotEnabled;
	}

	/**
	 * @return the pricing model that is appointed to the
	 *         scheduledTaskCollection during the allocation phase of the
	 *         broker process
	 */
	public String getPricingModel() {
		return pricingModel;
	}

	/**
	 * Setter for the pricing model appointed to the instance associated with
	 * the scheduledTaskCollection ]
	 * 
	 * @param model
	 *            the pricing model that has to be used
	 */
	public void setPricingModel(final String model) {
		pricingModel = model;
	}

	/**
	 * Getter for the number of actual task hours, which equals the size of the
	 * list with subtasks minus the amount of dummy subTasks in the list
	 * 
	 * @return the number of subtasks scheduled on the instance
	 */
	public int getNumberOfTaskHours() {
		int nrOfHours = 0;
		for (SubTask subTask : subTasks) {
			if (!subTask.isDummy()) {
				nrOfHours++;
			}
		}
		return nrOfHours;
	}

	/**
	 * Setter for the price of all the subTasks of the scheduledTaskCollection,
	 * this can be used for the on-demand and reserved pricing models since all
	 * subtasks hold the same price in that case. When the spot pricing model is
	 * used, this method can't be used.
	 * 
	 * @param price
	 *            the price to set for all subtasks of the
	 *            scheduledTaskCollection
	 */
	public void setPricesSubTasks(Price price) {
		for (SubTask subTask : subTasks) {
			if (!subTask.isDummy()) {
				subTask.setPrice(price);
			}
		}
	}

	/**
	 * Associate time stamps with all the subTasks by providing a start time and
	 * using every subTask in the list as a one hour long time frame.
	 * 
	 * @param start
	 *            the start time of the scheduling period
	 */
	public void fillInTimeStampsSubTasks(Date start) {
		int hoursToAdd = 0;
		for (SubTask subTask : subTasks) {
			if (!subTask.isDummy()) {
				Date startDate = new Date(start.getTime() + hoursToAdd
						* DateUtility.HOURINMILLISECONDS);
				Date endDate = new Date(start.getTime() + (hoursToAdd + 1)
						* DateUtility.HOURINMILLISECONDS);
				subTask.setTimePeriod(startDate, endDate);
			}
			hoursToAdd++;
		}
	}

}
