package be.ac.ua.broker.scheduling.workload;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This class is used to represent a collection of schedulerResults. It's the
 * output of the allocation phase of the broker and contains the schedules
 * tagged with time and cost information.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class SchedulerResultCollection {

	/**
	 * The start time of the scheduling period
	 */
	public static Date startDate;

	/**
	 * The end time of the scheduling period
	 */
	public static Date endDate;

	/**
	 * Mapping of a geographical region to a list of schedulerResults each of
	 * them contains the schedule for a specific instance specification
	 */
	private Map<String, List<SchedulerResult>> resultMap = new HashMap<String, List<SchedulerResult>>();

	/**
	 * Constructor for SchedulerResultCollection
	 * 
	 * @param start
	 *            start time of the scheduling period
	 * @param end
	 *            end time of the scheduling period
	 */
	public SchedulerResultCollection(Date start, Date end) {
		startDate = start;
		endDate = end;
	}

	/**
	 * Add a schedulerResult to the collection
	 * 
	 * @param schedule
	 *            the schedulerResult to add to the collection
	 */
	public void add(SchedulerResult schedule) {
		String key = schedule.getRegion();
		List<SchedulerResult> results = resultMap.get(key);
		if (results == null) {
			results = new ArrayList<SchedulerResult>();
		}
		results.add(schedule);
		resultMap.put(key, results);
	}

	/**
	 * Add a list of schedulerResults to the collection
	 * 
	 * @param schedules
	 *            the schedulerResults to add to the collection
	 */
	public void addAll(List<SchedulerResult> schedules) {
		for (SchedulerResult schedulerResult : schedules) {
			this.add(schedulerResult);
		}
	}

	/**
	 * Getter for all the schedules of a certain geographical region
	 * 
	 * @param region
	 *            the region to get the schedules for
	 * @return the list of schedules associated with the given region
	 */
	public List<SchedulerResult> get(String region) {
		return resultMap.get(region);
	}

	/**
	 * @return the start time of the scheduling period
	 */
	public Date getStartDate() {
		return startDate;
	}

	/**
	 * @return the end time of the scheduling period
	 */
	public Date getEndDate() {
		return endDate;
	}

}
