package be.ac.ua.broker.scheduling.workload;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.tasks.Workload;
import be.ac.ua.broker.input.tasks.WorkloadWM1;
import be.ac.ua.broker.input.tasks.WorkloadWM2;
import be.ac.ua.broker.scheduling.region.InstanceWorkloadCollection;
import be.ac.ua.broker.scheduling.workload.wm1.BasicTHOptimizer;
import be.ac.ua.broker.scheduling.workload.wm1.SuperSpotTHOptimizer;
import be.ac.ua.broker.scheduling.workload.wm1.SuperTHOptimizer;
import be.ac.ua.broker.scheduling.workload.wm2.BasicOptimizer;
import be.ac.ua.broker.scheduling.workload.wm2.SuperOptimizer;
import be.ac.ua.broker.utils.InputVerifier;
import be.ac.ua.broker.utils.InvalidInputException;

/**
 * This class is used to represent the scheduler that uses an optimizer to
 * determine the schedule that minimizes the total cost. This class is used
 * during the scheduling phase of the broker process.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class TaskScheduler {

	/**
	 * The list of workloads that need to be scheduled
	 */
	private List<Workload> tasks;

	/**
	 * The specification of the instance on which the workloads are being
	 * scheduled
	 */
	private InstanceSpecification specification;

	/**
	 * The start time of the scheduling period
	 */
	private Date startDate;

	/**
	 * Whether workload model one is being used
	 */
	private Boolean workloadmodel1;

	/**
	 * Constructor for TaskScheduler
	 * 
	 * @param wlColl
	 *            the workload collection that is associated with a single
	 *            instance specification that needs to be scheduled
	 * @param wm1
	 *            whether workload model one is being used
	 */
	public TaskScheduler(InstanceWorkloadCollection wlColl, boolean wm1) {
		tasks = wlColl.getWorkload();
		specification = wlColl.getSpecification();
		startDate = wlColl.getStartDate();
		workloadmodel1 = wm1;
	}

	/**
	 * Perform the scheduling phase of the broker process
	 * 
	 * @param VERSION
	 *            the scheduler option that needs to used by the scheduler
	 * @return the result of the scheduling phase of the broker
	 * @throws InvalidInputException
	 *             exception thrown when the provided scheduling option is not
	 *             supported by the broker according to the inputVerifier
	 */
	public SchedulerResult schedule(String VERSION)
			throws InvalidInputException {
		InputVerifier.schedulingVersionExists(VERSION);
		SchedulerResult returnValue = new SchedulerResult(specification);
		System.out.println("Sorting tasks");
		this.sortEDF();
		System.out.println("Adding tasks to optimizer");
		List<ScheduledTaskCollection> taskColls;
		if (workloadmodel1) {
			Optimizer<WorkloadWM1> optimizer = new BasicTHOptimizer();
			if (VERSION.equalsIgnoreCase(SchedulerOptions.OPTIMIZED)) {
				optimizer = new SuperTHOptimizer();
			} else if (VERSION.equalsIgnoreCase(SchedulerOptions.SPOTOPTIMIZED)) {
				optimizer = new SuperSpotTHOptimizer();
			}
			for (Workload t : tasks) {
				optimizer.add((WorkloadWM1) t);
			}
			System.out.println("Determine optimal solution");
			taskColls = optimizer.determineOptimal();
		} else {
			Optimizer<WorkloadWM2> optimizer = new BasicOptimizer();
			if (VERSION.equalsIgnoreCase(SchedulerOptions.OPTIMIZED)) {
				optimizer = new SuperOptimizer(startDate, specification, false);
			} else if (VERSION.equalsIgnoreCase(SchedulerOptions.SPOTOPTIMIZED)) {
				optimizer = new SuperOptimizer(startDate, specification, true);
			}
			for (Workload t : tasks) {
				optimizer.add((WorkloadWM2) t);
			}
			System.out.println("Determine optimal solution");
			taskColls = optimizer.determineOptimal();
		}
		System.out.println("Return result");
		for (ScheduledTaskCollection scheduledTaskCollection : taskColls) {
			returnValue.addScheduledTaskCollection(scheduledTaskCollection);
		}
		return returnValue;
	}

	/**
	 * Sort the tasks that need to be scheduled in an earliest deadline first
	 * manner.
	 */
	private void sortEDF() {
		Collections.sort(tasks);
	}

}
