package be.ac.ua.broker.scheduling.workload.wm1;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.input.tasks.WorkloadWM1;
import be.ac.ua.broker.scheduling.workload.Optimizer;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;

/**
 * This class is used to perform the basic scheduling for workload model one
 * tasks. Basic scheduling is the naive method in which every task is scheduled
 * on a separate resource. The task hours are scheduled consecutively starting
 * at the beginning of the schedule.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class BasicTHOptimizer implements Optimizer<WorkloadWM1> {

	/**
	 * List of deadlines of the tasks being scheduled
	 */
	private List<Integer> deadlines = new ArrayList<Integer>();

	/**
	 * List of tasks of workload model one being scheduled
	 */
	private List<WorkloadWM1> tasks = new ArrayList<WorkloadWM1>();

	/**
	 * The greatest deadline of all the tasks being scheduled
	 */
	private int greatestDeadline = 0;

	/**
	 * The created basic schedule
	 */
	private ArrayList<ScheduledTaskCollection> returnList;

	@Override
	public void add(WorkloadWM1 wl) {
		int hoursAvail = wl.getHoursTimeFrame();
		if (!deadlines.contains(hoursAvail)) {
			deadlines.add(hoursAvail);
		}
		tasks.add(wl);
		if (hoursAvail > greatestDeadline) {
			greatestDeadline = hoursAvail;
		}
	}

	@Override
	public List<ScheduledTaskCollection> determineOptimal() {
		returnList = new ArrayList<ScheduledTaskCollection>();
		Collections.sort(deadlines);
		for (int deadline : deadlines) {
			List<WorkloadWM1> wls = getWorkLoadsForDeadline(deadline);
			for (WorkloadWM1 workloadTH : wls) {
				this.scheduleSubTasks(workloadTH);
			}
		}
		return returnList;
	}

	/**
	 * Schedule the subtasks of the provided task of workload model one
	 * 
	 * @param wl
	 *            the task of workload model one of which the subtasks need to
	 *            be scheduled
	 */
	private void scheduleSubTasks(WorkloadWM1 wl) {
		int deadline = wl.getHoursTimeFrame();
		int currentTaskToSchedule = 0;
		List<SubTask> subtasks = wl.getTasks().getWorkload();
		while (currentTaskToSchedule != subtasks.size()) {
			ScheduledTaskCollection scheduledTaskCollection = new ScheduledTaskCollection();
			for (int i = 0; i < deadline; i++) {
				if (currentTaskToSchedule < subtasks.size()) {
					scheduledTaskCollection.add(subtasks
							.get(currentTaskToSchedule));
					currentTaskToSchedule++;
				}
			}
			returnList.add(scheduledTaskCollection);
		}
	}

	/**
	 * Getter for the tasks begin scheduled which have a given deadline.
	 * 
	 * @param deadline
	 *            the deadline of which we want to get the tasks.
	 * @return the list of workloads associated with the given deadline
	 */
	public List<WorkloadWM1> getWorkLoadsForDeadline(int deadline) {
		List<WorkloadWM1> returnList = new ArrayList<WorkloadWM1>();
		for (WorkloadWM1 wl : tasks) {
			if (wl.getHoursTimeFrame() == deadline) {
				returnList.add(wl);
			}
		}
		return returnList;
	}

}
