package be.ac.ua.broker.scheduling.workload.wm2;

import java.util.ArrayList;
import java.util.List;

import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.input.tasks.WorkloadWM2;
import be.ac.ua.broker.scheduling.workload.DummySubTask;
import be.ac.ua.broker.scheduling.workload.Optimizer;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;

/**
 * This class is used to perform the basic scheduling for workload model two
 * tasks. Basic scheduling is the naive method in which every task is scheduled
 * on a separate resource. The task hours are scheduled consecutively starting
 * at the beginning of the schedule.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class BasicOptimizer implements Optimizer<WorkloadWM2> {

	/**
	 * A list of subTasks is associated with every hour of the task, so the list
	 * of this list represents a single tasks. The surrounding list makes that
	 * we have a list of tasks of workload model two.
	 */
	private List<List<List<SubTask>>> subtasks = new ArrayList<List<List<SubTask>>>();

	/**
	 * For every task we store the number of hours that can be added to the
	 * length of the task in order to reach the associated deadline.
	 */
	private List<Integer> hoursToAdd = new ArrayList<Integer>();

	@Override
	public void add(WorkloadWM2 wl) {
		subtasks.add(wl.getWorkload());
		hoursToAdd.add(wl.getHoursTimeFrame());
	}

	@Override
	public List<ScheduledTaskCollection> determineOptimal() {
		List<List<SubTask>> consolidatedtasks = subtasks.get(0);
		for (int i = 1; i < subtasks.size(); i++) {
			List<List<SubTask>> otherTask = subtasks.get(i);
			consolidatedtasks = this.addLists(consolidatedtasks, otherTask);
		}
		int nrOfInstances = this.getNrOfInstances(consolidatedtasks);
		List<ScheduledTaskCollection> scheduledTasks = new ArrayList<ScheduledTaskCollection>();
		for (int i = 0; i < nrOfInstances; i++) {
			ScheduledTaskCollection taskColl = new ScheduledTaskCollection();
			for (List<SubTask> l : consolidatedtasks) {
				try {
					SubTask subT = l.get(i);
					taskColl.add(subT);
				} catch (java.lang.IndexOutOfBoundsException e) {
					taskColl.add(new DummySubTask());
				}
			}
			scheduledTasks.add(taskColl);
		}
		return scheduledTasks;
	}

	/**
	 * Merge the two provided lists of subTasks
	 * 
	 * @param task0
	 *            the first task to merge
	 * @param task1
	 *            the second task to merge
	 * @return the merged task
	 */
	private List<List<SubTask>> addLists(List<List<SubTask>> task0,
			List<List<SubTask>> task1) {
		if (task0.size() > task1.size()) {
			for (int i = 0; i < task1.size(); i++) {
				task0.get(i).addAll(task1.get(i));
			}
			return task0;
		} else {
			for (int i = 0; i < task0.size(); i++) {
				task1.get(i).addAll(task0.get(i));
			}
			return task1;
		}
	}

	/**
	 * Get the maximum/actual number of instances that is needed to run the
	 * provided task, or in other words what's the maximum load across all the
	 * different task hours of the task.
	 * 
	 * @param temp
	 *            the task to determine the maximum load of
	 * @return the number of instances needed in order to be able to run the
	 *         provided task
	 */
	private int getNrOfInstances(List<List<SubTask>> temp) {
		int returnValue = 0;
		for (List<SubTask> list : temp) {
			if (list.size() > returnValue) {
				returnValue = list.size();
			}
		}
		return returnValue;
	}

}