package be.ac.ua.broker.scheduling.workload.wm2;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.input.tasks.WorkloadWM2;
import be.ac.ua.broker.scheduling.workload.DummySubTask;
import be.ac.ua.broker.scheduling.workload.Optimizer;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.utils.CombinationGenerator;

/**
 * This class is used to perform the optimized/spot-optimized scheduling for
 * workload model two tasks. The RealOptimizer is used to perform the scheduling
 * within a bucket. The tasks are evenly distributed across buckets, to make the
 * algorithm feasible.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class RealOptimizer implements Optimizer<WorkloadWM2> {

	/**
	 * A list of subTasks is associated with every hour of the task, so the list
	 * of this list represents a single tasks. The surrounding linkedlist makes
	 * that we have a list of tasks of workload model two.
	 */
	private LinkedList<List<List<SubTask>>> subtasks = new LinkedList<List<List<SubTask>>>();

	/**
	 * For every task we store the number of hours that can be added to the
	 * length of the task in order to reach the associated deadline.
	 */
	private LinkedList<Integer> hoursToAdd = new LinkedList<Integer>();

	/**
	 * For every task we have a combination generator which is used to iterate
	 * all the possible divisions of the task hours within the time period from
	 * the beginning of the scheduling period until the deadline of the
	 * corresponding task.
	 */
	private List<CombinationGenerator> mapping = new ArrayList<CombinationGenerator>();

	/**
	 * The largest number of hours available for any task being scheduled this
	 * corresponds to the timer period of the task with the largest deadline.
	 */
	private int maxHours = 0;

	/**
	 * Whether spot-optimized scheduling is performed.
	 */
	private boolean useSpotOptimalization;

	/**
	 * The latest combination possibility that has been considered
	 */
	private BigInteger lastPrinted = new BigInteger("0");

	/**
	 * Constructor for RealOptimizer
	 * 
	 * @param use_spot_optimalization
	 *            whether spot-optimized scheduling is performed.
	 */
	public RealOptimizer(boolean use_spot_optimalization) {
		useSpotOptimalization = use_spot_optimalization;
	}

	@Override
	public void add(WorkloadWM2 wl) {
		int hoursAvail = wl.getHoursTimeFrame();
		List<List<SubTask>> arr = wl.getWorkload();
		int extra = hoursAvail - arr.size();
		if (useSpotOptimalization) {
			if (wl.getTaskSpecification().isSpotAllowed()) {
				subtasks.addLast(arr);
				hoursToAdd.addLast(extra);
			} else {
				subtasks.addFirst(arr);
				hoursToAdd.addFirst(extra);
			}
		} else {
			subtasks.addLast(arr);
			hoursToAdd.addLast(extra);
		}
		if (hoursAvail > maxHours) {
			maxHours = hoursAvail;
		}
	}

	@Override
	public List<ScheduledTaskCollection> determineOptimal() {
		int counter = 0;
		for (List<List<SubTask>> list : subtasks) {
			if (!list.isEmpty()) {
				CombinationGenerator possibilities = this.createPossibilities(
						list, hoursToAdd.get(counter));
				mapping.add(counter, possibilities);
				counter++;
			}
		}
		List<List<SubTask>> consolidatedtasks = this.findOptimalDivision();
		int nrOfInstances = this.getNrOfInstances(consolidatedtasks);
		List<ScheduledTaskCollection> scheduledTasks = new ArrayList<ScheduledTaskCollection>();
		for (int i = 0; i < nrOfInstances; i++) {
			ScheduledTaskCollection taskColl = new ScheduledTaskCollection();
			for (List<SubTask> l : consolidatedtasks) {
				try {
					SubTask subT = l.get(i);
					taskColl.add(subT);
				} catch (java.lang.IndexOutOfBoundsException e) {
					taskColl.add(new DummySubTask());
				}
			}
			scheduledTasks.add(taskColl);
		}
		return scheduledTasks;
	}

	/**
	 * Create the combinationGenerator for the given task, knowing how many
	 * hours are left after scheduling the task to reach its deadline.
	 * 
	 * @param list
	 *            the task being scheduled
	 * @param hoursToAdd
	 *            the hours that are left until the deadline
	 * @return the combinationGenerator for the provided task
	 */
	private CombinationGenerator createPossibilities(List<List<SubTask>> list,
			int hoursToAdd) {
		int[] sizes = new int[list.size()];
		int counter = 0;
		for (List<SubTask> i : list) {
			sizes[counter] = i.size();
			counter++;
		}
		CombinationGenerator x = new CombinationGenerator(list.size()
				+ hoursToAdd, list.size());
		x.getNext();
		return x;
	}

	/**
	 * @return the optimal schedule for the subtasks
	 */
	private List<List<SubTask>> findOptimalDivision() {
		List<List<SubTask>> optimal = new ArrayList<List<SubTask>>();
		int smallest = Integer.MAX_VALUE;
		while (this.hasNextPossibility()) {
			List<List<SubTask>> sum = this.getNextPossibility();
			int nrOfInstances = this.getNrOfInstances(sum);
			if (nrOfInstances < smallest) {
				smallest = nrOfInstances;
				optimal = sum;
			} else {
				sum = null;
			}
		}
		return optimal;
	}

	/**
	 * @return the next schedule possibility (iterate over all of them)
	 */
	private List<List<SubTask>> getNextPossibility() {
		List<List<SubTask>> consolidatedTasks = new ArrayList<List<SubTask>>();
		for (int i = 0; i < maxHours; i++) {
			consolidatedTasks.add(new ArrayList<SubTask>());
		}
		int taskCounter = 0;
		for (CombinationGenerator comb : mapping) {
			int[] division = comb.getCurrent();
			int subTaskCounter = 0;
			for (int i : division) {
				List<SubTask> tasks = consolidatedTasks.get(i);
				List<SubTask> subsToAdd = subtasks.get(taskCounter).get(
						subTaskCounter);
				tasks.addAll(subsToAdd);
				subTaskCounter++;
			}
			taskCounter++;
		}
		this.goToNextPossibility(0);
		return consolidatedTasks;
	}

	/**
	 * Go to the next schedule possibility
	 * 
	 * @param listNrFromBack
	 *            index in the list of combinationGenerators which one has to be
	 *            changed starting to number them from the last one
	 */
	private void goToNextPossibility(int listNrFromBack) {
		if (mapping.size() - 1 - listNrFromBack >= 0) {
			CombinationGenerator list = mapping.get(mapping.size() - 1
					- listNrFromBack);
			if (list.hasMore()) {
				list.getNext();
			} else {
				BigInteger bigInt = mapping.get(0).getNumLeft();
				if (!bigInt.equals(lastPrinted)) {
					lastPrinted = bigInt;
				}
				this.goToNextPossibility(listNrFromBack + 1);
			}
		}
	}

	/**
	 * @return whether there are still more possible schedules to iterate over
	 */
	private boolean hasNextPossibility() {
		return (!mapping.isEmpty() && mapping.get(0).hasMore());
	}

	/**
	 * Determine the highest load/number of instances needed for the provided
	 * schedule to run on EC2
	 * 
	 * @param sum
	 *            the possible schedule
	 * @return the number of instances needed for the provided schedule to run
	 *         on EC2
	 */
	private int getNrOfInstances(List<List<SubTask>> temp) {
		int returnValue = 0;
		for (List<SubTask> list : temp) {
			if (list.size() > returnValue) {
				returnValue = list.size();
			}
		}
		return returnValue;
	}

}
