package be.ac.ua.broker.scheduling.workload.wm2;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import be.ac.ua.broker.input.pricing.InstanceSpecification;
import be.ac.ua.broker.input.tasks.SubTask;
import be.ac.ua.broker.input.tasks.TaskSpecification;
import be.ac.ua.broker.input.tasks.WorkloadWM2;
import be.ac.ua.broker.scheduling.workload.Optimizer;
import be.ac.ua.broker.scheduling.workload.ScheduledTaskCollection;
import be.ac.ua.broker.utils.CombinationGenerator;

/**
 * This class is used to perform the optimized/spot-optimized scheduling for
 * workload model two tasks. The RealOptimizer is used to perform the scheduling
 * within a bucket. The tasks are evenly distributed across buckets, to make the
 * algorithm feasible.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class SuperOptimizer implements Optimizer<WorkloadWM2> {

	/**
	 * This class is used to split a task in evenly distributed parts for the
	 * created buckets.
	 * 
	 * @author Vermeersch Kurt
	 * @date May 2011
	 */
	class OptimizerTask {

		/**
		 * Task of workload model two
		 */
		private List<List<SubTask>> task;

		/**
		 * The deadline of the task
		 */
		private int deadline;

		/**
		 * Number of task hours that are already appointed to previous buckets
		 */
		private int alreadyDivided = 0;

		/**
		 * Specification of the task that is begin split into buckets
		 */
		private TaskSpecification specification;

		/**
		 * Constructor for OptimizerTask
		 * 
		 * @param spec
		 *            specification of the task that has to be split into
		 *            buckets
		 * @param t
		 *            the workload associated with the task that has to be split
		 *            into buckets
		 * @param dead
		 *            the deadline associated with the task that has to be split
		 *            into buckets
		 */
		public OptimizerTask(TaskSpecification spec, List<List<SubTask>> t,
				int dead) {
			task = t;
			deadline = dead;
			specification = spec;
		}

		/**
		 * @return the length of the task
		 */
		public int getTaskLength() {
			return task.size();
		}

		/**
		 * Get the part of the task between endVal and startVal, which means the
		 * next (endVal-startVal) task hours
		 * 
		 * @param startVal
		 *            the lower boundary of the interval of task hours to get
		 * @param endVal
		 *            the upper boundary of the interval of task hours to get
		 * @return the task hours between startVal and endVal
		 * @throws Exception
		 */
		public List<List<SubTask>> getPart(int startVal, int endVal)
				throws Exception {
			int interv = endVal - startVal;
			List<List<SubTask>> returnList = new ArrayList<List<SubTask>>();
			if (deadline == endVal) {
				while (alreadyDivided != task.size()) {
					returnList.add(task.get(alreadyDivided));
					alreadyDivided++;
				}
			} else {
				double percentage = ((double) interv / (double) deadline);
				double numberDouble = task.size() * percentage;
				int number = (int) Math.ceil(numberDouble);
				for (int i = 0; i < number; i++) {
					if ((returnList.size() < interv)
							&& (alreadyDivided < this.getTaskLength())) {
						returnList.add(task.get(alreadyDivided));
						alreadyDivided++;
					}
				}
			}
			return returnList;
		}

		/**
		 * Get an equally divided part of the task hours appointed to the
		 * interval with given number
		 * 
		 * @param i
		 *            the number of the bucket to get the appointed task hours
		 *            for
		 * @param countIntervals
		 *            the total number of buckets the task is divided in
		 * @return the task hours appointed to the asked for bucket
		 */
		public List<List<SubTask>> getEqualPart(int i, int countIntervals) {
			List<List<SubTask>> returnList = new ArrayList<List<SubTask>>();
			int sizeOfSmallSublists = task.size() / countIntervals;
			int sizeOfLargeSublists = sizeOfSmallSublists + 1;
			int numberOfLargeSublists = task.size() % countIntervals;
			int numberOfSmallSublists = countIntervals - numberOfLargeSublists;
			if (i < numberOfSmallSublists) {
				for (int j = 0; j < sizeOfSmallSublists; j++) {
					returnList.add(task.get(alreadyDivided));
					alreadyDivided++;
				}
			} else {
				for (int j = 0; j < sizeOfLargeSublists; j++) {
					returnList.add(task.get(alreadyDivided));
					alreadyDivided++;
				}
			}
			return returnList;
		}

		/**
		 * @return specification of the task that is being split into buckets
		 */
		public TaskSpecification getSpecification() {
			return specification;
		}

	}

	/**
	 * Threshold for the number of combination which is marked as feasible to
	 * handle while scheduling the task hours in a bucket, when the number of
	 * possible combinations is still higher, the task is splitted into twice as
	 * many buckets, ...
	 */
	private static BigInteger COMB_THRESHOLD = new BigInteger("1000000");

	/**
	 * List of deadlines of the tasks being scheduled
	 */
	private List<Integer> deadlines = new ArrayList<Integer>();

	/**
	 * List of the tasks being scheduled
	 */
	private List<OptimizerTask> tasks = new ArrayList<OptimizerTask>();

	/**
	 * Counter of the interval that is being scheduled
	 */
	private int inter = 0;

	/**
	 * Start time of the scheduling period
	 */
	private Date start;

	/**
	 * Specification of the instance for which we are creating a schedule
	 */
	private InstanceSpecification specification;

	/**
	 * Whether the spot-optimized scheduling option is being used
	 */
	private boolean useSpotOptimalization;

	/**
	 * Constructor for SuperOptimizer
	 * 
	 * @param startDate
	 *            start time of the scheduling period
	 * @param spec
	 *            specification of the instance for which a schedule is created
	 * @param use_spot_optimalization
	 *            whether the spot-optimized scheduling option is set
	 */
	public SuperOptimizer(Date startDate, InstanceSpecification spec,
			boolean use_spot_optimalization) {
		start = startDate;
		specification = spec;
		useSpotOptimalization = use_spot_optimalization;
	}

	@Override
	public void add(WorkloadWM2 wl) {
		int hoursAvail = wl.getHoursTimeFrame();
		if (!deadlines.contains(hoursAvail)) {
			deadlines.add(hoursAvail);
		}
		tasks.add(new OptimizerTask(wl.getTaskSpecification(),
				wl.getWorkload(), hoursAvail));
		if (hoursAvail > inter) {
			inter = hoursAvail;
		}
	}

	@Override
	public List<ScheduledTaskCollection> determineOptimal() {
		System.out.println("DetermineOptimal: " + inter + " [begin]");
		ArrayList<ScheduledTaskCollection> returnList = new ArrayList<ScheduledTaskCollection>();
		Collections.sort(deadlines);
		int nrOfIntervals = deadlines.size();
		if (nrOfIntervals == 1) {
			int countIntervals = this.getDesiredNumberOfIntervals();
			int sizeOfSmallSublists = this.getIntervalLength() / countIntervals;
			int sizeOfLargeSublists = sizeOfSmallSublists + 1;
			int numberOfLargeSublists = this.getIntervalLength()
					% countIntervals;
			int numberOfSmallSublists = countIntervals - numberOfLargeSublists;
			for (int i = 0; i < countIntervals; i++) {
				System.out.println("Optimizer " + i + " of " + countIntervals);
				int interv = sizeOfLargeSublists;
				if (i < numberOfSmallSublists) {
					interv = sizeOfSmallSublists;
				}
				Optimizer<WorkloadWM2> opt = new RealOptimizer(
						useSpotOptimalization);
				for (OptimizerTask task : tasks) {
					try {
						List<List<SubTask>> part = task.getEqualPart(i,
								countIntervals);
						if (!part.isEmpty()) {
							opt.add(new WorkloadWM2(task.getSpecification(),
									part, interv));
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
				List<ScheduledTaskCollection> optimalPart = opt
						.determineOptimal();
				returnList = this.mergeResults(returnList, optimalPart);
			}
		} else {
			int startVal = 0;
			for (int i = 0; i < nrOfIntervals; i++) {
				int endVal = deadlines.get(i);
				SuperOptimizer opt = new SuperOptimizer(start, specification,
						useSpotOptimalization);
				for (OptimizerTask task : tasks) {
					try {
						opt.add(new WorkloadWM2(task.getSpecification(), task
								.getPart(startVal, endVal), (endVal - startVal)));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
				List<ScheduledTaskCollection> optimalPart = opt
						.determineOptimal();
				returnList = this.mergeResults(returnList, optimalPart);
				startVal = endVal;
			}
		}
		System.out.println("DetermineOptimal: " + inter + " [end]");
		return returnList;
	}

	/**
	 * Merge two lists with schedules
	 * 
	 * @param origList
	 *            the first list
	 * @param optimalPart
	 *            the second list
	 * @return the merged list of schedules
	 */
	private ArrayList<ScheduledTaskCollection> mergeResults(
			List<ScheduledTaskCollection> origList,
			List<ScheduledTaskCollection> optimalPart) {
		ArrayList<ScheduledTaskCollection> returnList = new ArrayList<ScheduledTaskCollection>();
		int preFillAmount = 0;
		int postFillAmount = 0;
		for (int i = 0; i < origList.size(); i++) {
			ScheduledTaskCollection origPart = origList.get(i);
			preFillAmount = origPart.getNumberOfSubTasks();
			if (optimalPart.size() > i) {
				ScheduledTaskCollection newPart = optimalPart.get(i);
				postFillAmount = newPart.getNumberOfSubTasks();
				origPart.merge(newPart);
			} else {
				origPart.postFill(postFillAmount);
			}
			returnList.add(origPart);
		}
		for (int i = origList.size(); i < optimalPart.size(); i++) {
			ScheduledTaskCollection partI = optimalPart.get(i);
			partI.preFill(preFillAmount);
			returnList.add(partI);
		}
		return returnList;
	}

	/**
	 * @return the desired number of intervals to split the tasks in, in order
	 *         to fall under threshold of the number of possible combinations
	 */
	private int getDesiredNumberOfIntervals() {
		List<Integer> intervalPossibilities = new ArrayList<Integer>();
		for (OptimizerTask task : tasks) {
			intervalPossibilities.add(this.getDesiredNumberOfIntervals(task
					.getTaskLength()));
		}
		Collections.sort(intervalPossibilities);
		return intervalPossibilities.get(intervalPossibilities.size() - 1);
	}

	/**
	 * Determine the number of buckets needed for a given task in order to fall
	 * under the threshold of the number of possible combinations
	 * 
	 * @param taskLength
	 *            the length of the task to determine the number of buckets for
	 * @return the number of buckets needed
	 */
	private int getDesiredNumberOfIntervals(int taskLength) {
		int countIntervals = 1;
		if (taskLength < this.getIntervalLength()) {
			CombinationGenerator comb = new CombinationGenerator(
					this.getIntervalLength(), taskLength);
			while ((new BigInteger(comb.getNumLeft().toString()))
					.compareTo(SuperOptimizer.COMB_THRESHOLD) > 0) {
				comb = new CombinationGenerator(this.getIntervalLength()
						/ countIntervals, taskLength / countIntervals);
				countIntervals *= 2;
			}
		}
		return countIntervals;
	}

	/**
	 * @return the length of a single interval/bucket in hours
	 */
	public int getIntervalLength() {
		return inter;
	}

}
