package be.ac.ua.broker.utils;

import java.math.BigInteger;

/**
 * This class is used to systematically generate the next possible combination
 * when it is needed, this utility class is needed because it is impossible to
 * store a very large number of combinations in memory at once.
 * 
 * @author Vermeersch Kurt (based on http://www.merriampark.com/comb.htm
 *         implementation)
 * @date May 2011
 */
public class CombinationGenerator {

	/**
	 * Whether debug output is printed during the generation of the
	 * combinations.
	 */
	private static boolean VERBOSE = false;

	/**
	 * The current combination.
	 */
	private int[] a;

	/**
	 * The number of possible integers.
	 */
	private int n;

	/**
	 * The size of one combination.
	 */
	private int r;

	/**
	 * The number of combinations that still need to be generated.
	 */
	private BigInteger numLeft;

	/**
	 * The total number of combinations that need to be generated.
	 */
	private BigInteger total;

	/**
	 * Constructor for CombinationGenerator
	 * 
	 * @param n
	 *            number of possible integers
	 * @param r
	 *            size of one combination
	 */
	public CombinationGenerator(int n, int r) {
		this.n = n;
		this.r = r;
		if (VERBOSE) {
			System.out.println("n=" + n);
			System.out.println("r=" + r);
		}
		if (r > n) {
			throw new IllegalArgumentException();
		}
		if (n < 1) {
			throw new IllegalArgumentException();
		}
		a = new int[r];
		BigInteger nFact = getFactorial(n);
		BigInteger rFact = getFactorial(r);
		BigInteger nminusrFact = getFactorial(n - r);
		total = nFact.divide(rFact.multiply(nminusrFact));
		if (VERBOSE) {
			System.out.println("after divide total=" + total);
		}
		reset();
	}

	/**
	 * Reset the combination generator, start iterating again from the first
	 * combination possible.
	 */
	public void reset() {
		for (int i = 0; i < a.length; i++) {
			a[i] = i;
		}
		numLeft = new BigInteger(total.toString());
	}

	/**
	 * @return the number of combinations we still need to iterate over
	 */
	public BigInteger getNumLeft() {
		return numLeft;
	}

	/**
	 * @return whether there are more combinations or we iterated over all of
	 *         them already
	 */
	public boolean hasMore() {
		return numLeft.compareTo(BigInteger.ZERO) == 1;
	}

	/**
	 * @return the total number of combinations
	 */
	public BigInteger getTotal() {
		return total;
	}

	/**
	 * Compute the factorial of the given integer
	 * 
	 * @param n
	 *            the integer to calculate the factorial of
	 * @return the calculated factorial
	 */
	private static BigInteger getFactorial(int n) {
		BigInteger fact = BigInteger.ONE;
		for (int i = n; i > 1; i--) {
			fact = fact.multiply(new BigInteger(Integer.toString(i)));
		}
		return fact;
	}

	/**
	 * Generate next combination (algorithm from Rosen p. 286)
	 * 
	 * @return the next combination
	 */
	public int[] getNext() {

		if (numLeft.equals(total)) {
			numLeft = numLeft.subtract(BigInteger.ONE);
			return a;
		}

		int i = r - 1;
		while (a[i] == n - r + i) {
			i--;
		}
		a[i] = a[i] + 1;
		for (int j = i + 1; j < r; j++) {
			a[j] = a[i] + j - i;
		}

		numLeft = numLeft.subtract(BigInteger.ONE);
		return a;

	}

	/**
	 * @return the current combination
	 */
	public int[] getCurrent() {
		return a;
	}

}