package be.ac.ua.broker.utils;

import java.text.DateFormat;
import java.text.FieldPosition;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * This class contains some date utilities needed in the application.
 * 
 * @author Vermeersch Kurt
 * @date October 2010
 */
public class DateUtility {

	/**
	 * Date formatter that uses the format <year>-<month>-<day>.
	 */
	private static DateFormat formatter = new SimpleDateFormat(
			"yyyy-MM-dd HH:mm");

	/**
	 * Representation of an hour in milliseconds
	 */
	public static long HOURINMILLISECONDS = 60 * 60 * 1000;

	/**
	 * This class represents a DateFormatter that returns the week number of the
	 * date to format.
	 * 
	 * @author Vermeersch Kurt
	 * @date October 2010
	 */
	public static class WeekDateFormat extends DateFormat {

		/** Serial Version ID */
		private static final long serialVersionUID = 1L;

		@Override
		public StringBuffer format(Date date, StringBuffer toAppendTo,
				FieldPosition fieldPosition) {
			int week = DateUtility.getWeekOfTheYear(date);
			if (DateUtility.earliestDateOfTheWeek(date).equals(date)) {
				return toAppendTo.append("week " + week);
			}
			return toAppendTo;
		}

		@Override
		public Date parse(String source, ParsePosition pos) {
			return null;
		}

	}

	/**
	 * Getter for the week number in the year in which the given date has place
	 * 
	 * @param date
	 *            the date to return the week number for
	 * @return the number of the week in the corresponding year
	 */
	public static int getWeekOfTheYear(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * Getter for the day of the given date
	 * 
	 * @param d
	 *            the date to return the day of
	 * @return the day of the given date
	 */
	public static String getDayofTheDate(Date d) {
		String day = null;
		DateFormat f = new SimpleDateFormat("EEEE");
		try {
			day = f.format(d);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return day;
	}

	/**
	 * Convert the given string representation of a date to the actual date
	 * object using the date formatter
	 * 
	 * @param date
	 *            the string representation of the date
	 * @return the actual date object for the given string representation of the
	 *         date
	 */
	public static Date toDate(String date) {
		Date returnVal = null;
		try {
			returnVal = formatter.parse(date);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return returnVal;
	}

	/**
	 * Convert the given date to a string that represents the time of the date
	 * 
	 * @param date
	 *            the date to return the string representation of the time for
	 * @return the string representation of the time of the given date
	 */
	public static String toTime(Date date) {
		SimpleDateFormat formatterTime = new SimpleDateFormat("hh:mm:ss a");
		return formatterTime.format(date);
	}

	/**
	 * Return the earliest date that lies in the same week as the given date
	 * 
	 * @param dateObj
	 *            the date for which we have to determine the earliest date that
	 *            lies in the same week as this date
	 * @return the earliest date that lies in the same week as the week that the
	 *         given date lies in
	 */
	public static Date earliestDateOfTheWeek(Date dateObj) {
		int week = DateUtility.getWeekOfTheYear(dateObj);
		Date prevDate = null;
		Calendar cal = Calendar.getInstance();
		while (week == DateUtility.getWeekOfTheYear(dateObj)) {
			prevDate = dateObj;
			cal.setTime(dateObj);
			cal.add(Calendar.DAY_OF_YEAR, -1);
			dateObj = cal.getTime();
		}
		return prevDate;
	}

	/**
	 * Getter for the number of hours between a given start and end date
	 * 
	 * @param startDate
	 *            the lower boundary of the time interval
	 * @param endDate
	 *            the upper boundary of the time interval
	 * @return the number of hours between startDate and endDates
	 */
	public static int getNumberOfHoursBetween(Date startDate, Date endDate) {
		long differenceMs = endDate.getTime() - startDate.getTime();
		int numberOfHoursBetween = (int) (differenceMs / (60 * 60 * 1000));
		return numberOfHoursBetween;
	}

	/**
	 * Getter for the double representation of the number of hours between a
	 * given start and end date
	 * 
	 * @param startDate
	 *            the lower boundary of the time interval
	 * @param endDate
	 *            the upper boundary of the time interval
	 * @return the double representation of the number of hours between
	 *         startDate and endDates
	 */
	public static double getDoubleNumberOfHoursBetween(Date startDate,
			Date endDate) {
		long differenceMs = endDate.getTime() - startDate.getTime();
		double numberOfHoursBetween = ((double) differenceMs / (double) (60 * 60 * 1000));
		return numberOfHoursBetween;
	}

	/**
	 * Getter for the number of minutes between a given start and end date
	 * 
	 * @param startDate
	 *            the lower boundary of the time interval
	 * @param endDate
	 *            the upper boundary of the time interval
	 * @return the number of minutes between startDate and endDates
	 */
	public static int getNumberOfMinutesBetween(final Date startDate,
			final Date endDate) {
		long differenceMs = endDate.getTime() - startDate.getTime();
		int numberOfMinutesBetween = (int) (differenceMs / (60 * 1000));
		return numberOfMinutesBetween;
	}

	/**
	 * Transform a date object to the string representation.
	 * 
	 * @param date
	 *            the date to transform
	 * @return the formatted string representation of the given date
	 */
	public static String dateToString(final Date date) {
		return formatter.format(date);
	}

}
