package be.ac.ua.broker.utils;

import java.rmi.activation.UnknownObjectException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

/**
 * This class represents an execution timer, which can be used to time the
 * running of different parts of a Java program. The parts are identified by a
 * uniquely given string key. The ExecutionTimer is used in the benchmarking of
 * the broker.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class ExecutionTimer {

	/**
	 * Mapping of a timer name to a timer value, the time when the timer started
	 * running.
	 */
	private Map<String, Long> timerMapping = new HashMap<String, Long>();

	/**
	 * Mapping of a timer name to a boolean representing whether the
	 * corresponding timer was stopped already.
	 */
	private Map<String, Boolean> timerStoppedMapping = new HashMap<String, Boolean>();

	/**
	 * Start a timer with a given name
	 * 
	 * @param timerName
	 *            name/key of the timer to start running
	 */
	public void startTiming(String timerName) {
		// Get current time
		long start = System.currentTimeMillis();
		timerMapping.put(timerName, start);
		timerStoppedMapping.put(timerName,false);
	}

	/**
	 * Stop the timer with the given name/key
	 * 
	 * @param timerName
	 *            name/key of the timer to stop running
	 * @throws UnknownObjectException
	 *             exception thrown when no timer was started previously with
	 *             the given name/key value
	 */
	public void stopTiming(String timerName) throws UnknownObjectException {
		// Get current time
		Long stop = System.currentTimeMillis();
		Long start = timerMapping.get(timerName);
		if (start == null) {
			throw new UnknownObjectException("timer was not started!");
		}
		timerMapping.put(timerName, stop - start);
		timerStoppedMapping.put(timerName,true);
	}

	/**
	 * Clear all the timing information from the timer.
	 */
	public void clear() {
		timerMapping.clear();
		timerStoppedMapping.clear();
	}

	/**
	 * Get the elapsed time by the timer with the given name/key value.
	 * 
	 * @param timerName
	 *            name/key of the timer to get the elapsed time of
	 * @return the elapsed time according to the requested timer.
	 * @throws UnknownObjectException
	 *             exception thrown when timer was not yet started/stopped
	 */
	public long getElapsedTime(String timerName) throws UnknownObjectException {
		Long elapsed = timerMapping.get(timerName);
		if (elapsed == null) {
			throw new UnknownObjectException("timer was not started!");
		} else if(!timerStoppedMapping.get(timerName)){
			throw new UnknownObjectException("timer was not stopped!");
		}
		return elapsed;
	}

	/**
	 * Get the total elapsed time by all the different timings. This value is
	 * only valuable when the different timers, using different names/key
	 * values, are timing consecutive events (they do not overlap) in time.
	 * 
	 * @return the total elapsed time.
	 * @throws UnknownObjectException
	 *             exception thrown when timer was not yet started/stopped
	 */
	public long getTotalElapsedTime() throws UnknownObjectException {
		Long totalTime = new Long(0);
		Set<Entry<String, Long>> entries = timerMapping.entrySet();
		for (Entry<String, Long> entry : entries) {
			if(!timerStoppedMapping.get(entry.getKey())){
				throw new UnknownObjectException("timer "+entry.getKey()+" was not stopped!");
			}
			totalTime += entry.getValue();
		}
		return totalTime;
	}

	/**
	 * Get the elapsed time in seconds by the timer with the given name/key
	 * value.
	 * 
	 * @param timerName
	 *            name/key of the timer to get the elapsed time of
	 * @return the elapsed time in seconds according to the requested timer.
	 * @throws UnknownObjectException
	 *             exception thrown when timer was not yet started/stopped
	 */
	public float getElapsedTimeInSeconds(String timerName)
			throws UnknownObjectException {
		return this.getElapsedTime(timerName) / 1000F;
	}

	/**
	 * Get the total elapsed time in seconds by all the different timings. This
	 * value is only valuable when the different timers, using different
	 * names/key values, are timing consecutive events (they do not overlap) in
	 * time.
	 * 
	 * @return the total elapsed time in seconds.
	 * @throws UnknownObjectException
	 *             exception thrown when timer was not yet started/stopped
	 */
	public float getTotalElapsedTimeInSeconds() throws UnknownObjectException {
		return this.getTotalElapsedTime() / 1000F;
	}

}
