package be.ac.ua.broker.utils;

import java.util.HashSet;
import java.util.Set;

import be.ac.ua.broker.allocation.AllocatorOptions;
import be.ac.ua.broker.input.pricing.EC2Specifications;
import be.ac.ua.broker.scheduling.workload.SchedulerOptions;

/**
 * This class contains some broker input verification utilities. It provides
 * static methods to verify whether an existing EC2 instance, region or
 * operating system is provided. It also enables the user to check whether an
 * existing scheduling and allocation scheme option is provided to the broker.
 * Make sure to call the setup function before using the utilities of this
 * class.
 * 
 * @author Vermeersch Kurt
 * @date May 2011
 */
public class InputVerifier {

	/** The set of instance types that are supported by our broker application */
	private static Set<String> instances = null;

	/**
	 * The set of geographical regions that are supported by our broker
	 * application
	 */
	private static Set<String> regions = null;

	/**
	 * The set of operating systems that are supported by our broker application
	 */
	private static Set<String> oses = null;

	/**
	 * The set of scheduling options that are supported by our broker
	 * application
	 */
	private static Set<String> schedulingVersions = null;

	/**
	 * The set of allocation options that are supported by our broker
	 * application
	 */
	private static Set<String> allocationVersions = null;

	/**
	 * Fill the sets with the values representing the supported items
	 */
	public static void setup() {
		instances = EC2Specifications.getInstanceTypeMapping().keySet();
		regions = EC2Specifications.getRegionMapping().keySet();
		oses = EC2Specifications.getOSMapping().keySet();
		schedulingVersions = new HashSet<String>();
		schedulingVersions.add(SchedulerOptions.BASIC);
		schedulingVersions.add(SchedulerOptions.OPTIMIZED);
		schedulingVersions.add(SchedulerOptions.SPOTOPTIMIZED);
		allocationVersions = new HashSet<String>();
		allocationVersions.add(AllocatorOptions.ALLONDEMAND);
		allocationVersions.add(AllocatorOptions.NOSPOT);
		allocationVersions.add(AllocatorOptions.SPOT);
		allocationVersions.add(AllocatorOptions.SPOTOPTIMAL);
	}

	/**
	 * Check whether a given instance type is supported by the broker
	 * 
	 * @param inst
	 *            the instance type to check
	 * @return whether the instance type is supported by the broker
	 * @throws InvalidInputException
	 *             exception thrown when the provided instance type is not
	 *             supported by the broker
	 */
	public static boolean instanceExists(final String inst)
			throws InvalidInputException {
		InputVerifier.checkInitialization();
		boolean returnValue = instances.contains(inst);
		if (!returnValue) {
			throw new InvalidInputException(
					"Non existing instance type provided as input (" + inst
							+ ")");
		}
		return returnValue;
	}

	/**
	 * Check whether a given geographical region is supported by the broker
	 * 
	 * @param reg
	 *            the geographical region to check
	 * @return whether the geographical region is supported by the broker
	 * @throws InvalidInputException
	 *             exception thrown when the provided geographical region is not
	 *             supported by the broker
	 */
	public static boolean regionExists(final String reg) throws InvalidInputException {
		InputVerifier.checkInitialization();
		boolean returnValue = regions.contains(reg);
		if (!returnValue) {
			throw new InvalidInputException(
					"Non existing region provided as input (" + reg + ")");
		}
		return returnValue;
	}

	/**
	 * Check whether a given operating system is supported by the broker
	 * 
	 * @param os
	 *            the operating system to check
	 * @return whether the operating system is supported by the broker
	 * @throws InvalidInputException
	 *             exception thrown when the provided operating system is not
	 *             supported by the broker
	 */
	public static boolean osExists(final String os) throws InvalidInputException {
		InputVerifier.checkInitialization();
		boolean returnValue = oses.contains(os);
		if (!returnValue) {
			throw new InvalidInputException(
					"Non existing os provided as input (" + os + ")");
		}
		return returnValue;
	}

	/**
	 * Check whether the InputVerifier setup method was already called,
	 * if this is not the case it gets executed.
	 */
	private static void checkInitialization() {
		if (oses == null || regions == null || instances == null) {
			InputVerifier.setup();
		}
	}

	/**
	 * Check whether a given scheduling option is supported by the broker
	 * 
	 * @param version
	 *            the scheduling option to check
	 * @return whether the scheduling option is supported by the broker
	 * @throws InvalidInputException
	 *             exception thrown when the provided scheduling option is not
	 *             supported by the broker
	 */
	public static boolean schedulingVersionExists(final String version)
			throws InvalidInputException {
		InputVerifier.checkInitialization();
		boolean returnValue = schedulingVersions.contains(version);
		if (!returnValue) {
			throw new InvalidInputException(
					"Non existing scheduling version provided as input ("
							+ version + ")");
		}
		return returnValue;
	}

	/**
	 * Getter for the set of supported scheduling options of the broker
	 * 
	 * @return the set of supported scheduling options
	 */
	public static Set<String> getSchedulingVersions() {
		InputVerifier.checkInitialization();
		return schedulingVersions;
	}

	/**
	 * Check whether a given allocation option is supported by the broker
	 * 
	 * @param version
	 *            the allocation option to check
	 * @return whether the allocation option is supported by the broker
	 * @throws InvalidInputException
	 *             exception thrown when the provided allocation option is not
	 *             supported by the broker
	 */
	public static boolean allocationVersionExists(final String version)
			throws InvalidInputException {
		InputVerifier.checkInitialization();
		boolean returnValue = allocationVersions.contains(version);
		if (!returnValue) {
			throw new InvalidInputException(
					"Non existing allocation version provided as input ("
							+ version + ")");
		}
		return returnValue;
	}

	/**
	 * Getter for the set of supported allocation options of the broker
	 * 
	 * @return the set of supported allocation options
	 */
	public static Set<String> getAllocationsVersions() {
		InputVerifier.checkInitialization();
		return allocationVersions;
	}

}
